// Copyright (c) 2001  David Muse
// See the file COPYING for more information.

public class SQLRConnection {

	/** 
	 *  Initiates a connection to "server" on "port"
	 *  or to the unix "socket" on the local machine
	 *  and authenticates with "user" and "password".
	 *  Failed connections will be retried for 
	 *  "tries" times on interval "retrytime"
	 *  or on for a default number of times on
	 *  a default interval if left unspecified.
	 *
	 *  If the "socket" parameter is neither 
	 *  NULL nor "" then an attempt will be made to 
	 *  connect through it before attempting to 
	 *  connect to "server" on "port".  If it is 
	 *  NULL or "" then no attempt will be made to 
	 *  connect through the socket.  */
	public SQLRConnection(String server, int port, String socket,
						String user, String password,
						int retrytime, int tries) {
		connection=sqlrcon_alloc(server,port,socket,
						user,password,retrytime,tries);
	}
	/** Disconnects and ends the session if
	 * it hasn't been ended already.  */
	public native void	delete();


	/** Ends the session. */
	public native int	endSession();

	/** Disconnects this connection from the current
	 *  session but leaves the session open so 
	 *  that another connection can connect to it 
	 *  using resumeSession().  */
	public native int	suspendSession();

	/** Returns the inet port that the connection is 
	 *  communicating over. This parameter may be 
	 *  passed to another connection for use in
	 *  the resumeSession() method. */
	public native int	getConnectionPort();

	/** Returns the unix socket that the connection 
	 *  is communicating over. This parameter may be 
	 *  passed to another connection for use in
	 *  the resumeSession() method. */
	public native String	getConnectionSocket();

	/** Resumes a session previously left open 
	 *  using suspendSession().
	 *  Returns 1 on success and 0 on failure. */
	public native int	resumeSession(int port, String socket);


	/** Returns 1 if the database is up and 0
	 *  if it's down.  */
	public native int	ping();
	/** Returns the type of database: 
	 *    oracle7, oracle8, postgresql, mysql, etc.  */
	public native String	identify();


	/** Instructs the database to perform a commit
	 *  after every successful query. */
	public native int	autoCommitOn();
	/** Instructs the database to wait for the 
	 *  client to tell it when to commit. */
	public native int	autoCommitOff();
	/** Issues a commit. Returns 1 if the commit succeeded, 0 if it
	 *  failed and -1 if an error occurred. */
	public native int	commit();
	/** Issues a rollback. Returns 1 if the rollback succeeded, 0 if it
	 *  failed and -1 if an error occurred. */
	public native int	rollback();


	/** Causes verbose debugging information to be 
	 *  sent to standard output.  Another way to do
	 *  this is to start a query with "-- debug\n". */
	public native void	debugOn();

	/** Turns debugging off. */
	public native void	debugOff();

	/** Returns 0 if debugging is off and 1 if 
	 *  debugging is on. */
	public native int	getDebug();


	/** connection is used internally, it's just
	 *  public to make the JNI wrapper work faster.  */
	public int	connection;
	private native int	sqlrcon_alloc(String server, int port, 
						String socket, String user, 
						String password, 
						int retrytime, int tries);
	static {
		System.loadLibrary("SQLRConnection");
	}
}
