// Copyright (c) 1999-2001  David Muse
// See the file COPYING for more information

#include <config.h>
#include <defaults.h>

#include <sqlrelay/sqlrclient.h>
#include <rudiments/commandline.h>
#if defined(HAVE_LIBXML) || defined(HAVE_GNOME_XML)
	#include <configfile.h>
#endif
#include <iostream.h>

#include <stdlib.h>
#include <string.h>

int	main(int argc, char **argv) {

	#include <version.h>

	#if defined(HAVE_LIBXML) || defined(HAVE_GNOME_XML)
		configfile	*cfgfile=NULL;
		usernode	*currentnode=NULL;
	#endif

	commandline	*cmdline=new commandline(argc,argv);
	char		*host;
	int		port;
	char		*socket;
	char		*user;
	char		*password;
	char		*query="";
	int		debug=0;

#if defined(HAVE_LIBXML) || defined(HAVE_GNOME_XML)
	char	*config=cmdline->value("-config");
	if (!(config && config[0])) {
		config=DEFAULT_CONFIG_FILE;
	}
	char	*id=cmdline->value("-id");
	if (!(id && id[0])) {
#endif


		if (argc<7) {
			cout << "usage: query  host port socket ";
			cout << "user password query [debug]" << ENDL;
			#if defined(HAVE_LIBXML) || defined(HAVE_GNOME_XML)
				cout << "  or   query  [-config configfile] ";
				cout << "-id id query [debug]" << ENDL;
			#endif
			exit(0);
		}

		host=argv[1];
		port=atoi(argv[2]);
		socket=argv[3];
		user=argv[4];
		password=argv[5];
		query=argv[6];
		if (argv[7] && !strcmp(argv[7],"debug")) {
			debug=1;
		}

#if defined(HAVE_LIBXML) || defined(HAVE_GNOME_XML)
	} else {

		cfgfile=new configfile();
		if (cfgfile->parse(config,id)) {

			// get the host/port/socket/username/password
			host="localhost";
			port=cfgfile->getPort();
			socket=cfgfile->getUnixPort();
			currentnode=cfgfile->getUsers();
			user=currentnode->getUser();
			password=currentnode->getPassword();

			// find the query and optional debug
			if (cmdline->found("debug")) {
				debug=1;
			}

			// find the query
			for (int i=1; i<argc; i++) {
				if (argv[i][0]=='-') {
					i++;
					continue;
				}
				if (!strcmp(argv[i],"debug")) {
					continue;
				}
				query=argv[i];
				break;
			}
		} else {
			delete cfgfile;
			delete cmdline;
			exit(0);
		}
	}
#endif



	sqlrconnection	*sqlrcon=new sqlrconnection(host,port,
						socket,user,password,0,1);
	if (debug) {
		sqlrcon->debugOn();
	}

	sqlrcursor	*sqlrcur=new sqlrcursor(sqlrcon);
	sqlrcur->dontGetColumnInfo();
	sqlrcur->sendQuery(query);
	sqlrcon->endSession();

	for (int i=0; i<sqlrcur->rowCount(); i++) {
		for (int j=0; j<sqlrcur->colCount(); j++) {
			cout << "\"" << sqlrcur->getField(i,j) << "\"";
			if (j<sqlrcur->colCount()-1) {
				cout << ",";
			} else {
				cout << ENDL;
			}
		}
	}

#if defined(HAVE_LIBXML) || defined(HAVE_GNOME_XML)
	if (cfgfile) {
		delete cfgfile;
	}
#endif
	delete cmdline;
	delete sqlrcur;
	delete sqlrcon;
}
