// Copyright (c) 1999-2001  David Muse
// See the file COPYING for more information

#include <config.h>
#include <defaults.h>

#include <sqlrelay/sqlrclient.h>
#include <rudiments/commandline.h>
#if defined(HAVE_LIBXML) || defined(HAVE_GNOME_XML)
	#include <configfile.h>
#endif
#include <iostream.h>
#include <iomanip.h>
#include <fstream.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#ifdef HAVE_STRINGS_H
	#include <strings.h>
#endif
#include <time.h>


#ifdef HAVE_READLINE
	#include <rudiments/text.h>
	// This is an interesting story...
	// readline 2's include files don't list any parameters for any of
	// the functions.  This is fine for C, but not C++, at least not with
	// the compiler I'm using, even with the extern "C" {} bit.
	// This is fixed in readline 4, but, to maintain compatibility with 
	// readline 2, I define the functions myself.
	extern "C" {
		extern char *readline(char *prompt);
		extern void add_history(char *line);
		extern void read_history(char *file);
		extern void write_history(char *file);
		extern void history_truncate_file(char *file, int line);
	}
#endif

class	environment {
	public:
			environment();
		int	color;
		int	headers;
		int	stats;
		int	debug;
		int	final;
};

environment::environment() {
	color=0;
	headers=1;
	stats=1;
	debug=0;
	final=0;
}

class	sqlrsh {
	public:
		void	execute(int argc, char **argv);
	private:
		void	startupMessage(environment *env,
					char *host, int port, char *user);
		void	systemRcFile(sqlrconnection *sqlrcon, 
					sqlrcursor *sqlrcur, 
					environment *env);
		void	userRcFile(sqlrconnection *sqlrcon, 
					sqlrcursor *sqlrcur, 
					environment *env);
		void	runScript(sqlrconnection *sqlrcon,
					sqlrcursor *sqlrcur, environment *env, 
					char *file, int returnerror);
		int	getCommandFromFile(ifstream *file, 
					strstream *cmdbuffer);
		int	runCommand(sqlrconnection *sqlrcon, 
					sqlrcursor *sqlrcur, 
					environment *env, char *command);
		int	commandType(char *command);
		void	internalCommand(environment *env, char *command);
		void	externalCommand(sqlrconnection *sqlrcon,
					sqlrcursor *sqlrcur, environment *env, 
					char *command);
		void	initStats(environment *env);
		void	displayError(sqlrcursor *sqlrcur, environment *env);
		void	displayHeader(sqlrcursor *sqlrcur, environment *env);
		void	displayResultSet(sqlrcursor *sqlrcur, environment *env);
		void	displayStats(sqlrcursor *sqlrcur, environment *env);
		void	displayHelp(environment *env);
		void	interactWithUser(sqlrconnection *sqlrcon,
					sqlrcursor *sqlrcur, environment *env);
		void	prompt(int promptcount);
		void	error(char *errstring);
		void	addLineToCommand(strstream *command, strstream *line,
						int final);

		void	setColor(environment *env, int value);
		void	black(environment *env);
		void	red(environment *env);
		void	green(environment *env);
		void	yellow(environment *env);
		void	blue(environment *env);
		void	magenta(environment *env);
		void	cyan(environment *env);
		void	white(environment *env);
};

void	sqlrsh::systemRcFile(sqlrconnection *sqlrcon, sqlrcursor *sqlrcur, 
						environment *env) {

	// build file name
	char	*systemrcfile=new char[strlen(SYSTEM_SQLRSHRC)+1];
	sprintf(systemrcfile,"%s",SYSTEM_SQLRSHRC);

	// process the file
	runScript(sqlrcon,sqlrcur,env,systemrcfile,0);

	// clean up
	delete[] systemrcfile;
}

void	sqlrsh::userRcFile(sqlrconnection *sqlrcon, sqlrcursor *sqlrcur, 
						environment *env) {

	// get user's home directory
	char	*home=home=getenv("HOME");
	if (!home) {
		home="~";
	}

	// build rcfilename
	char	*userrcfile=new char[strlen(home)+10+1];
	sprintf(userrcfile,"%s/.sqlrshrc",home);

	// process the file
	runScript(sqlrcon,sqlrcur,env,userrcfile,0);

	// clean up
	delete[] userrcfile;
}

void	sqlrsh::runScript(sqlrconnection *sqlrcon, sqlrcursor *sqlrcur, 
			environment *env, char *file, int returnerror) {

	// open the file
	ifstream	*scriptfile=new ifstream(file);
	if (scriptfile && scriptfile->good()) {

		while (1) {
		
			// get a command
			strstream	*command=new strstream();
			if (!getCommandFromFile(scriptfile,command)) {
				break;
			}

			// run the command
			runCommand(sqlrcon,sqlrcur,env,command->str());

			// clean up
			delete command;
		}

		// close the file
		scriptfile->close();
	} else {

		// error message
		if (returnerror) {
			char	*errmesg=new char[20+strlen(file)+1];
			sprintf(errmesg,"Couldn't open file: %s",file);
			error(errmesg);
			delete[] errmesg;
		}
	}

	// clean up
	if (scriptfile) {
		delete scriptfile;
	}
}

int	sqlrsh::getCommandFromFile(ifstream *file, strstream *cmdbuffer) {

	char	character;
	
	while (1) {

		// get a character from the file
		character=file->rdbuf()->sgetc();

		// look for end of file
		if (character==-1) {
			*cmdbuffer << ENDS;
			return 0;
		}

		// look for an escape character
		if (character=='\\') {
			file->rdbuf()->snextc();
			character=file->rdbuf()->sgetc();
			*cmdbuffer << character;
			file->rdbuf()->snextc();
		}

		// look for an end of command delimiter
		if (character==';') {
			*cmdbuffer << ENDS;
			file->rdbuf()->snextc();
			return 1;
		}

		// write character to buffer and move on
		*cmdbuffer << character;
		file->rdbuf()->snextc();
	}
		
}

int	sqlrsh::runCommand(sqlrconnection *sqlrcon, sqlrcursor *sqlrcur, 
					environment *env, char *command) {

	int	cmdtype=commandType(command);

	if (cmdtype>0) {
		// if the command an internal command, run it as one
		internalCommand(env,command);
		return 1;
	} else if (cmdtype==0) {
		// if the command is not an internal command, 
		// execute it as a query and display the result set
		externalCommand(sqlrcon,sqlrcur,env,command);
		return 1;
	} else {
		return 0;
	}
}

int	sqlrsh::commandType(char *command) {

	// skip white space
	char	*ptr=command;
	while (*ptr==' ' || *ptr=='	' || *ptr=='\n') {
		*ptr++;
	}

	// compare to known internal commands
	if (!strncasecmp(ptr,"color",5) ||
		!strncasecmp(ptr,"headers",7) ||
		!strncasecmp(ptr,"stats",5) ||
		!strncasecmp(ptr,"debug",5) ||
		!strncasecmp(ptr,"final",5) ||
		!strncasecmp(ptr,"help",4)) {

		// return value of 1 is internal command
		return 1;
	}

	// look for an exit command
	if (!strncasecmp(ptr,"quit",4) ||
		!strncasecmp(ptr,"exit",4)) {
		return -1;
	}

	// return value of 0 is external command
	return 0;
}

void	sqlrsh::internalCommand(environment *env, char *command) {

	// skip white space
	char	*ptr=command;
	while (*ptr==' ' || *ptr=='	' || *ptr=='\n') {
		*ptr++;
	}

	// compare to known internal commands
	int	cmdtype=0;
	if (!strncasecmp(ptr,"color",5)) {
		ptr=ptr+5;
		cmdtype=1;
	} else if (!strncasecmp(ptr,"headers",7)) {
		ptr=ptr+7;
		cmdtype=2;
	} else if (!strncasecmp(ptr,"stats",5)) {	
		ptr=ptr+5;
		cmdtype=3;
	} else if (!strncasecmp(ptr,"debug",5)) {	
		ptr=ptr+5;
		cmdtype=4;
	} else if (!strncasecmp(ptr,"final",5)) {	
		ptr=ptr+5;
		cmdtype=5;
	} else if (!strncasecmp(ptr,"help",4)) {	
		displayHelp(env);
		return;
	} else {
		return;
	}

	// skip white space
	while (*ptr==' ' || *ptr=='	' || *ptr=='\n') {
		*ptr++;
	}

	// on or off?
	int	toggle;
	if (!strncasecmp(ptr,"on",2)) {
		toggle=1;
	} else if (!strncasecmp(ptr,"off",3)) {
		toggle=0;
	} else {
		return;
	}

	// set parameter
	if (cmdtype==1) {
		env->color=toggle;
	} else if (cmdtype==2) {
		env->headers=toggle;
	} else if (cmdtype==3) {
		env->stats=toggle;
	} else if (cmdtype==4) {
		env->debug=toggle;
	} else if (cmdtype==5) {
		env->final=toggle;
	}
}

void	sqlrsh::externalCommand(sqlrconnection *sqlrcon,
				sqlrcursor *sqlrcur, environment *env, 
				char *command) {

	// init stats
	initStats(env);

	// handle debug
	if (env->debug) {
		sqlrcon->debugOn();
	}

	// handle a commit/rollback
	if (!strncasecmp(command,"commit",6)) {

		sqlrcon->commit();

	} else if (!strncasecmp(command,"rollback",8)) {

		sqlrcon->rollback();

	} else {

		// send the query
		sqlrcur->sendQuery(command);
		if (env->final) {
			sqlrcon->endSession();
		}

		// look for an error
		if (sqlrcur->errorMessage()) {

			// display the error
			displayError(sqlrcur,env);

		} else {

			// display the header
			displayHeader(sqlrcur,env);

			// display the result set
			displayResultSet(sqlrcur,env);
		}
	}

	// set debug back off
	sqlrcon->debugOff();

	// display statistics
	displayStats(sqlrcur,env);
}

void	sqlrsh::initStats(environment *env) {

	if (!env->stats) {
		return;
	}

	// call clock here or something
	clock();
}

void	sqlrsh::displayError(sqlrcursor *sqlrcur, environment *env) {

	cyan(env);
	cout << sqlrcur->errorMessage() << endl << ENDL;
	white(env);
}

void	sqlrsh::displayHeader(sqlrcursor *sqlrcur, environment *env) {

	if (!env->headers) {
		return;
	}

	// display column names
	int	charcount=0;
	int	colcount=sqlrcur->colCount();
	char	*name;
	int	namelen;
	int	longest;

	// iterate through columns
	for (int i=0; i<sqlrcur->colCount(); i++) {

		// write the column name
		if (i%2==1) {
			green(env);
		} else {
			yellow(env);
		}
		name=sqlrcur->getColumnName(i);
		cout << name;
		white(env);

		// which is longer, field name or longest field
		namelen=strlen(name);
		longest=sqlrcur->getLongest(i);
		if (namelen>longest) {
			longest=namelen;
		}
		charcount=charcount+longest;

		// pad after the name with spaces
		for (int j=namelen; j<longest; j++) {
			cout << " ";
		}

		// put an extra space between names
		if (i<colcount-1) {
			cout << " ";
			charcount=charcount+1;
		}
	}
	cout << ENDL;

	// display delimiter
	red(env);
	for (int i=0; i<charcount; i++) {
		cout << "=";
	}
	cout << ENDL;
	white(env);
}

void	sqlrsh::displayResultSet(sqlrcursor *sqlrcur, environment *env) {

	// display column names
	int	colcount=sqlrcur->colCount();
	int	namelen;
	int	longest;

	for (int i=0; i<sqlrcur->rowCount(); i++) {
		for (int j=0; j<colcount; j++) {

			// write the column value
			if (i%2==1) {
				cyan(env);
			} else {
				white(env);
			}
			cout << sqlrcur->getField(i,j);
			white(env);

			// which is longer, field name or longest field
			longest=sqlrcur->getLongest(j);
			if (env->headers) {
				namelen=strlen(sqlrcur->getColumnName(j));
				if (namelen>longest) {
					longest=namelen;
				}
			}

			// pad after the name with spaces
			for (int k=strlen(sqlrcur->getField(i,j)); 
						k<longest; k++) {
				cout << " ";
			}

			// put an extra space between names
			if (j<colcount-1) {
				cout << " ";
			}
		}
		cout << ENDL;
	}
}

void	sqlrsh::displayStats(sqlrcursor *sqlrcur, environment *env) {

	if (!env->stats) {
		return;
	}

	// call clock again, display results
	red(env);
	cout << "	Rows Returned   : ";
	magenta(env);
	cout << sqlrcur->rowCount() << ENDL;
	red(env);
	cout << "	Fields Returned : ";
	magenta(env);
	cout << sqlrcur->rowCount()*sqlrcur->colCount() << ENDL;
	red(env);
	cout << "	System time     : ";
	magenta(env);
	cout << clock() << ENDL;
	white(env);
}

void	sqlrsh::displayHelp(environment *env) {

	cout << ENDL;
	yellow(env);
	cout << "	To run a query, simply type it at the prompt," << ENDL;
	cout << "	followed by a semicolon.  Queries may be " << ENDL;
	cout << "	split over multiple lines." << ENDL;
	cout << ENDL;
	cyan(env);
	cout << "		color on/off	- ";
	green(env);
	cout << "toggles colorizing" << ENDL;
	cyan(env);
	cout << "		headers on/off	- ";
	green(env);
	cout << "toggles column descriptions before result set" << ENDL;
	cyan(env);
	cout << "		stats on/off	- ";
	green(env);
	cout << "toggles statistics after result set" << ENDL;
	cyan(env);
	cout << "		debug on/off	- ";
	green(env);
	cout << "toggles debug messages" << ENDL;
	cyan(env);
	cout << "		final on/off	- ";
	green(env);
	cout << "toggles use of one session per query" << ENDL;
	cyan(env);
	cout << "		exit/quit	- ";
	green(env);
	cout << "exits" << ENDL;
	cout << ENDL;
	yellow(env);
	cout << "	All commands must be followed by a semicolon.";
	cout << ENDL;
	white(env);
}

void	sqlrsh::startupMessage(environment *env,
				char *host, int port, char *user) {

	red(env);
	cout << "SQLRShell - ";
	green(env);
	cout << "Version " << "0.22" << ENDL;
	yellow(env);
	cout << "	Connected to: ";
	blue(env);
	cout << host << ":" << port << " as ";
	cout << user << endl << ENDL;
	yellow(env);
	cout << "	type help; for a help." << endl << ENDL;
	white(env);
}

void	sqlrsh::interactWithUser(sqlrconnection *sqlrcon, sqlrcursor *sqlrcur, 
							environment *env) {

	// init some variables
	char		buffer1;
	char		buffer2;
	strstream	*command;
	int		exit=0;
	int		promptcount;

	while (!exit) {

		// prompt the user
		promptcount=0;
		
		// get the command
		command=new strstream();
		#ifdef HAVE_READLINE
			int	done=0;
			while (!done) {
				strstream	*prmpt=new strstream();
				*prmpt << promptcount << "> " << ENDS;
				char	*cmd=readline(prmpt->str());
				if (cmd[0]) {
					text::rightTrim(cmd);
					add_history(cmd);
				}
				int	len=strlen(cmd);
				done=0;
				for (int i=0; i<len; i++) {
					if (i==len-1) {
					       if (cmd[i]==';') {
							*command << ENDS;
							done=1;
						} else {
							*command << cmd[i];
							*command << ENDL;
						}
					} else if (cmd[i]>=32 || 
							cmd[i]=='	') {
						*command << cmd[i];
					}
				}
				if (!done) {
					promptcount++;
					*command << " ";
				}
				delete[] cmd;
				delete prmpt;
			}
		#else
			prompt(promptcount);
			while (1) {
				cin.read(&buffer1,1);
				if (buffer1==';') {
					cin.read(&buffer2,1);
					if (buffer2=='\n') {
						*command << ENDS;
						break;
					} else {
						*command << buffer1;
					}
				} else if (buffer1=='\n') {
					promptcount++;
					*command << " ";
					prompt(promptcount);
				} else if (buffer1>=32 || buffer1=='	') {
					*command << buffer1;
				}
			}
		#endif

		// run the command
		if (!runCommand(sqlrcon,sqlrcur,env,command->str())) {	
			exit=1;
		}

		delete command;
	}
}

void	sqlrsh::prompt(int promptcount) {

	cout << promptcount << "> ";
}

void	sqlrsh::error(char *errstring) {

	// print the error
	cerr << errstring << ENDL;
}

void	sqlrsh::execute(int argc, char **argv) {


	commandline	*cmdline=new commandline(argc,argv);
#if defined(HAVE_LIBXML) || defined(HAVE_GNOME_XML)
	configfile	*cfgfile=NULL;
	usernode	*currentnode=NULL;
#endif
	char		*host;
	int		port;
	char		*socket;
	char		*user;
	char		*password;
	char		*script=NULL;

#if defined(HAVE_LIBXML) || defined(HAVE_GNOME_XML)
	char	*config=cmdline->value("-config");
	if (!(config && config[0])) {
		config=DEFAULT_CONFIG_FILE;
	}
	char	*id=cmdline->value("-id");

	if (!(id && id[0])) {
#endif

		if (argc<6) {
			cout << "usage: sqlrsh  host port socket ";
			cout << "user password [script]" << ENDL;
			#if defined(HAVE_LIBXML) || defined(HAVE_GNOME_XML)
				cout << "  or   sqlrsh  [-config configfile] ";
				cout << "-id id [script]" << ENDL;
			#endif
			exit(0);
		}

		host=argv[1];
		port=atoi(argv[2]);
		socket=argv[3];
		user=argv[4];
		password=argv[5];
		if (argv[6]) {
			script=argv[6];
		}

#if defined(HAVE_LIBXML) || defined(HAVE_GNOME_XML)
	} else {

		cfgfile=new configfile();
		if (cfgfile->parse(config,id)) {

			// get the host/port/socket/username/password
			host="localhost";
			port=cfgfile->getPort();
			socket=cfgfile->getUnixPort();
			currentnode=cfgfile->getUsers();
			user=currentnode->getUser();
			password=currentnode->getPassword();
		} else {
			delete cfgfile;
			delete cmdline;
			return;
		}

		// find the script if there is one
		for (int i=1; i<argc; i++) {
			if (argv[i][0]=='-') {
				i++;
				continue;
			}
			script=argv[i];
			break;
		}
	}
#endif

	// connect to sql relay
	sqlrconnection	*sqlrcon=new sqlrconnection(host,port,socket,
							user,password,0,1);
	sqlrcursor	*sqlrcur=new sqlrcursor(sqlrcon);

	// set up an environment
	environment	*env=new environment();

	// process RC files
	systemRcFile(sqlrcon,sqlrcur,env);
	userRcFile(sqlrcon,sqlrcur,env);


	#ifdef HAVE_READLINE

		// handle the history file
		char	*filename;
		char	*home=getenv("HOME");
		if (home && home[0]) {
			filename=new char[strlen(home)+16+1];
			sprintf(filename,"%s/.sqlrsh_history",home);

			// create the history file if it doesn't exist now
			FILE	*historyfile=fopen(filename,"a");
			fclose(historyfile);

			read_history(filename);
		}
	#endif

	// if a script was specified, run it otherwise go into interactive mode
	if (script) {
		runScript(sqlrcon,sqlrcur,env,script,1);
	} else {
		startupMessage(env,host,port,user);
		interactWithUser(sqlrcon,sqlrcur,env);
	}

	// clean up
	#ifdef HAVE_READLINE
		if (home && home[0]) {
			write_history(filename);
			history_truncate_file(filename,100);
			delete[] filename;
		}
	#endif


#if defined(HAVE_LIBXML) || defined(HAVE_GNOME_XML)
	if (cfgfile) {
		delete cfgfile;
	}
#endif
	if (cmdline) {
		delete cmdline;
	}
	delete sqlrcur;
	delete sqlrcon;
	delete env;
}

void	sqlrsh::setColor(environment *env, int value) {
	if (env->color) {
		cout << "\033[0;" << value << "m";
	}
}

void	sqlrsh::black(environment *env) {
	setColor(env,30);
}

void	sqlrsh::red(environment *env) {
	setColor(env,31);
}

void	sqlrsh::green(environment *env) {
	setColor(env,32);
}

void	sqlrsh::yellow(environment *env) {
	setColor(env,33);
}

void	sqlrsh::blue(environment *env) {
	setColor(env,34);
}

void	sqlrsh::magenta(environment *env) {
	setColor(env,35);
}

void	sqlrsh::cyan(environment *env) {
	setColor(env,36);
}

void	sqlrsh::white(environment *env) {
	setColor(env,37);
}



int	main(int argc, char **argv) {

	#include <version.h>

	sqlrsh	s;
	s.execute(argc,argv);
}
