// Copyright (c) 2000  David Muse
// See the file COPYING for more information

#include <configfile.h>
#include <iostream.h>

#include <string.h>

configfile::configfile() {
	doc=NULL;
	firstinstance=NULL;
	lastinstance=NULL;
	currentfile=NULL;
}

configfile::~configfile() {
	close();
}

void	configfile::blank() {

	currentfile=NULL;

	if (doc) {
		xmlFreeDoc(doc);
	}

	doc=xmlNewDoc((const xmlChar *)"1.0");
	#if defined(HAVE_LIBXML)
	doc->children=xmlNewDocNode(doc,NULL,(const xmlChar *)"instances",NULL);
	root=doc->children;
	#elif defined(HAVE_GNOME_XML)
	doc->root=xmlNewDocNode(doc,NULL,(const xmlChar *)"instances",NULL);
	root=doc->root;
	#endif
}

int	configfile::parse(char *filename) {

	currentfile=filename;

	if (doc) {
		xmlFreeDoc(doc);
	}

	// parse the file
	doc=xmlParseFile(filename);

	if (doc) {

		// get the root element
		root=xmlDocGetRootElement(doc);

		if (root) {

			// build the instance list
			firstinstance=NULL;
			lastinstance=NULL;
			#if defined(HAVE_LIBXML)
			if (root->children && root->children->next) {
				currentnode=root->children->next;
			#elif defined(HAVE_GNOME_XML)
			if (root->children) {
				currentnode=root->children;
			#endif
				firstinstance=new instance(currentnode);
				currentinstance=firstinstance;
				lastinstance=firstinstance;
				while (1) {
					#if defined(HAVE_LIBXML)
					if (currentnode->next && 
						currentnode->next->next) {
						currentnode=currentnode
								->next->next;
					#elif defined(HAVE_GNOME_XML)
					if (currentnode->next) {
						currentnode=currentnode->next;
					#endif
						currentinstance->next=new 
							instance(currentnode);
						currentinstance->next->last=
								currentinstance;
						currentinstance=currentinstance
								->next;
						lastinstance=currentinstance;
					} else {
						break;
					}
				}
			}
			return 1;
		}
		return 0;
	} else {
		return 0;
	}
}

int	configfile::write() {
	return xmlSaveFile(currentfile,doc);
}

int	configfile::write(char *filename) {
	return xmlSaveFile(filename,doc);
}

void	configfile::close() {

	// delete the instance list
	if (firstinstance) {
		currentinstance=firstinstance;
		while (currentinstance) {
			firstinstance=currentinstance;
			currentinstance=currentinstance->next;
			delete firstinstance;
		}
	}
	firstinstance=NULL;
	lastinstance=NULL;

	// free the doc
	if (doc) {
		xmlFreeDoc(doc);
		doc=NULL;
	}

	// set currentfile NULL
	currentfile=NULL;
}

char	*configfile::currentFile() {
	return currentfile;
}

instance	*configfile::addInstance(char *id, char *port, 
				char *unixport, char *dbase,
				char *connections, char *maxconnections,
				char *maxqueuelength, char *growby,
				char *ttl, char *endofsession,
				char *sessiontimeout,
				char *runasuser, char *runasgroup,
				char *cursors, char *authtier,
				char *handoff,
				char *deniedips, char *allowedips,
				char *debug) {

	xmlNodePtr	newchild=xmlNewChild(root,NULL,
					(const xmlChar *)"instance",NULL);

	xmlSetProp(newchild,
			(const xmlChar *)"id",
			(const xmlChar *)id);
	xmlSetProp(newchild,
			(const xmlChar *)"port",
			(const xmlChar *)port);
	xmlSetProp(newchild,
			(const xmlChar *)"unixport",
			(const xmlChar *)unixport);
	xmlSetProp(newchild,
			(const xmlChar *)"dbase",
			(const xmlChar *)dbase);
	xmlSetProp(newchild,
			(const xmlChar *)"connections",
			(const xmlChar *)connections);
	xmlSetProp(newchild,
			(const xmlChar *)"maxconnections",
			(const xmlChar *)maxconnections);
	xmlSetProp(newchild,
			(const xmlChar *)"maxqueuelength",
			(const xmlChar *)maxqueuelength);
	xmlSetProp(newchild,
			(const xmlChar *)"growby",
			(const xmlChar *)growby);
	xmlSetProp(newchild,
			(const xmlChar *)"ttl",
			(const xmlChar *)ttl);
	xmlSetProp(newchild,
			(const xmlChar *)"endofsession",
			(const xmlChar *)endofsession);
	xmlSetProp(newchild,
			(const xmlChar *)"sessiontimeout",
			(const xmlChar *)sessiontimeout);
	xmlSetProp(newchild,
			(const xmlChar *)"runasuser",
			(const xmlChar *)runasuser);
	xmlSetProp(newchild,
			(const xmlChar *)"runasgroup",
			(const xmlChar *)runasgroup);
	xmlSetProp(newchild,
			(const xmlChar *)"cursors",
			(const xmlChar *)cursors);
	xmlSetProp(newchild,
			(const xmlChar *)"authtier",
			(const xmlChar *)authtier);
	xmlSetProp(newchild,
			(const xmlChar *)"handoff",
			(const xmlChar *)handoff);
	xmlSetProp(newchild,
			(const xmlChar *)"deniedips",
			(const xmlChar *)deniedips);
	xmlSetProp(newchild,
			(const xmlChar *)"allowedips",
			(const xmlChar *)allowedips);
	xmlSetProp(newchild,
			(const xmlChar *)"debug",
			(const xmlChar *)debug);

	xmlNodePtr	newusers=xmlNewChild(newchild,NULL,
					(const xmlChar *)"users",NULL);
	xmlNodePtr	newconnections=xmlNewChild(newchild,NULL,
					(const xmlChar *)"connections",NULL);

	if (lastinstance) {
		lastinstance->next=new instance(newchild);
		lastinstance->next->last=lastinstance;
		lastinstance=lastinstance->next;
	} else {
		lastinstance=new instance(newchild);
		firstinstance=lastinstance;
	}
	return lastinstance;
}

void	configfile::deleteInstance(instance *instanceptr) {

	if (instanceptr==firstinstance) {
		firstinstance=instanceptr->next;
	}
	if (instanceptr==lastinstance) {
		lastinstance=instanceptr->last;
	}
	if (instanceptr->next) {
		instanceptr->next->last=instanceptr->last;
	}
	if (instanceptr->last) {
		instanceptr->last->next=instanceptr->next;
	}
	delete instanceptr;
}

instance	*configfile::findInstance(char *id) {

	char	*currentid;

	currentinstance=firstinstance;

	while (currentinstance) {
		currentid=currentinstance->getId();
		if (currentid && !strcmp(id,currentid)) {
			delete currentid;
			return currentinstance;
		} else {
			if (currentinstance->next) {
				currentinstance=currentinstance->next;
			} else {
				break;
			}
		}
	}

	return NULL;
}

instance	*configfile::firstInstance() {
	return firstinstance;
}

instance::instance(xmlNodePtr instancenode) {

	this->instancenode=instancenode;

	next=NULL;
	last=NULL;

	// find the first user
	xmlNodePtr	currentnode=instancenode->children;
	while (currentnode && strcmp((char *)currentnode->name,"users")) {
		currentnode=currentnode->next;
	}
	users=currentnode;

	// build the user list
	firstuser=NULL;
	lastuser=NULL;
	#if defined(HAVE_LIBXML)
	if (currentnode && currentnode->children && 
			currentnode->children->next) {
		currentnode=currentnode->children->next;
	#elif defined(HAVE_GNOME_XML)
	if (currentnode && currentnode->children) {
		currentnode=currentnode->children;
	#endif

		firstuser=new user(currentnode);
		currentuser=firstuser;
		lastuser=firstuser;

		while (1) {
			#if defined(HAVE_LIBXML)
			if (currentnode->next && currentnode->next->next) {
				currentnode=currentnode->next->next;
			#elif defined(HAVE_GNOME_XML)
			if (currentnode->next) {
				currentnode=currentnode->next;
			#endif
				currentuser->next=new 
						user(currentnode);
				currentuser->next->last=currentuser;
				currentuser=currentuser->next;
				lastuser=currentuser;
			} else {
				break;
			}
		}
	}


	// find the first connection
	currentnode=instancenode->children;
	while (currentnode && strcmp((char *)currentnode->name,"connections")) {
		currentnode=currentnode->next;
	}
	connections=currentnode;

	// build the connection list
	firstconnection=NULL;
	lastconnection=NULL;
	#if defined(HAVE_LIBXML)
	if (currentnode && currentnode->children && 
			currentnode->children->next) {
		currentnode=currentnode->children->next;
	#elif defined(HAVE_GNOME_XML)
	if (currentnode && currentnode->children) {
		currentnode=currentnode->children;
	#endif

		firstconnection=new connection(currentnode);
		currentconnection=firstconnection;
		lastconnection=firstconnection;

		while (1) {
			#if defined(HAVE_LIBXML)
			if (currentnode->next && currentnode->next->next) {
				currentnode=currentnode->next->next;
			#elif defined(HAVE_GNOME_XML)
			if (currentnode->next) {
				currentnode=currentnode->next;
			#endif
				currentconnection->next=new 
						connection(currentnode);
				currentconnection=currentconnection->next;
				currentconnection->next->last=currentconnection;
				lastconnection=currentconnection;
			} else {
				break;
			}
		}
	}
}

instance::~instance() {

	// delete the user list
	if (firstuser) {
		currentuser=firstuser;
		while (currentuser) {
			firstuser=currentuser;
			currentuser=currentuser->next;
			delete firstuser;
		}
	}

	// delete the connection list
	if (firstconnection) {
		currentconnection=firstconnection;
		while (currentconnection) {
			firstconnection=currentconnection;
			currentconnection=currentconnection->next;
			delete firstconnection;
		}
	}

	if (next) {
		next->last=last;
	}
	if (last) {
		last->next=next;
	}
	xmlUnlinkNode(instancenode);
}

char	*instance::getId() {
	char	*retval=(char *)xmlGetProp(instancenode,
					(const xmlChar *)"id");
	if (retval) {
		return retval;
	}
	return "";
}

char	*instance::getPort() {
	char	*retval=(char *)xmlGetProp(instancenode,
					(const xmlChar *)"port");
	if (retval) {
		return retval;
	}
	return "";
}

char	*instance::getUnixPort() {
	char	*retval=(char *)xmlGetProp(instancenode,
					(const xmlChar *)"socket");
	if (retval) {
		return retval;
	} else {
		retval=(char *)xmlGetProp(instancenode,
					(const xmlChar *)"unixport");
		if (retval) {
			return retval;
		}
	}
	return "";
}

char	*instance::getDbase() {
	char	*retval=(char *)xmlGetProp(instancenode,
					(const xmlChar *)"dbase");
	if (retval) {
		return retval;
	}
	return "";
}

char	*instance::getConnections() {
	char	*retval=(char *)xmlGetProp(instancenode,
					(const xmlChar *)"connections");
	if (retval) {
		return retval;
	}
	return "";
}

char	*instance::getMaxConnections() {
	char	*retval=(char *)xmlGetProp(instancenode,
					(const xmlChar *)"maxconnections");
	if (retval) {
		return retval;
	}
	return "";
}

char	*instance::getMaxQueueLength() {
	char	*retval=(char *)xmlGetProp(instancenode,
					(const xmlChar *)"maxqueuelength");
	if (retval) {
		return retval;
	}
	return "";
}

char	*instance::getGrowby() {
	char	*retval=(char *)xmlGetProp(instancenode,
					(const xmlChar *)"growby");
	if (retval) {
		return retval;
	}
	return "";
}

char	*instance::getTtl() {
	char	*retval=(char *)xmlGetProp(instancenode,
					(const xmlChar *)"ttl");
	if (retval) {
		return retval;
	}
	return "";
}

char	*instance::getEndOfSession() {
	char	*retval=(char *)xmlGetProp(instancenode,
					(const xmlChar *)"endofsession");
	if (retval) {
		return retval;
	}
	return "";
}

char	*instance::getSessionTimeout() {
	char	*retval=(char *)xmlGetProp(instancenode,
					(const xmlChar *)"sessiontimeout");
	if (retval) {
		return retval;
	}
	return "";
}

char	*instance::getRunAsUser() {
	char	*retval=(char *)xmlGetProp(instancenode,
					(const xmlChar *)"runasuser");
	if (retval) {
		return retval;
	}
	return "";
}

char	*instance::getRunAsGroup() {
	char	*retval=(char *)xmlGetProp(instancenode,
					(const xmlChar *)"runasgroup");
	if (retval) {
		return retval;
	}
	return "";
}

char	*instance::getCursors() {
	char	*retval=(char *)xmlGetProp(instancenode,
					(const xmlChar *)"cursors");
	if (retval) {
		return retval;
	}
	return "";
}

char	*instance::getAuthTier() {
	char	*retval=(char *)xmlGetProp(instancenode,
					(const xmlChar *)"authtier");
	if (retval) {
		return retval;
	}
	return "";
}

char	*instance::getHandoff() {
	char	*retval=(char *)xmlGetProp(instancenode,
					(const xmlChar *)"handoff");
	if (retval) {
		return retval;
	}
	return "";
}

char	*instance::getDeniedIps() {
	char	*retval=(char *)xmlGetProp(instancenode,
					(const xmlChar *)"deniedips");
	if (retval) {
		return retval;
	}
	return "";
}

char	*instance::getAllowedIps() {
	char	*retval=(char *)xmlGetProp(instancenode,
					(const xmlChar *)"allowedips");
	if (retval) {
		return retval;
	}
	return "";
}

char	*instance::getDebug() {
	char	*retval=(char *)xmlGetProp(instancenode,
					(const xmlChar *)"debug");
	if (retval) {
		return retval;
	}
	return "";
}

void	instance::setId(char *id) {
	xmlSetProp(instancenode,
			(const xmlChar *)"id",
			(const xmlChar *)id);
}

void	instance::setPort(char *port) {
	xmlSetProp(instancenode,
			(const xmlChar *)"port",
			(const xmlChar *)port);
}

void	instance::setUnixPort(char *unixport) {
	xmlSetProp(instancenode,
			(const xmlChar *)"socket",
			(const xmlChar *)unixport);
}


void	instance::setDbase(char *dbase) {
	xmlSetProp(instancenode,
			(const xmlChar *)"dbase",
			(const xmlChar *)dbase);
}

void	instance::setConnections(char *connections) {
	xmlSetProp(instancenode,
			(const xmlChar *)"connections",
			(const xmlChar *)connections);
}

void	instance::setMaxConnections(char *maxconnections) {
	xmlSetProp(instancenode,
			(const xmlChar *)"maxconnections",
			(const xmlChar *)maxconnections);
}

void	instance::setMaxQueueLength(char *maxqueuelength) {
	xmlSetProp(instancenode,
			(const xmlChar *)"maxqueuelength",
			(const xmlChar *)maxqueuelength);
}

void	instance::setGrowby(char *growby) {
	xmlSetProp(instancenode,
			(const xmlChar *)"growby",
			(const xmlChar *)growby);
}

void	instance::setTtl(char *ttl) {
	xmlSetProp(instancenode,
			(const xmlChar *)"ttl",
			(const xmlChar *)ttl);
}

void	instance::setEndOfSession(char *endofsession) {
	xmlSetProp(instancenode,
			(const xmlChar *)"endofsession",
			(const xmlChar *)endofsession);
}

void	instance::setSessionTimeout(char *sessiontimeout) {
	xmlSetProp(instancenode,
			(const xmlChar *)"sessiontimeout",
			(const xmlChar *)sessiontimeout);
}

void	instance::setRunAsUser(char *runasuser) {
	xmlSetProp(instancenode,
			(const xmlChar *)"runasuser",
			(const xmlChar *)runasuser);
}

void	instance::setRunAsGroup(char *runasgroup) {
	xmlSetProp(instancenode,
			(const xmlChar *)"runasgroup",
			(const xmlChar *)runasgroup);
}

void	instance::setCursors(char *cursors) {
	xmlSetProp(instancenode,
			(const xmlChar *)"cursors",
			(const xmlChar *)cursors);
}

void	instance::setAuthTier(char *authtier) {
	xmlSetProp(instancenode,
			(const xmlChar *)"authtier",
			(const xmlChar *)authtier);
}

void	instance::setHandoff(char *handoff) {
	xmlSetProp(instancenode,
			(const xmlChar *)"handoff",
			(const xmlChar *)handoff);
}

void	instance::setDeniedIps(char *deniedips) {
	xmlSetProp(instancenode,
			(const xmlChar *)"deniedips",
			(const xmlChar *)deniedips);
}

void	instance::setAllowedIps(char *allowedips) {
	xmlSetProp(instancenode,
			(const xmlChar *)"allowedips",
			(const xmlChar *)allowedips);
}

void	instance::setDebug(char *debug) {
	xmlSetProp(instancenode,
			(const xmlChar *)"debug",
			(const xmlChar *)debug);
}

user	*instance::addUser(char *usr, char *password) {

	xmlNodePtr	newchild=xmlNewChild(users,NULL,
					(const xmlChar *)"user",NULL);

	xmlSetProp(newchild,
			(const xmlChar *)"user",
			(const xmlChar *)usr);
	xmlSetProp(newchild,
			(const xmlChar *)"password",
			(const xmlChar *)password);

	if (lastuser) {
		lastuser->next=new user(newchild);
		lastuser->next->last=lastuser;
		lastuser=lastuser->next;
	} else {
		lastuser=new user(newchild);
		firstuser=lastuser;
	}
	return lastuser;
}

void	instance::deleteUser(user *userptr) {

	if (userptr==firstuser) {
		firstuser=userptr->next;
	}
	if (userptr==lastuser) {
		lastuser=userptr->last;
	}
	if (userptr->next) {
		userptr->next->last=userptr->last;
	}
	if (userptr->last) {
		userptr->last->next=userptr->next;
	}
	delete userptr;
}

user	*instance::findUser(char *userid) {

	char	*currentuserid;

	currentuser=firstuser;

	while (currentuser) {
		currentuserid=currentuser->getUser();
		if (currentuserid && !strcmp(userid,currentuserid)) {
			delete currentuserid;
			return currentuser;
		} else {
			if (currentuser->next) {
				currentuser=currentuser->next;
			} else {
				break;
			}
		}
	}

	return NULL;
}

user	*instance::firstUser() {
	return firstuser;
}

user::user(xmlNodePtr usernode) {
	this->usernode=usernode;
	next=NULL;
	last=NULL;
}

user::~user() {
	if (next) {
		next->last=last;
	}
	if (last) {
		last->next=next;
	}
	xmlUnlinkNode(usernode);
}

char	*user::getUser() {
	char	*retval=(char *)xmlGetProp(usernode,
						(const xmlChar *)"user");
	if (retval) {
		return retval;
	}
	return "";
}

char	*user::getPassword() {
	char	*retval=(char *)xmlGetProp(usernode,
						(const xmlChar *)"password");
	if (retval) {
		return retval;
	}
	return "";
}

void	user::setUser(char *user) {
	xmlSetProp(usernode,
			(const xmlChar *)"user",
			(const xmlChar *)user);
}

void	user::setPassword(char *password) {
	xmlSetProp(usernode,
			(const xmlChar *)"password",
			(const xmlChar *)password);
}

user	*user::nextUser() {
	return next;
}

connection	*instance::addConnection(char *connectionid, char *string, 
				char *metric) {

	xmlNodePtr	newchild=xmlNewChild(connections,NULL,
					(const xmlChar *)"connection",NULL);

	xmlSetProp(newchild,
			(const xmlChar *)"connectionid",
			(const xmlChar *)connectionid);
	xmlSetProp(newchild,
			(const xmlChar *)"string",
			(const xmlChar *)string);
	xmlSetProp(newchild,
			(const xmlChar *)"metric",
			(const xmlChar *)metric);

	if (lastconnection) {
		lastconnection->next=new connection(newchild);
		lastconnection->next->last=lastconnection;
		lastconnection=lastconnection->next;
	} else {
		lastconnection=new connection(newchild);
		firstconnection=lastconnection;
	}
	return lastconnection;
}

void	instance::deleteConnection(connection *connectionptr) {

	if (connectionptr==firstconnection) {
		firstconnection=connectionptr->next;
	}
	if (connectionptr==lastconnection) {
		lastconnection=connectionptr->last;
	}
	if (connectionptr->next) {
		connectionptr->next->last=connectionptr->last;
	}
	if (connectionptr->last) {
		connectionptr->last->next=connectionptr->next;
	}
	delete connectionptr;
}

connection	*instance::findConnection(char *connectionnode) {

	char	*currentconnid;

	currentconnection=firstconnection;

	while (currentconnection) {
		currentconnid=currentconnection->getConnectionId();
		if (currentconnid && !strcmp(connectionnode,currentconnid)) {
			delete currentconnid;
			return currentconnection;
		} else {
			if (currentconnection->next) {
				currentconnection=currentconnection->next;
			} else {
				break;
			}
		}
	}

	return NULL;
}


connection	*instance::firstConnection() {
	return firstconnection;
}

instance	*instance::nextInstance() {
	return next;
}

connection::connection(xmlNodePtr connectionnode) {
	this->connectionnode=connectionnode;
	next=NULL;
	last=NULL;
}

connection::~connection() {
	if (next) {
		next->last=last;
	}
	if (last) {
		last->next=next;
	}
	xmlUnlinkNode(connectionnode);
}

char	*connection::getConnectionId() {
	char	*retval=(char *)xmlGetProp(connectionnode,
					(const xmlChar *)"connectionid");
	if (retval) {
		return retval;
	}
	return "";
}

char	*connection::getString() {
	char	*retval=(char *)xmlGetProp(connectionnode,
					(const xmlChar *)"string");
	if (retval) {
		return retval;
	}
	return "";
}

char	*connection::getMetric() {
	char	*retval=(char *)xmlGetProp(connectionnode,
					(const xmlChar *)"metric");
	if (retval) {
		return retval;
	}
	return "";
}

void	connection::setConnectionId(char *connectionid) {
	xmlSetProp(connectionnode,
			(const xmlChar *)"connectionid",
			(const xmlChar *)connectionid);
}

void	connection::setString(char *string) {
	xmlSetProp(connectionnode,
			(const xmlChar *)"string",
			(const xmlChar *)string);
}

void	connection::setMetric(char *metric) {
	xmlSetProp(connectionnode,
			(const xmlChar *)"metric",
			(const xmlChar *)metric);
}

connection	*connection::nextConnection() {
	return next;
}
