// Copyright (c) 2000  David Muse
// See the file COPYING for more information.

#ifndef CONFIGFILE_H
#define CONFIGFILE_H

#include <config.h>

#if defined(HAVE_LIBXML)
	#include <libxml/xmlmemory.h>
	#include <libxml/parser.h>
#elif defined(HAVE_GNOME_XML)
	#define children childs
	#include <gnome-xml/xmlmemory.h>
	#include <gnome-xml/parser.h>
#endif

class	user {
	friend class instance;
	public:
			user(xmlNodePtr usernode);
			~user();
		char	*getUser();
		char	*getPassword();
		void	setUser(char *user);
		void	setPassword(char *password);
		user	*nextUser();
	private:
		xmlNodePtr	usernode;
		user		*next;
		user		*last;
};

class	connection {
	friend class instance;
	public:
			connection(xmlNodePtr connectionnode);
			~connection();
		char	*getConnectionId();
		char	*getString();
		char	*getMetric();
		void	setConnectionId(char *connectionid);
		void	setString(char *string);
		void	setMetric(char *metric);
		connection	*nextConnection();
	private:
		xmlNodePtr	connectionnode;
		connection	*next;
		connection	*last;
};


class	instance {
	friend class configfile;
	public:
			instance(xmlNodePtr instancenode);
			~instance();
		char	*getId();
		char	*getPort();
		char	*getUnixPort();
		char	*getDbase();
		char	*getConnections();
		char	*getMaxConnections();
		char	*getMaxQueueLength();
		char	*getGrowby();
		char	*getTtl();
		char	*getEndOfSession();
		char	*getSessionTimeout();
		char	*getRunAsUser();
		char	*getRunAsGroup();
		char	*getCursors();
		char	*getAuthTier();
		char	*getHandoff();
		char	*getDeniedIps();
		char	*getAllowedIps();
		char	*getDebug();
		void	setId(char *id);
		void	setPort(char *port);
		void	setUnixPort(char *unixport);
		void	setDbase(char *dbase);
		void	setConnections(char *connections);
		void	setMaxConnections(char *maxconnections);
		void	setMaxQueueLength(char *maxqueuelength);
		void	setGrowby(char *growby);
		void	setTtl(char *ttl);
		void	setEndOfSession(char *endofsession);
		void	setSessionTimeout(char *sessiontimeout);
		void	setRunAsUser(char *runasuser);
		void	setRunAsGroup(char *runasgroup);
		void	setCursors(char *cursors);
		void	setAuthTier(char *authtier);
		void	setHandoff(char *handoff);
		void	setDeniedIps(char *deniedips);
		void	setAllowedIps(char *allowedips);
		void	setDebug(char *debug);
		user	*addUser(char *usr, char *password);
		void	deleteUser(user *usr);
		user	*findUser(char *userid);
		user	*firstUser();
		connection	*addConnection(char *connectionid, 
					char *string, 
					char *metric);
		void	deleteConnection(connection *conn);
		connection	*findConnection(char *conn);
		connection	*firstConnection();
		instance	*nextInstance();
	private:
		xmlNodePtr	instancenode;
		xmlNodePtr	users;
		xmlNodePtr	connections;
		user		*firstuser;
		user		*currentuser;
		user		*lastuser;
		connection	*firstconnection;
		connection	*currentconnection;
		connection	*lastconnection;
		instance	*next;
		instance	*last;
};

class	configfile {
	public:
				configfile();
				~configfile();
		void		blank();
		int		parse(char *filename);
		int		write();
		int		write(char *filename);
		void		close();
		char		*currentFile();
		instance	*addInstance(char *id,
					char *port, char *unixport,
					char *dbase, char *connections, 
					char *maxconnections, 
					char *maxqueuelength, char *growby, 
					char *ttl, char *endofsession,
					char *sessiontimeout,
					char *runasuser, char *runasgroup,
					char *cursors, char *authtier, 
					char *handoff,
					char *deniedips, char *allowedips,
					char *debug);
		void		deleteInstance(instance *inst);
		instance	*findInstance(char *id);
		instance	*firstInstance();
	private:
		xmlDocPtr	doc;
		xmlNodePtr	root;

		xmlNodePtr	currentnode;

		instance	*firstinstance;
		instance	*currentinstance;
		instance	*lastinstance;

		char		*currentfile;
};

#endif
