// Copyright (c) 2000-2001  David Muse
// See the file COPYING for more information

#ifndef DB2CCONNECTION_H
#define DB2CCONNECTION_H

#define FETCH_AT_ONCE		10
#define MAX_SELECT_LIST_SIZE	256
#define MAX_ITEM_BUFFER_SIZE	4096

#define NUM_CONNECT_STRING_VARS 3

#include <connection.h>

#include <fstream.h>
#include <strstream.h>

#include <sqlcli1.h>

struct column {
	char	name[MAX_ITEM_BUFFER_SIZE];
	int	namelength;
	int	type;
	int	length;
};

class db2connection;

class db2cursor : public cursor {
	friend class db2connection;
	public:
			db2cursor(connection *conn);
			~db2cursor();
	private:
		int	prepareQuery(char *query, long length);
		int	inputBindValue(char *variable, 
					unsigned short variablesize,
					char *value, 
					unsigned short valuesize,
					short *isnull);
		int	inputBindValue(char *variable, 
					unsigned short variablesize,
					unsigned long *value);
		int	inputBindValue(char *variable, 
					unsigned short variablesize,
					double *value, 
					unsigned short precision,
					unsigned short scale);
		int	outputBindValue(char *variable, 
					unsigned short variablesize,
					char *value, 
					unsigned short valuesize,
					short *isnull);
		int	executeQuery(char *query, long length);
		char	*getErrorMessage(int *liveconnection);
		void	returnRowCounts();
		void	returnColumnCount();
		void	returnColumnInfo();
		int	noRowsToReturn();
		int	skipRow();
		int	fetchRow();
		void	returnRow();

		long		erg;
		SQLHSTMT	stmt;
		long		result;
		SQLSMALLINT	ncols;
		SQLINTEGER 	affectedrows;
		char		field[MAX_SELECT_LIST_SIZE]
					[FETCH_AT_ONCE]
					[MAX_ITEM_BUFFER_SIZE];
		SQLINTEGER	indicator[MAX_SELECT_LIST_SIZE]
						[FETCH_AT_ONCE];
		column 		col[MAX_SELECT_LIST_SIZE];

		int		row;
		int		maxrow;
		int		totalrows;
		int		rownumber;

		strstream	*errormsg;

		db2connection	*db2conn;
};

class db2connection : public connection {
	friend class db2cursor;
	private:
		int	getNumberOfConnectStringVars();
		void	handleConnectString();
		int	logIn();
		cursor	*initCursor();
		void	deleteCursor(cursor *curs);
		void	logOut();
		short	nullBindValue();
		int	bindValueIsNull(short isnull);
		unsigned short	autoCommitOn();
		unsigned short	autoCommitOff();
		int	commit();
		int	rollback();
		int	ping();
		char	*identify();

		SQLHENV		env;
		long		erg;
		SQLHDBC		dbc;

		char		*server;
		char		*user;
		char		*password;
		int		autocommit;
};

#endif
