// Copyright (c) 2000-2001  David Muse
// See the file COPYING for more information

#ifndef FREETDSCONNECTION_H
#define FREETDSCONNECTION_H

// FETCH_AT_ONCE must be 1 until freetds supports array fetches
#define FETCH_AT_ONCE		1
#define MAX_SELECT_LIST_SIZE	256
#define MAX_ITEM_BUFFER_SIZE	4096

// this code is here in case freetds ever supports bind vars
#define MAX_BIND_VARS		512

#define NUM_CONNECT_STRING_VARS 8

#include <connection.h>

#include <fstream.h>

extern "C" {
	#include <ctpublic.h>
	#include <ctfunctions.h>
}

class freetdsconnection;

class freetdscursor : public cursor {
	friend class freetdsconnection;
	private:
			freetdscursor(connection *conn);
			~freetdscursor();
		int	openCursor(int id);
		int	prepareQuery(char *query, long length);

		// this code is here in case freetds ever supports bind vars
		/*int	inputBindValue(char *variable,
						unsigned short variablesize,
						char *value,
						unsigned short valuesize,
						short *isnull);
		int	inputBindValue(char *variable,
						unsigned short variablesize,
						unsigned long *value);
		int	inputBindValue(char *variable,
						unsigned short variablesize,
						double *value,
						unsigned short precision,
						unsigned short scale);
		int	outputBindValue(char *variable, 
						unsigned short variablesize,
						char *value, 
						unsigned short valuesize, 
						short *isnull);*/

		int	executeQuery(char *query, long length);
		char	*getErrorMessage(int *liveconnection);
		void	returnRowCounts();
		void	returnColumnCount();
		void	returnColumnInfo();
		int	noRowsToReturn();
		int	skipRow();
		int	fetchRow();
		void	returnRow();
		void	cleanUpData();

		double		tdsversion;
		int		returnedcolumns;

		CS_COMMAND	*cmd;
		CS_INT		results_type;
		CS_INT		ncols;
		CS_INT		affectedrows;

		CS_INT		rowsread;
		int		row;
		int		maxrow;
		int		totalrows;

		// this code is here in case freetds ever supports bind vars
		/*CS_DATAFMT	parameter[MAX_BIND_VARS];
		int		paramindex;*/

		CS_DATAFMT	column[MAX_SELECT_LIST_SIZE];
		char		data[MAX_SELECT_LIST_SIZE]
					[FETCH_AT_ONCE]
					[MAX_ITEM_BUFFER_SIZE];
		CS_INT		datalength[MAX_SELECT_LIST_SIZE]
					[FETCH_AT_ONCE];
		CS_SMALLINT	nullindicator[MAX_SELECT_LIST_SIZE]
					[FETCH_AT_ONCE];

		freetdsconnection	*freetdsconn;
};

class freetdsconnection : public connection {
	friend class freetdscursor;
	public:
			freetdsconnection();
			~freetdsconnection();
	private:
		int	getNumberOfConnectStringVars();
		void	handleConnectString();
		int	logIn();
		void	logInError(char *error);
		cursor	*initCursor();
		void	deleteCursor(cursor *curs);
		void	logOut();
		int	ping();
		char	*identify();
		char	bindVariablePrefix();

		CS_CONTEXT	*context;
		CS_LOCALE	*locale;
		CS_CONNECTION	*dbconn;

		char		*sybaseenv;
		char		*dsqueryenv;

		char		*sybase;
		char		*user;
		char		*password;
		char		*server;
		char		*db;
		char		*charset;
		char		*language;
		char		*encryption;
		int		enc;
		char		*hostname;
		char		*packetsize;

		static	strstream	*errorstring;
		static	int		deadconnection;

		static	CS_RETCODE	csMessageCallback(CS_CONTEXT *ctxt,
						CS_CLIENTMSG *msgp);
		static	CS_RETCODE	clientMessageCallback(CS_CONTEXT *ctxt,
						CS_CONNECTION *cnn,
						CS_CLIENTMSG *msgp);
		static	CS_RETCODE	serverMessageCallback(CS_CONTEXT *ctxt,
						CS_CONNECTION *cnn,
						CS_SERVERMSG *msgp);
};

#endif
