// Copyright (c) 2000-2001  David Muse
// See the file COPYING for more information

#ifndef ODBCCCONNECTION_H
#define ODBCCCONNECTION_H

#define FETCH_AT_ONCE		10
#define MAX_SELECT_LIST_SIZE	256
#define MAX_ITEM_BUFFER_SIZE	4096

#define NUM_CONNECT_STRING_VARS 3

#include <fstream.h>
#include <strstream.h>

#ifdef HAVE_IODBC
	#include <iodbcinst.h>
#endif
#include <sql.h>
#include <sqlext.h>
#include <sqltypes.h>

// note that connection.h must be included after sqltypes.h to
// get around a problem with CHAR/xmlChar in gnome-xml
#include <connection.h>

#include <config.h>

struct column {
	char	name[MAX_ITEM_BUFFER_SIZE];
	int	namelength;
	int	type;
	int	length;
};

class odbcconnection;

class odbccursor : public cursor {
	friend class odbcconnection;
	private:
			odbccursor(connection *conn);
			~odbccursor();
		int	prepareQuery(char *query, long length);
		int	inputBindValue(char *variable, 
					unsigned short variablesize,
					char *value, 
					unsigned short valuesize,
					short *isnull);
		int	inputBindValue(char *variable, 
					unsigned short variablesize,
					unsigned long *value);
		int	inputBindValue(char *variable, 
					unsigned short variablesize,
					double *value, 
					unsigned short precision,
					unsigned short scale);
		int	outputBindValue(char *variable, 
					unsigned short variablesize,
					char *value, 
					unsigned short valuesize,
					short *isnull);
		short	nonNullBindValue();
		short	nullBindValue();
		int	bindValueIsNull(short isnull);
		int	executeQuery(char *query, long length);
		char	*getErrorMessage(int *liveconnection);
		void	returnRowCounts();
		void	returnColumnCount();
		void	returnColumnInfo();
		int	noRowsToReturn();
		int	skipRow();
		int	fetchRow();
		void	returnRow();


		long		erg;
		SQLHSTMT	stmt;
		long		result;
		SQLSMALLINT	ncols;
		SQLINTEGER 	affectedrows;

// this code is here in case unixodbc ever 
// successfully supports array fetches

/*#ifdef HAVE_UNIXODBC
		char		field[MAX_SELECT_LIST_SIZE]
					[FETCH_AT_ONCE]
					[MAX_ITEM_BUFFER_SIZE];
		SQLINTEGER	indicator[MAX_SELECT_LIST_SIZE]
						[FETCH_AT_ONCE];
#else*/
		char		field[MAX_SELECT_LIST_SIZE]
					[MAX_ITEM_BUFFER_SIZE];
		SQLINTEGER	indicator[MAX_SELECT_LIST_SIZE];
//#endif
		column 		col[MAX_SELECT_LIST_SIZE];

		int		row;
		int		maxrow;
		int		totalrows;
		int		rownumber;

		strstream	*errormsg;

		odbcconnection	*odbcconn;
};

class odbcconnection : public connection {
	friend class odbccursor;
	private:
		int	getNumberOfConnectStringVars();
		void	handleConnectString();
		int	logIn();
		cursor	*initCursor();
		void	deleteCursor(cursor *curs);
		void	logOut();
#if (ODBCVER>=0x0300)
		unsigned short	autoCommitOn();
		unsigned short	autoCommitOff();
		int	commit();
		int	rollback();
#endif
		int	ping();
		char	*identify();

		long	erg;
		SQLHENV	env;
		SQLHDBC	dbc;

		char	*dsn;
		char	*user;
		char	*password;
		int	autocommit;
};

#endif
