// Copyright (c) 1999-2001  David Muse
// See the file COPYING for more information

#include <oracle8connection.h>

#include <config.h>
#include <datatypes.h>

#include <iostream.h>

#include <stdlib.h>
#include <string.h>
#ifdef HAVE_STRINGS_H
	#include <strings.h>
#endif

oracle8connection::oracle8connection() {

	oraclehomeenv=NULL;
	oraclesidenv=NULL;
	twotaskenv=NULL;

	statementmode=OCI_DEFAULT;

#ifdef OCI_ATTR_PROXY_CREDENTIALS
	newsession=NULL;
#endif

	// init OCI
	OCIInitialize(OCI_DEFAULT,(dvoid *)0, 
			(dvoid*(*)(dvoid *,size_t))0,
			(dvoid*(*)(dvoid *,dvoid *,size_t))0, 
			(void(*)(dvoid *,dvoid *))0);

	// init the environment
	OCIEnvInit((OCIEnv **)&env,OCI_DEFAULT,(size_t)0,(dvoid **)0); 

	// allocate an error handle
	OCIHandleAlloc((dvoid *)env,(dvoid **)&err,OCI_HTYPE_ERROR,
				(size_t)0,(dvoid **)0);

	// allocate a server handle
	OCIHandleAlloc((dvoid *)env,(dvoid **)&srv,OCI_HTYPE_SERVER,
				(size_t)0,(dvoid **)0);

	// allocate a service context handle
	OCIHandleAlloc((dvoid *)env,(dvoid **)&svc,OCI_HTYPE_SVCCTX,
				(size_t)0,(dvoid **)0);


}

oracle8connection::~oracle8connection() {

	// free the service, server and error handles
	OCIHandleFree(svc,OCI_HTYPE_SVCCTX);
	OCIHandleFree(srv,OCI_HTYPE_SERVER);
	OCIHandleFree(err,OCI_HTYPE_ERROR);

	if (oraclehomeenv) {
		delete[] oraclehomeenv;
	}
	if (oraclesidenv) {
		delete[] oraclesidenv;
	}
	if (twotaskenv) {
		delete[] twotaskenv;
	}
}

int	oracle8connection::getNumberOfConnectStringVars() {
	return NUM_CONNECT_STRING_VARS;
}

void	oracle8connection::handleConnectString() {
	user=connectStringValue("user");
	password=connectStringValue("password");
	sid=connectStringValue("oracle_sid");
	home=connectStringValue("oracle_home");
	char	*autocom=connectStringValue("autocommit");
	setAutoCommitBehavior((autocom && !strcasecmp(autocom,"yes")));
}

int	oracle8connection::logIn() {

	// handle ORACLE_HOME
	if (home) {
		oraclehomeenv=new char[strlen(home)+13];
		sprintf(oraclehomeenv,"ORACLE_HOME=%s",home);
		if (!setEnv("ORACLE_HOME",home,oraclehomeenv)) {
			cerr << "Failed to set ORACLE_HOME environment variable." << ENDL;
			return 0;
		}
	} else {
		if (!getenv("ORACLE_HOME")) {
			cerr << "No ORACLE_HOME environment variable set or specified in connect string." << ENDL;
			return 0;
		}
	}

	// handle ORACLE_SID
	if (sid) {
		oraclesidenv=new char[strlen(sid)+12];
		sprintf(oraclesidenv,"ORACLE_SID=%s",sid);
		if (!setEnv("ORACLE_SID",sid,oraclesidenv)) {
			cerr << "Failed to set ORACLE_SID environment variable." << ENDL;
			return 0;
		}
	} else {
		if (!getenv("ORACLE_SID")) {
			cerr << "No ORACLE_SID environment variable set or specified in connect string." << ENDL;
			return 0;
		}
	}

	// handle TWO_TASK
	if (sid) {
		twotaskenv=new char[strlen(sid)+10];
		sprintf(twotaskenv,"TWO_TASK=%s",sid);
		if (!setEnv("TWO_TASK",sid,twotaskenv)) {
			cerr << "Failed to set TWO_TASK environment variable." << ENDL;
			return 0;
		}
	} else {
		if (!getenv("TWO_TASK")) {
			cerr << "No TWO_TASK environment variable set or specified in connect string." << ENDL;
			return 0;
		}
	}

	// attach to the server
	if (OCIServerAttach(srv,err,(text *)sid,strlen(sid),0)!=OCI_SUCCESS) {
		return 0;
	}

	// attach the server to the service
	OCIAttrSet((dvoid *)svc,OCI_HTYPE_SVCCTX,
				(dvoid *)srv,(ub4)0,
				OCI_ATTR_SERVER,(OCIError *)err);

	// allocate a session handle
	OCIHandleAlloc((dvoid *)env,(dvoid **)&session,(ub4)OCI_HTYPE_SESSION,
				(size_t)0,(dvoid **)0);

	// set username and password
	OCIAttrSet((dvoid *)session,(ub4)OCI_HTYPE_SESSION,
				(dvoid *)user,(ub4)strlen(user),
				(ub4)OCI_ATTR_USERNAME,err);
	OCIAttrSet((dvoid *)session,(ub4)OCI_HTYPE_SESSION,
				(dvoid *)password,(ub4)strlen(password),
				(ub4)OCI_ATTR_PASSWORD,err);

	// start a session
	if (OCISessionBegin(svc,err,session,
			OCI_CRED_RDBMS,(ub4)OCI_DEFAULT)!=OCI_SUCCESS) {

		// get the error message from oracle
		text	message[512];
		for (int i=0; i<512; i++) {
			message[i]=(char)NULL;
		}
		sb4	errcode;
		OCIErrorGet((dvoid *)err,1,(text *)0,&errcode,
				message,sizeof(message),OCI_HTYPE_ERROR);
		message[511]=(char)NULL;
		cerr << message << ENDL;

		return 0;
	}

	// attach the session to the service
	OCIAttrSet((dvoid *)svc,(ub4)OCI_HTYPE_SVCCTX,
				(dvoid *)session,(ub4)0,
				(ub4)OCI_ATTR_SESSION,err);

	// allocate a transaction handle
	OCIHandleAlloc((dvoid *)env,(dvoid **)&trans,OCI_HTYPE_TRANS,0,0);

	// attach the transaction to the service
	OCIAttrSet((dvoid *)svc,OCI_HTYPE_SVCCTX,
				(dvoid *)trans,(ub4)0,
				(ub4)OCI_ATTR_TRANS,err);
	return 1;
}

cursor	*oracle8connection::initCursor() {
	return (cursor *)new oracle8cursor((connection *)this);
}

void	oracle8connection::deleteCursor(cursor *curs) {
	delete (oracle8cursor *)curs;
}

void	oracle8connection::logOut() {

#ifdef OCI_ATTR_PROXY_CREDENTIALS
	if (newsession) {
		OCISessionEnd(svc,err,newsession,OCI_DEFAULT);
		OCIHandleFree(newsession,OCI_HTYPE_SESSION);
	}
#endif
	OCIHandleFree(trans,OCI_HTYPE_TRANS);
	OCISessionEnd(svc,err,session,OCI_DEFAULT);
	OCIHandleFree(session,OCI_HTYPE_SESSION);
	OCIServerDetach(srv,err,OCI_DEFAULT);
}

#ifdef OCI_ATTR_PROXY_CREDENTIALS
int	oracle8connection::changeUser(char *newuser, char *newpassword) {

	// delete any previously existing "newsessions"
	if (newsession) {
		OCISessionEnd(svc,err,newsession,OCI_DEFAULT);
		OCIHandleFree(newsession,OCI_HTYPE_SESSION);
		newsession=NULL;
	}

	// create a session handle for the new user
	if (OCIHandleAlloc((dvoid *)env,(dvoid **)&newsession,
				(ub4)OCI_HTYPE_SESSION,
				(size_t)0,(dvoid **)0)!=OCI_SUCCESS) {
		return 0;
	}

	// set the user name
	OCIAttrSet((dvoid *)newsession,(ub4)OCI_HTYPE_SESSION,
				(dvoid *)newuser,(ub4)strlen(newuser),
				(ub4)OCI_ATTR_USERNAME,err);

	// don't set the password, use the proxy
	OCIAttrSet((dvoid *)newsession,(ub4)OCI_HTYPE_SESSION,
				(dvoid *)session,(ub4)0,
				(ub4)OCI_ATTR_PROXY_CREDENTIALS,err);

	// start the session
	if (OCISessionBegin(svc,err,newsession,
			OCI_CRED_PROXY,(ub4)OCI_DEFAULT)!=OCI_SUCCESS) {
		return 0;
	}

	// switch to the new session
	if (OCIAttrSet((dvoid *)svc,(ub4)OCI_HTYPE_SVCCTX,
				(dvoid *)newsession,(ub4)0,
				(ub4)OCI_ATTR_SESSION,err)!=OCI_SUCCESS) {
		return 0;
	}
	return 1;
}
#endif

unsigned short	oracle8connection::autoCommitOn() {
	statementmode=OCI_COMMIT_ON_SUCCESS;
	return 1;
}

unsigned short	oracle8connection::autoCommitOff() {
	statementmode=OCI_DEFAULT;
	return 1;
}

int	oracle8connection::commit() {
	return (OCITransCommit(svc,err,OCI_DEFAULT)==OCI_SUCCESS);
}

int	oracle8connection::rollback() {
	return (OCITransRollback(svc,err,OCI_DEFAULT)==OCI_SUCCESS);
}

int	oracle8connection::ping() {
	int	retval=0;
	oracle8cursor	*cur=new oracle8cursor(this);
	if (cur->openCursor(-1) && 
		cur->prepareQuery("select 1 from dual",18) && 
		cur->executeQuery("select 1 from dual",18)) {
		cur->cleanUpData();
		cur->closeCursor();
		retval=1;
	}
	delete cur;
	return retval;
}

char	*oracle8connection::identify() {
	return "oracle8";
}

oracle8cursor::oracle8cursor(connection *conn) : cursor(conn) {
	prepared=0;
	errormessage=NULL;
	oracle8conn=(oracle8connection *)conn;
	fetchatonce=FETCH_AT_ONCE;
}

oracle8cursor::~oracle8cursor() {
	if (errormessage) {
		delete errormessage;
	}
}

int	oracle8cursor::openCursor(int id) {

	// allocate a cursor handle
	if (OCIHandleAlloc((dvoid *)oracle8conn->env,(dvoid **)&stmt,
			OCI_HTYPE_STMT,(size_t)0,(dvoid **)0)!=OCI_SUCCESS) {
		return 0;
	}

	// set the number of rows to prefetch
	if (OCIAttrSet((dvoid *)stmt,OCI_HTYPE_STMT,
				(dvoid *)&fetchatonce,(ub4)0,
				OCI_ATTR_PREFETCH_ROWS,
				(OCIError *)oracle8conn->err)) {
		return 0;
	}
	return 1;
}

int	oracle8cursor::closeCursor() {

	if (OCIHandleFree(stmt,OCI_HTYPE_STMT)!=OCI_SUCCESS) {
		return 0;
	}
	return 1;
}

int	oracle8cursor::prepareQuery(char *query, long length) {

	// keep a pointer to the query and length in case it needs to be 
	// reprepared later
	this->query=query;
	this->length=length;

	// reset the statement type
	stmttype=0;

	// prepare the query
	if (OCIStmtPrepare(stmt,oracle8conn->err,(text *)query,(ub4)length,
			(ub4)OCI_NTV_SYNTAX,(ub4)OCI_DEFAULT)!=OCI_SUCCESS) {
		return 0;
	}
	return 1;
}

void	oracle8cursor::checkRePrepare() {

	// Oracle8 appears to have a bug.
	// You can prepare, bind, execute, rebind, re-execute, etc. with
	// selects, but not with DML, it has to be re-prepared.  What a drag.
	if (!prepared && stmttype && stmttype!=OCI_STMT_SELECT) {
		cleanUpData();
		prepareQuery(query,length);
		prepared=1;
	}
}

int	oracle8cursor::inputBindValue(char *variable, 
						unsigned short variablesize,
						char *value, 
						unsigned short valuesize,
						short *isnull) {
	checkRePrepare();

	OCIBind	*bindpp=NULL;

	// the size of the value must include the terminating NULL
	if (isNumber(variable+1,variablesize-1)) {
		if (!atoi(variable+1)) {
			return 0;
		}
		if (OCIBindByPos(stmt,&bindpp,oracle8conn->err,
				(ub4)atoi(variable+1),
				(dvoid *)value,(sb4)valuesize+1,
				NULL_TERMINATED_STRING,
				(dvoid *)isnull,(ub2 *)0,(ub2 *)0,0,(ub4 *)0,
				OCI_DEFAULT)!=OCI_SUCCESS) {
			return 0;
		}
	} else {
		if (OCIBindByName(stmt,&bindpp,oracle8conn->err,
				(text *)variable,(sb4)variablesize,
				(dvoid *)value,(sb4)valuesize+1,
				NULL_TERMINATED_STRING,
				(dvoid *)isnull,(ub2 *)0,(ub2 *)0,0,(ub4 *)0,
				OCI_DEFAULT)!=OCI_SUCCESS) {
			return 0;
		}
	}
	return 1;
}


int	oracle8cursor::inputBindValue(char *variable, 
						unsigned short variablesize,
						unsigned long *value) {
	checkRePrepare();

	OCIBind	*bindpp=NULL;

	if (isNumber(variable+1,variablesize-1)) {
		if (!atoi(variable+1)) {
			return 0;
		}
		if (OCIBindByPos(stmt,&bindpp,oracle8conn->err,
				(ub4)atoi(variable+1),
				(dvoid *)value,(sb4)sizeof(long),
				LONG_BIND_TYPE,
				(dvoid *)0,(ub2 *)0,(ub2 *)0,0,(ub4 *)0,
				OCI_DEFAULT)!=OCI_SUCCESS) {
			return 0;
		}
	} else {
		if (OCIBindByName(stmt,&bindpp,oracle8conn->err,
				(text *)variable,(sb4)variablesize,
				(dvoid *)value,(sb4)sizeof(long),
				LONG_BIND_TYPE,
				(dvoid *)0,(ub2 *)0,(ub2 *)0,0,(ub4 *)0,
				OCI_DEFAULT)!=OCI_SUCCESS) {
			return 0;
		}
	}
	return 1;
}


int	oracle8cursor::inputBindValue(char *variable, 
						unsigned short variablesize,
						double *value,
						unsigned short precision,
						unsigned short scale) {
	checkRePrepare();

	OCIBind	*bindpp=NULL;

	if (isNumber(variable+1,variablesize-1)) {
		if (!atoi(variable+1)) {
			return 0;
		}
		if (OCIBindByPos(stmt,&bindpp,oracle8conn->err,
				(ub4)atoi(variable+1),
				(dvoid *)value,(sb4)sizeof(double),
				DOUBLE_BIND_TYPE,
				(dvoid *)0,(ub2 *)0,(ub2 *)0,0,(ub4 *)0,
				OCI_DEFAULT)!=OCI_SUCCESS) {
			return 0;
		}
	} else {
		if (OCIBindByName(stmt,&bindpp,oracle8conn->err,
				(text *)variable,(sb4)variablesize,
				(dvoid *)value,(sb4)sizeof(double),
				DOUBLE_BIND_TYPE,
				(dvoid *)0,(ub2 *)0,(ub2 *)0,0,(ub4 *)0,
				OCI_DEFAULT)!=OCI_SUCCESS) {
			return 0;
		}
	}
	return 1;
}

int	oracle8cursor::outputBindValue(char *variable, 
						unsigned short variablesize,
						char *value,
						unsigned short valuesize, 
						short *isnull) {
	checkRePrepare();

	OCIBind	*bindpp=NULL;
	if (isNumber(variable+1,variablesize-1)) {
		if (!atoi(variable+1)) {
			return 0;
		}
		if (OCIBindByPos(stmt,&bindpp,oracle8conn->err,
				(ub4)atoi(variable+1),
				(dvoid *)value,
				(sb4)valuesize,
				NULL_TERMINATED_STRING,
				(dvoid *)isnull,(ub2 *)0,(ub2 *)0,0,(ub4 *)0,
				OCI_DEFAULT)!=OCI_SUCCESS) {
			return 0;
		}
	} else {
		if (OCIBindByName(stmt,&bindpp,oracle8conn->err,
				(text *)variable,(sb4)variablesize,
				(dvoid *)value,
				(sb4)valuesize,
				NULL_TERMINATED_STRING,
				(dvoid *)isnull,(ub2 *)0,(ub2 *)0,0,(ub4 *)0,
				OCI_DEFAULT)!=OCI_SUCCESS) {
			return 0;
		}
	}
	return 1;
}

int	oracle8cursor::executeQuery(char *query, long length) {

	// initialize the column count
	ncols=0;

	// get the type of the query (select, insert, update, etc...)
	if (OCIAttrGet(stmt,OCI_HTYPE_STMT,
			(dvoid *)&stmttype,(ub4 *)NULL,
			OCI_ATTR_STMT_TYPE,oracle8conn->err)!=OCI_SUCCESS) {
		return 0;
	}

	// set up how many times to iterate; 0 for selects, 1 for non-selects
	ub4	iters=1;
	if (stmttype==OCI_STMT_SELECT) {
		iters=0;
	}

	// initialize row counters
	row=0;
	maxrow=0;
	totalrows=0;

	// execute the query
	if (OCIStmtExecute(oracle8conn->svc,stmt,oracle8conn->err,iters,
				(ub4)0,NULL,NULL,
				oracle8conn->statementmode)!=OCI_SUCCESS) {
		return 0;
	}

	// reset the prepared flag
	prepared=0;

	// if the query is a select, describe/define it
	if (stmttype==OCI_STMT_SELECT) {

		// get the column count
		if (OCIAttrGet((dvoid *)stmt,OCI_HTYPE_STMT,
				(dvoid *)&ncols,(ub4 *)NULL,
				OCI_ATTR_PARAM_COUNT,
				oracle8conn->err)!=OCI_SUCCESS) {
			return 0;
		}

		// run through the columns...
		for (int i=0; i<ncols; i++) {

			// get the entire column definition
			if (OCIParamGet(stmt,OCI_HTYPE_STMT,
				oracle8conn->err,
				(dvoid **)&desc[i].paramd,i+1)!=OCI_SUCCESS) {
				return 0;
			}

			// get the column name
			if (OCIAttrGet((dvoid *)desc[i].paramd,OCI_DTYPE_PARAM,
				(dvoid **)&desc[i].buf,
				(ub4 *)&desc[i].buflen,
				(ub4)OCI_ATTR_NAME,
				oracle8conn->err)!=OCI_SUCCESS) {
				return 0;
			}

			// get the column type
			if (OCIAttrGet((dvoid *)desc[i].paramd,OCI_DTYPE_PARAM,
				(dvoid *)&desc[i].dbtype,(ub4 *)NULL,
				(ub4)OCI_ATTR_DATA_TYPE,
				oracle8conn->err)!=OCI_SUCCESS) {
				return 0;
			}

			// is the column a LOB?
			if (desc[i].dbtype==BLOB_TYPE ||
				desc[i].dbtype==CLOB_TYPE ||
				desc[i].dbtype==BFILE_TYPE) {

				// return 0 for the size of lobs
				desc[i].dbsize=0;

				// allocate a lob descriptor
				if (OCIDescriptorAlloc(
						(void *)oracle8conn->env,
						(void **)&def_lob[i],
						OCI_DTYPE_LOB,0,0)) {
					return 0;
				}

				// define the column as a lob
				if (OCIDefineByPos(stmt,&def[i],
					oracle8conn->err,
					i+1,
					(dvoid *)def_lob[i],
					(sb4)-1,
					desc[i].dbtype,
					(dvoid *)0,
					0,
					(ub2 *)0,
					OCI_DEFAULT)!=OCI_SUCCESS) {
					return 0;
				}

			} else {

				// get the column size
				if (OCIAttrGet((dvoid *)desc[i].paramd,
					OCI_DTYPE_PARAM,
					(dvoid *)&desc[i].dbsize,(ub4 *)NULL,
					(ub4)OCI_ATTR_DATA_SIZE,
					oracle8conn->err)!=OCI_SUCCESS) {
					return 0;
				}

				// if the column is not a LOB, define it,
				// translated to a NULL terminated string
				if (OCIDefineByPos(stmt,&def[i],
					oracle8conn->err,
					i+1,
					(dvoid *)def_buf[i],
					(sb4)MAX_ITEM_BUFFER_SIZE,
					SQLT_STR,
					(dvoid *)def_indp[i],
					(ub2 *)def_col_retlen[i],
					def_col_retcode[i],
					OCI_DEFAULT)!=OCI_SUCCESS) {
					return 0;
				}

				// set the lob member to NULL
				def_lob[i]=NULL;
			}
		}
	}
	return 1;
}

int	oracle8cursor::queryIsNotSelect() {

	if (stmttype==OCI_STMT_SELECT) {
		return 0;
	}
	return 1;
}

int	oracle8cursor::queryIsCommitOrRollback() {

	// apparantly in OCI8, the cursor type gets 
	// set to 0 for both commits and rollbacks
	if (stmttype==0) {
		return 1;
	}
	return 0;
}

char	*oracle8cursor::getErrorMessage(int *liveconnection) {

	// get the message from oracle
	text	message[512];
	for (int i=0; i<512; i++) {
		message[i]=(char)NULL;
	}
	sb4	errcode;
	OCIErrorGet((dvoid *)oracle8conn->err,1,
			(text *)0,&errcode,
			message,sizeof(message),
			OCI_HTYPE_ERROR);
	message[511]=(char)NULL;

	// check for dead connection
	if (errcode==3114 || errcode==3113) {
		*liveconnection=0;
	} else {
		*liveconnection=1;
	}

	// only return an error message if the error wasn't a dead database
	if (errormessage) {
		delete errormessage;
	}
	errormessage=new strstream();
	if (*liveconnection) {
		*errormessage << message;
	}
	*errormessage << ENDS;

	return errormessage->str();
}

void	oracle8cursor::returnRowCounts() {

	// get the row count
	ub4	rows;
	OCIAttrGet(stmt,OCI_HTYPE_STMT,
			(dvoid *)&rows,(ub4 *)NULL,
			OCI_ATTR_ROW_COUNT,oracle8conn->err);

	// don't know how to get affected row count in OCI8
	conn->sendRowCounts((long)-1,(long)rows);
}

void	oracle8cursor::returnColumnCount() {
	conn->sendColumnCount(ncols);
}

void	oracle8cursor::returnColumnInfo() {

	// a useful variable
	int	type;

	// for each column...
	for (int i=0; i<ncols; i++) {

		// set column type
		if (desc[i].dbtype==VARCHAR2_TYPE) {
			type=VARCHAR2_DATATYPE;
		} else if (desc[i].dbtype==NUMBER_TYPE) {
			type=NUMBER_DATATYPE;
		} else if (desc[i].dbtype==LONG_TYPE) {
			type=LONG_DATATYPE;
		} else if (desc[i].dbtype==ROWID_TYPE) {
			type=ROWID_DATATYPE;
		} else if (desc[i].dbtype==DATE_TYPE) {
			type=DATE_DATATYPE;
		} else if (desc[i].dbtype==RAW_TYPE) {
			type=RAW_DATATYPE;
		} else if (desc[i].dbtype==LONG_RAW_TYPE) {
			type=LONG_RAW_DATATYPE;
		} else if (desc[i].dbtype==CHAR_TYPE) {
			type=CHAR_DATATYPE;
		} else if (desc[i].dbtype==MLSLABEL_TYPE) {
			type=MLSLABEL_DATATYPE;
		} else if (desc[i].dbtype==BLOB_TYPE) {
			type=BLOB_DATATYPE;
		} else if (desc[i].dbtype==CLOB_TYPE) {
			type=CLOB_DATATYPE;
		} else if (desc[i].dbtype==BFILE_TYPE) {
			type=BFILE_DATATYPE;
		} else {
			type=UNKNOWN_DATATYPE;
		}

		// send the column definition
		conn->sendColumnDefinition((char *)desc[i].buf,
					(short)desc[i].buflen,
					type,
					(int)desc[i].dbsize);
	}
}

int	oracle8cursor::noRowsToReturn() {
	if (stmttype!=OCI_STMT_SELECT) {
		return 1;
	}
	return 0;
}

int	oracle8cursor::skipRow() {
	if (fetchRow()) {
		row++;
		return 1;
	}
	return 0;
}

int	oracle8cursor::fetchRow() {
	if (row==FETCH_AT_ONCE) {
		row=0;
	}
	if (row>0 && row==maxrow) {
		return 0;
	}
	if (row==0) {
		OCIStmtFetch(stmt,oracle8conn->err,fetchatonce,
					OCI_FETCH_NEXT,OCI_DEFAULT);
		ub4	currentrow;
		OCIAttrGet(stmt,OCI_HTYPE_STMT,
				(dvoid *)&currentrow,(ub4 *)NULL,
				OCI_ATTR_ROW_COUNT,oracle8conn->err);
		if (currentrow==totalrows) {
			return 0;
		}
		maxrow=currentrow-totalrows;
		totalrows=currentrow;
	}
	return 1;
}

void	oracle8cursor::returnRow() {

	for (sword col=0; col<ncols; col++) {

		// handle NULL's
		if (def_indp[col][row]) {
			conn->sendNullField();
			continue;
		}

		// in OCI8, longs are just like other datatypes, but LOBS
		// are different
		if (desc[col].dbtype==BLOB_TYPE ||
			desc[col].dbtype==CLOB_TYPE ||
			desc[col].dbtype==BFILE_TYPE) {

			// handle lob datatypes
			ub4	retlen=MAX_ITEM_BUFFER_SIZE;
			sb4	offset=1;

			conn->startSendingLong();

			sword	retval=OCI_NEED_DATA;
			while (retval==OCI_NEED_DATA) {

				// read a segment from the lob
				retval=OCILobRead(oracle8conn->svc,
						oracle8conn->err,
						def_lob[col],
						&retlen,
						offset,
						(dvoid *)&def_buf[col][row],
						MAX_ITEM_BUFFER_SIZE,
						(dvoid *)0,
				(sb4 (*)(dvoid *, CONST dvoid *, ub4,ub1))0,
						(ub2)0,
						(ub1)SQLCS_IMPLICIT);

				// send the lob segment
				conn->sendLongSegment(
						(char *)def_buf[col][row],
						(long)retlen);

				offset=offset+retlen;
			}

			conn->endSendingLong();

			continue;

		}

		// handle normal datatypes
		conn->sendField((char *)def_buf[col][row],
					(int)def_col_retlen[col][row]);
	}

	// increment the row counter
	row++;
}

void	oracle8cursor::cleanUpData() {

	// OCI8 version of ocan(), but since it uses OCIStmtFetch we
	// only want to run it if the statement was a select
	if (stmttype==OCI_STMT_SELECT) {
		OCIStmtFetch(stmt,oracle8conn->err,0,
				OCI_FETCH_NEXT,OCI_DEFAULT);
	}

	// free resources
	for (int i=0; i<ncols; i++) {
		if (def_lob[i]) {
			OCIHandleFree(def_lob[i],OCI_DTYPE_LOB);
		}
		if (def[i]) {
			OCIHandleFree(def[i],OCI_HTYPE_DEFINE);
		}
	}
}
