// Copyright (c) 2000-2001  David Muse
// See the file COPYING for more information

#ifndef ORACLECONNECTION_H
#define ORACLECONNECTION_H

#define FETCH_AT_ONCE		10
#define MAX_SELECT_LIST_SIZE	256
#define MAX_ITEM_BUFFER_SIZE	4096

#define NUM_CONNECT_STRING_VARS 3

#include <connection.h>

#include <strstream.h>
#include <fstream.h>

extern "C" {
	#include <oci.h>

	#define VARCHAR2_TYPE 1
	#define	NUMBER_TYPE 2
	#define	LONG_TYPE 8
	#define ROWID_TYPE 11
	#define DATE_TYPE 12
	#define RAW_TYPE 23
	#define LONG_RAW_TYPE 24
	#define CHAR_TYPE 96
	#define MLSLABEL_TYPE 105
	#define CLOB_TYPE 112
	#define BLOB_TYPE 113
	#define BFILE_TYPE 114

	#define LONG_BIND_TYPE 3
	#define DOUBLE_BIND_TYPE 4
	#define NULL_TERMINATED_STRING 5
}

struct describe {
	OCIParam	*paramd;
	sb4		dbsize;
	sb2		dbtype;
	text		*buf;
	sb4		buflen;
};

class oracle8connection;

class oracle8cursor : public cursor {
	friend class oracle8connection;
	private:
			oracle8cursor(connection *conn);
			~oracle8cursor();
		int	openCursor(int id);
		int	closeCursor();
		int	prepareQuery(char *query, long length);
		int	inputBindValue(char *variable, 
					unsigned short variablesize,
					char *value, 
					unsigned short valuesize,
					short *isnull);
		int	inputBindValue(char *variable, 
					unsigned short variablesize,
					unsigned long *value);
		int	inputBindValue(char *variable, 
					unsigned short variablesize,
					double *value,
					unsigned short precision,
					unsigned short scale);
		int	outputBindValue(char *variable, 
					unsigned short variablesize,
					char *value,
					unsigned short valuesize,
					short *isnull);
		int	executeQuery(char *query, long length);
		int	queryIsNotSelect();
		int	queryIsCommitOrRollback();
		char	*getErrorMessage(int *liveconnection);
		void	returnRowCounts();
		void	returnColumnCount();
		void	returnColumnInfo();
		int	noRowsToReturn();
		int	skipRow();
		int	fetchRow();
		void	returnRow();
		void	cleanUpData();

		void	checkRePrepare();

		OCIStmt		*stmt;
		ub2		stmttype;
		sword		ncols;
		strstream	*errormessage;

		describe	desc[MAX_SELECT_LIST_SIZE];
		OCIDefine	*def[MAX_SELECT_LIST_SIZE];
		OCILobLocator	*def_lob[MAX_SELECT_LIST_SIZE];
		ub1		def_buf[MAX_SELECT_LIST_SIZE]
					[FETCH_AT_ONCE][MAX_ITEM_BUFFER_SIZE];
		sb2		def_indp[MAX_SELECT_LIST_SIZE][FETCH_AT_ONCE];
		ub2		def_col_retlen[MAX_SELECT_LIST_SIZE]
						[FETCH_AT_ONCE];
		ub2		def_col_retcode[MAX_SELECT_LIST_SIZE]
						[FETCH_AT_ONCE];

		int		row;
		int		maxrow;
		int		totalrows;

		int		fetchatonce;

		char		*query;
		int		length;
		int		prepared;

		oracle8connection	*oracle8conn;
};
	
class oracle8connection : public connection {
	friend class oracle8cursor;
	public:
				oracle8connection();
				~oracle8connection();
	private:
		int	getNumberOfConnectStringVars();
		void	handleConnectString();
		int	logIn();
		cursor	*initCursor();
		void	deleteCursor(cursor *curs);
		void	logOut();
#ifdef OCI_ATTR_PROXY_CREDENTIALS
		int		changeUser(char *newuser, char *newpassword);
#endif
		unsigned short	autoCommitOn();
		unsigned short	autoCommitOff();
		int	commit();
		int	rollback();
		int	ping();
		char	*identify();

		ub4		statementmode;

		OCIEnv		*env;
		OCIServer	*srv;
		OCIError	*err;
		OCISvcCtx	*svc;
		OCISession	*session;
		OCITrans	*trans;

#ifdef OCI_ATTR_PROXY_CREDENTIALS
		OCISession	*newsession;
#endif

		char		*user;
		char		*password;
		char		*home;
		char		*sid;
		int		autocommit;

		char		*oraclehomeenv;
		char		*oraclesidenv;
		char		*twotaskenv;
};

#endif
