// Copyright (c) 1999-2001  David Muse
// See the file COPYING for more information

#include <config.h>
#include <defaults.h>
#include <configfile.h>
#include <connection.h>

#include <rudiments/permissions.h>

#include <iostream.h>
#include <iomanip.h>

#include <math.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ipc.h>
#include <stdlib.h>
#include <time.h>
#ifdef HAVE_UNISTD_H
	#include <unistd.h>
#endif
#include <string.h>
#ifdef HAVE_STRINGS_H
	#include <strings.h>
#endif

#include <defines.h>


connection::connection() : daemonprocess(), serversocketpool() {

	tmpdirlen=strlen(TMP_DIR);

	init=0;
	semset=NULL;
	idmemory=NULL;
	sockseqname=NULL;

	unixsocket=new char[tmpdirlen+23];
	sprintf(unixsocket,"%s/",TMP_DIR);
	unixsocketptr=unixsocket+tmpdirlen+1;
	serversockun=NULL;
	serversockin=NULL;
	handoffsockun=NULL;

	inetport=0;
	cursorcount=0;
	authonconn=0;
	authc=NULL;
	lastuserbuffer[0]=(char)NULL;
	lastpasswordbuffer[0]=(char)NULL;
	lastauthsuccess=0;

	handoffsockun=NULL;
	autocommit=0;
	checkautocommit=0;
	performautocommit=0;

	#ifdef SERVER_DEBUG
	debug=0;
	debugfile=NULL;
	#endif

	cmdl=NULL;
}

connection::~connection() {

	#ifdef SERVER_DEBUG
	debugPrint(0,"deleting the cmdl...");
	#endif
	if (cmdl) {
		delete cmdl;
	}

	#ifdef SERVER_DEBUG
	debugPrint(0,"deleting authc...");
	#endif
	if (authc) {
		delete authc;
	}
	#ifdef SERVER_DEBUG
	debugPrint(0,"done deleting authc");
	#endif


	#ifdef SERVER_DEBUG
	debugPrint(0,"deleting idmemory...");
	#endif
	if (idmemory) {
		delete idmemory;
	}
	#ifdef SERVER_DEBUG
	debugPrint(0,"done deleting idmemory");
	#endif


	#ifdef SERVER_DEBUG
	debugPrint(0,"deleting semset...");
	#endif
	if (semset) {
		delete semset;
	}
	#ifdef SERVER_DEBUG
	debugPrint(0,"done deleting semset");
	#endif


	#ifdef SERVER_DEBUG
	debugPrint(0,"deleting sockseqname...");
	#endif
	if (sockseqname) {
		delete[] sockseqname;
	}
	#ifdef SERVER_DEBUG
	debugPrint(0,"done deleting sockseqname");
	#endif


	#ifdef SERVER_DEBUG
	debugPrint(0,"deleting unixsocket...");
	#endif
	if (unixsocket) {
		unlink(unixsocket);
		delete[] unixsocket;
	}
	#ifdef SERVER_DEBUG
	debugPrint(0,"done deleting unixsocket");
	#endif

	#ifdef SERVER_DEBUG
	closeDebugFile();
	#endif
}

int	connection::initConnection(int argc, char **argv) {

	init=1;

	// process the command line
	cmdl=new commandline(argc,argv);

	// get the id from the command line
	id=cmdl->value("-id");
	if (!id[0]) {
		id=DEFAULT_ID;
		cerr << "Warning: using default id." << ENDL;
	}

	// check for pid file
	char	*pidfile=new char[tmpdirlen+15+strlen(id)+1];
	sprintf(pidfile,"%s/sqlr-listener-%s",TMP_DIR,id);
	if (!checkForPidFile(pidfile)) {
		cerr << ENDL << "sqlr-connection error:" << ENDL;
		cerr << "	The file " << TMP_DIR;
		cerr << "/sqlr-listener-" << id;
		cerr << " was not found." << ENDL;
		cerr << "	This usually means that the sqlr-listener ";
		cerr << "is not running." << ENDL;
		cerr << "	The sqlr-listener must be running ";
		cerr << "for the sqlr-connection to start." << ENDL << ENDL;
		delete[] pidfile;
		return 0;
	}
	delete[] pidfile;

	// get the config file from the command line
	char	*config=cmdl->value("-config");
	if (!config[0]) {
		config=DEFAULT_CONFIG_FILE;
	}

	// get the connection id from the command line
	char	*connectionid=cmdl->value("-connectionid");

	// get the time to live from the command line
	ttl=atoi(cmdl->value("-ttl"));

	// parse the config file(s)
	configfile		*cfgfile=NULL;
	connectstringnode	*connectstringlist=NULL;
	dynamicscaling=0;
	cfgfile=new configfile();
	if (cfgfile->parse(config,id)) {

		// run as user/group specified in the config file
		char	*runasuser=cfgfile->getRunAsUser();
		char	*runasgroup=cfgfile->getRunAsGroup();
		if (runasuser[0] && runasgroup[0]) {
			runAsUser(runasuser);
			runAsGroup(runasgroup);
		}

		connectstringlist=cfgfile->getConnectStrings();

		// commit or rollback at end of session?
		if (!strcmp(cfgfile->getEndOfSession(),"rollback")) {
			eoscommit=0;
		} else {
			eoscommit=1;
		}

		// how long before a suspended session times out
		sessiontimeout=cfgfile->getSessionTimeout();
		if (!sessiontimeout) {
			sessiontimeout=60;
		}

		// see if dynamic scaling will be used
		if (cfgfile->getMaxQueueLength()>=0 &&
			cfgfile->getMaxConnections()>
				cfgfile->getConnections() &&
			cfgfile->getGrowBy()>0 && cfgfile->getTtl()>0) {
			dynamicscaling=1;
		}

		// how many cursors should we use, create an array for them
		cursorcount=cfgfile->getCursors();
		if (cursorcount==0) {
			cursorcount=DEFAULT_CURSOR_COUNT;
		}
		cur=new cursor *[cursorcount];
		for (int i=0; i<cursorcount; i++) {
			cur[i]=NULL;
		}

		// Get where authentication will be performed. If the 
		// authentication attribute is not "listener" or "database" 
		// then authenticate here.
		char	*authtier=cfgfile->getAuthTier();
		if (strcmp(authtier,"listener")) {
			if (!strcmp(authtier,"database")) {
				authonconn=2;
			} else {
				authc=new authenticator(cfgfile);
				authonconn=1;
			}
		}

		// determine whether to listen on inet and/or unix ports
		listenoninet=0;
		if (cfgfile->getPort()) {
			listenoninet=1;
		}
		listenonunix=0;
		char	*up=cfgfile->getUnixPort();
		if (up && up[0]) {
			listenonunix=1;
		}

		// if the connection is configured not to listen on either
		// inet or unix ports, listen on the inet port
		if (!listenonunix && !listenoninet) {
			listenoninet=1;
		}

		// get handoff method
		char	*handoffmethod=cfgfile->getHandOff();
		passdescriptor=0;
		if (!strcmp(handoffmethod,"pass")) {
			passdescriptor=1;
		}

		#ifdef SERVER_DEBUG
		openDebugFile();
		#endif
	}

	// if a connectionid was supplied, find and use it's connect string
	connectstringnode	*currentnode;
	char	*connectstring=NULL;
	if (connectionid[0]) {
		currentnode=connectstringlist;
		while (currentnode) {
			if (!strcmp(connectionid,
					currentnode->getConnectionId())) {
				connectstring=currentnode->getString();
				break;
			}
			currentnode=currentnode->getNext();
		}
	}

	// if no connectstring was found or supplied, if there's only
	// one, use it, otherwise use the default
	if (!connectstring) {
		if (connectstringlist && !connectstringlist->getNext()) {
			connectstring=connectstringlist->getString();
		} else {
			connectstring=DEFAULT_CONNECTSTRING;
			cerr << "Warning: using default connectstring." << ENDL;
		}
	}

	// initialize the database up/down filename
	updown=new char[tmpdirlen+1+strlen(id)+1+strlen(connectionid)+1];
	sprintf(updown,"%s/%s-%s",TMP_DIR,id,connectionid);

	// set up connect strings
	connectstringcount=getNumberOfConnectStringVars();
	connectstringvar=new strstream *[connectstringcount];
	connectstringval=new strstream *[connectstringcount];
	for (int i=0; i<connectstringcount; i++) {
		connectstringvar[i]=NULL;
		connectstringval[i]=NULL;
	}
	parseConnectString(connectstring);

	// set some useful variables
	handleConnectString();

	// initialize the shared memory segment filename
	idfilename=new char[tmpdirlen+1+strlen(id)+1];
	sprintf(idfilename,"%s/%s",TMP_DIR,id);

	// clean up
	if (cfgfile) {
		delete cfgfile;
	}

	// set the unix socket name
	getUnixSocket();

	return 1;
}

void	connection::parseConnectString(char *connectstring) {

	// handle connect string of form:
	//	 name1=value1;name2=value2;name3=value3;

	char	*ptr1=connectstring;
	char	*ptr2=connectstring;
	int	current=0;
	while (*ptr2) {
		while (*ptr2 && *ptr2!='=') {
			ptr2++;
		}
		if (!*ptr2) {
			break;
		}
		connectstringvar[current]=new strstream();
		while (ptr1!=ptr2) {
			*connectstringvar[current] << (char)*ptr1;
			ptr1++;
		}
		*connectstringvar[current] << ENDS;
		ptr1++;
		ptr2++;
		while (*ptr2 && *ptr2!=';') {
			ptr2++;
		}
		connectstringval[current]=new strstream();
		while (*ptr1 && ptr1!=ptr2) {
			*connectstringval[current] << (char)*ptr1;
			ptr1++;
		}
		*connectstringval[current] << ENDS;

		if (!*ptr2) {
			break;
		}

		current++;
		ptr1++;
		ptr2++;
	}
}

char	*connection::connectStringValue(char *variable) {

	// search for the given variable, return the given value,
	// search is case insensitive
	for (int i=0; i<connectstringcount; i++) {
		if (!connectstringvar[i]) {
			break;
		}
		if (!strcasecmp(variable,connectstringvar[i]->str())) {
			return connectstringval[i]->str();
		}
	}
	return NULL;
}

void	connection::closeConnection() {

	// decrement the connection counter
	if (dynamicscaling && semset && idmemory) {
		decrementConnectionCount();
	}

	// delete the shared memory segment name
	delete[] idfilename;

	// deregister and close the handoff socket if necessary
	if (passdescriptor && handoffsockun) {
		deRegisterForHandoff();
		delete handoffsockun;
	}

	// delete the updown file name
	delete[] updown;

	// clean up connect strings
	for (int i=0; i<connectstringcount; i++) {
		delete connectstringvar[i];
		delete connectstringval[i];
	}
	delete[] connectstringvar;
	delete[] connectstringval;


	// close the cursors
	closeCursors();


	// try to log out
	#ifdef SERVER_DEBUG
	debugPrint(0,"logging out...");
	#endif
	logOut();
	#ifdef SERVER_DEBUG
	debugPrint(0,"done logging out");
	#endif


	// clear the pool
	#ifdef SERVER_DEBUG
	debugPrint(0,"removing all sockets...");
	#endif
	removeAllSockets();
	#ifdef SERVER_DEBUG
	debugPrint(0,"done removing all sockets");
	#endif


	// close, clean up all sockets
	#ifdef SERVER_DEBUG
	debugPrint(0,"deleting unix socket...");
	#endif
	if (serversockun) {
		delete serversockun;
	}
	#ifdef SERVER_DEBUG
	debugPrint(0,"done deleting unix socket");
	#endif


	#ifdef SERVER_DEBUG
	debugPrint(0,"deleting inet socket...");
	#endif
	if (serversockin) {
		delete serversockin;
	}
	#ifdef SERVER_DEBUG
	debugPrint(0,"done deleting inet socket");
	#endif


	#ifdef SERVER_DEBUG
	debugPrint(0,"deleting handoff socket...");
	#endif
	if (handoffsockun) {
		delete handoffsockun;
	}
	#ifdef SERVER_DEBUG
	debugPrint(0,"done deleting handoff socket");
	#endif
}

void	connection::incrementConnectionCount() {

	#ifdef SERVER_DEBUG
	debugPrint(0,"incrementing connection count...");
	#endif

	// wait for access
	semset->wait(4);

	// increment the counter
	unsigned int	*connectioncount=(unsigned int *)idmemory->getPointer();
	(*connectioncount)++;

	#ifdef SERVER_DEBUG
	debugPrint(1,(long)(*connectioncount));
	#endif

	// signal the scaler
	if (dynamicscaling) {
		semset->signal(8);
	}

	// signal that others may have access
	semset->signal(4);

	#ifdef SERVER_DEBUG
	debugPrint(0,"done incrementing connection count");
	#endif
}

void	connection::decrementConnectionCount() {

	#ifdef SERVER_DEBUG
	debugPrint(0,"decrementing connection count...");
	#endif

	// wait for access
	semset->wait(4);

	// decrement the counter
	unsigned int	*connectioncount=(unsigned int *)idmemory->getPointer();
	(*connectioncount)--;

	#ifdef SERVER_DEBUG
	debugPrint(1,(long)(*connectioncount));
	#endif

	// signal that others may have access
	semset->signal(4);

	#ifdef SERVER_DEBUG
	debugPrint(0,"done decrementing connection count");
	#endif
}

void	connection::decrementSessionCount() {

	#ifdef SERVER_DEBUG
	debugPrint(0,"decrementing session count...");
	#endif

	// wait for access
	semset->wait(5);

	// decrement the counter
	unsigned int	*sessioncount=
				(unsigned int *)((long)idmemory->getPointer()+
					sizeof(unsigned int));
	(*sessioncount)--;

	#ifdef SERVER_DEBUG
	debugPrint(1,(long)(*sessioncount));
	#endif

	// signal that others may have access
	semset->signal(5);

	#ifdef SERVER_DEBUG
	debugPrint(0,"done decrementing session count");
	#endif
}

void	connection::createSharedMemoryAndSemaphores() {

	#ifdef SERVER_DEBUG
	debugPrint(0,"attaching to shared memory and semaphores");
	#endif

	// initialize shared memory segment for passing port
	#ifdef SERVER_DEBUG
	debugPrint(1,"attaching to shared memory...");
	#endif
	idmemory=new sharedmemory();
	idmemory->attach(ftok(idfilename,0));


	// initialize the announce semaphore
	#ifdef SERVER_DEBUG
	debugPrint(1,"attaching to semaphore...");
	#endif
	semset=new semaphoreset();
	semset->attach(ftok(idfilename,0),10);

	#ifdef SERVER_DEBUG
	debugPrint(0,"done attaching to shared memory and semaphores");
	#endif
}

void	connection::listen() {

	// detach from the controlling tty
	#ifndef SERVER_DEBUG
	detach();
	#endif

	blockSignals();

	#ifdef SERVER_DEBUG
	debugPrint(0,"logging in...");
	#endif
	if (!logIn()) {
		#ifdef SERVER_DEBUG
		debugPrint(0,"log in failed");
		#endif
		cerr << "Couldn't log into database." << ENDL;
		return;
	}
	#ifdef SERVER_DEBUG
	debugPrint(0,"done logging in");
	#endif

	#ifdef SERVER_DEBUG
	debugPrint(0,"setting autocommit...");
	#endif
	if (autocommit) {
		if (!autoCommitOn()) {
			#ifdef SERVER_DEBUG
			debugPrint(0,"setting autocommit on failed");
			#endif
			cerr << "Couldn't set autocommit on." << ENDL;
			return;
		}
	} else {
		if (!autoCommitOff()) {
			#ifdef SERVER_DEBUG
			debugPrint(0,"setting autocommit off failed");
			#endif
			cerr << "Couldn't set autocommit off." << ENDL;
			return;
		}
	}
	#ifdef SERVER_DEBUG
	debugPrint(0,"done setting autocommit");
	#endif

	if (!initCursors()) {
		// display couldn't create cursors in error
		cerr << "Couldn't create cursors." << ENDL;
		return;
	}

	createSharedMemoryAndSemaphores();

	// increment connection counter
	if (dynamicscaling) {
		incrementConnectionCount();
	}

	markDatabaseAvailable();

	// if we're not passing descriptors around, listen on 
	// inet and unix sockets for client connections
	if (!passdescriptor) {
		openSockets();
	}

	for (;;) {

		waitForAvailableDatabase();
		initSession();
		announceAvailability();

		// loop to handle suspended sessions
		for (;;) {

			int	success=waitForClient();
			if (success==1) {

				suspendsession=0;

				// have a session with the client
				clientSession();

				// break out of the loop unless the client
				// suspended the session
				if (!suspendsession) {
					break;
				}

			} else if (success==-1) {

				// if waitForClient() errors out, break out of
				// the suspendsession loop and loop back
				// for another session
				break;

			} else {

				// if waitForClient() times out waiting for
				// someone to pick up the suspended
				// session, roll it back and kill it
				if (suspendsession) {
					if (isTransactional()) {
						rollback();
					}
					suspendsession=0;
				}
			}
		}

		if (dynamicscaling) {
			decrementSessionCount();
		}
	}
}

void	connection::blockSignals() {

	// the daemon class handles SIGTERM's and SIGINT's
	// and we need to handle SIGALRMS so dynamically spawned daemons
	// can shut down but we're going to block all other signals
	signalset	set;
	set.removeAllSignals();

	#ifdef HAVE_SIGHUP
		set.addSignal(SIGHUP);
	#endif
	#ifdef HAVE_SIGQUIT
		set.addSignal(SIGQUIT);
	#endif
	#ifdef HAVE_SIGILL
		set.addSignal(SIGILL);
	#endif
	#ifdef HAVE_SIGTRAP
		set.addSignal(SIGTRAP);
	#endif
	#ifdef HAVE_SIGABRT
		set.addSignal(SIGABRT);
	#endif
	#ifdef HAVE_SIGIOT
		set.addSignal(SIGIOT);
	#endif
	#ifdef HAVE_SIGBUS
		set.addSignal(SIGBUS);
	#endif
	#ifdef HAVE_SIGFPE
		set.addSignal(SIGFPE);
	#endif
	#ifdef HAVE_SIGUSR1
		set.addSignal(SIGUSR1);
	#endif
	#ifdef HAVE_SIGSEGV
		set.addSignal(SIGSEGV);
	#endif
	#ifdef HAVE_SIGUSR2
		set.addSignal(SIGUSR2);
	#endif
	#ifdef HAVE_SIGPIPE
		set.addSignal(SIGPIPE);
	#endif
	#ifdef HAVE_SIGSTKFLT
		set.addSignal(SIGSTKFLT);
	#endif
	#ifdef HAVE_SIGCHLD
		set.addSignal(SIGCHLD);
	#endif
	#ifdef HAVE_SIGCONT
		set.addSignal(SIGCONT);
	#endif
	#ifdef HAVE_SIGSTOP
		set.addSignal(SIGSTOP);
	#endif
	#ifdef HAVE_SIGTSTP
		set.addSignal(SIGTSTP);
	#endif
	#ifdef HAVE_SIGTTIN
		set.addSignal(SIGTTIN);
	#endif
	#ifdef HAVE_SIGTTOU
		set.addSignal(SIGTTOU);
	#endif
	#ifdef HAVE_SIGURG
		set.addSignal(SIGURG);
	#endif
	#ifdef HAVE_SIGXCPU
		set.addSignal(SIGXCPU);
	#endif
	#ifdef HAVE_SIGXFSZ
		set.addSignal(SIGXFSZ);
	#endif
	#ifdef HAVE_SIGVTALRM
		set.addSignal(SIGVTALRM);
	#endif
	#ifdef HAVE_SIGPROF
		set.addSignal(SIGPROF);
	#endif
	#ifdef HAVE_SIGWINCH
		set.addSignal(SIGWINCH);
	#endif
	#ifdef HAVE_SIGIO
		set.addSignal(SIGIO);
	#endif
	#ifdef HAVE_SIGPOLL
		set.addSignal(SIGPOLL);
	#endif
	#ifdef HAVE_SIGPWR
		set.addSignal(SIGPWR);
	#endif
	#ifdef HAVE_SIGUNUSED
		set.addSignal(SIGUNUSED);
	#endif
	#ifdef HAVE_SIGEMT
		set.addSignal(SIGEMT);
	#endif
	#ifdef HAVE_SIGSYS
		set.addSignal(SIGSYS);
	#endif
	#ifdef HAVE_SIGCLD
		set.addSignal(SIGCLD);
	#endif
	#ifdef HAVE_SIGWAITING
		set.addSignal(SIGWAITING);
	#endif
	#ifdef HAVE_SIGLWP
		set.addSignal(SIGLWP);
	#endif
	#ifdef HAVE_SIGFREEZE
		set.addSignal(SIGFREEZE);
	#endif
	#ifdef HAVE_SIGTHAW
		set.addSignal(SIGTHAW);
	#endif
	#ifdef HAVE_SIGCANCEL
		set.addSignal(SIGCANCEL);
	#endif
	#ifdef HAVE_SIGLOST
		set.addSignal(SIGLOST);
	#endif
	#ifdef HAVE__SIGRTMIN
		set.addSignal(_SIGRTMIN);
	#endif
	#ifdef HAVE__SIGRTMAX
		set.addSignal(_SIGRTMAX);
	#endif
	#ifdef HAVE_SIGRTMIN
		set.addSignal(SIGRTMIN);
	#endif
	#ifdef HAVE_SIGRTMAX
		set.addSignal(SIGRTMAX);
	#endif

	signalmanager::ignoreSignals(set.getSignalSet());
}

void	connection::openSockets() {

	#ifdef SERVER_DEBUG
	debugPrint(0,"listening on sockets...");
	#endif

	// get the next available unix socket and open it
	if (listenonunix && unixsocketptr[0]) {

		#ifdef SERVER_DEBUG
		debugPrintStart();
		debugTab(1);
		debugPrint("listening on unix socket: ");
		debugPrint(unixsocket);
		debugPrintEnd();
		#endif

		if (!serversockun) {
			serversockun=new serversocket();
			if (serversockun->listenOnUnixPort(unixsocket,
					permissions::everyoneReadWrite(),5)) {
				addSocket(serversockun);
			} else {
				cerr << "Could not listen on unix socket: ";
				cerr << unixsocket << ENDL;
				cerr << "Make sure that the file and ";
				cerr << "directory are readable and writable.";
				cerr << ENDL << ENDL;
			}
		}
	}

	// open the next available inet socket
	if (listenoninet) {

		#ifdef SERVER_DEBUG
		debugPrintStart();
		debugTab(1);
		debugPrint("listening on inet socket: ");
		debugPrint((long)inetport);
		debugPrintEnd();
		#endif

		if (!serversockin) {
			serversockin=new serversocket();
			serversockin->listenOnInetPort(inetport,5);
			if (inetport==0) {
				inetport=serversockin->getPort();
			}
			addSocket(serversockin);
		}
	}

	#ifdef SERVER_DEBUG
	debugPrint(0,"done listening on sockets");
	#endif
}

void	connection::reLogIn() {

	#ifdef SERVER_DEBUG
	debugPrint(4,"relogging in...");
	#endif

	// attempt to log in over and over, once every 5 seconds
	closeCursors();
	logOut();
	for (;;) {
			
		#ifdef SERVER_DEBUG
		debugPrint(5,"trying...");
		#endif

		if (logIn()) {
			if (!initCursors()) {
				closeCursors();
				logOut();
			} else {
				break;
			}
		}
		sleep(5);
	}

	#ifdef SERVER_DEBUG
	debugPrint(4,"done relogging in");
	#endif
}

void	connection::announceAvailability() {

	#ifdef SERVER_DEBUG
	debugPrint(0,"announcing availability...");
	#endif

	// if we're passing around file descriptors, connect to listener 
	// if we haven't already and pass it this process's pid
	if (passdescriptor && !handoffsockun) {
		registerForHandoff();
	}

	// handle time-to-live
	alarm(ttl);

	// wait on the write mutex
	semset->wait(0);

	// cancel time-to-live alarm
	alarm(0);

	// get a pointer to the shared memory segment
	void	*idmemoryptr=(void *)((long)idmemory->getPointer()+
						(2*sizeof(unsigned int)));

	// if we're passing descriptors around, write the 
	// pid to the segment otherwise write ports
	if (passdescriptor) {

		#ifdef SERVER_DEBUG
		debugPrint(1,"handoff=pass");
		#endif

		*((unsigned long *)idmemoryptr)=(unsigned long)getpid();
	} else {

		#ifdef SERVER_DEBUG
		debugPrint(1,"handoff=reconnect");
		#endif

		int	unixsocketlen=strlen(unixsocket);
		memcpy((void *)idmemoryptr,(void *)unixsocket,unixsocketlen);
		idmemoryptr=(void *)((long)idmemoryptr+unixsocketlen);
		*((char *)idmemoryptr)=':';
		idmemoryptr=(void *)((long)idmemoryptr+1);

		// right-align the inet port
		*((unsigned short *)idmemoryptr)=inetport & 0xff;
		*((unsigned short *)(idmemoryptr+1))=inetport >> 8;
	}

	// signal that it's ok to read
	semset->signal(2);

	// wait for read to finish
	semset->wait(3);

	// signal on the write mutex
	semset->signal(0);

	#ifdef SERVER_DEBUG
	debugPrint(0,"done announcing availability...");
	#endif
}

void	connection::registerForHandoff() {

	#ifdef SERVER_DEBUG
	debugPrint(0,"registering for handoff...");
	#endif

	// construct the name of the socket to connect to
	char	*handoffsockname=new char[tmpdirlen+1+strlen(id)+8+1];
	sprintf(handoffsockname,"%s/%s-handoff",TMP_DIR,id);

	#ifdef SERVER_DEBUG
	debugPrintStart();
	debugTab(1);
	debugPrint("handoffsockname: ");
	debugPrint(handoffsockname);
	debugPrintEnd();
	#endif

	// Try to connect over and over forever on 1 second intervals.
	// If the connect succeeds but the write fails, loop back and
	// try again.
	while (1) {

		#ifdef SERVER_DEBUG
		debugPrint(1,"trying...");
		#endif

		handoffsockun=new clientsocket();
		handoffsockun->connectToServer(handoffsockname,1,0);
		if (handoffsockun->writeData(
				(unsigned long)getpid())==
					sizeof(unsigned long)) {
			break;
		}
		deRegisterForHandoff();
		delete handoffsockun;
	}

	// clean up
	delete[] handoffsockname;

	#ifdef SERVER_DEBUG
	debugPrint(0,"done registering for handoff");
	#endif
}

void	connection::deRegisterForHandoff() {

	#ifdef SERVER_DEBUG
	debugPrint(0,"de-registering for handoff...");
	#endif

	// construct the name of the socket to connect to
	char	*removehandoffsockname=new char[tmpdirlen+1+strlen(id)+14+1];
	sprintf(removehandoffsockname,"%s/%s-removehandoff",TMP_DIR,id);

	#ifdef SERVER_DEBUG
	debugPrintStart();
	debugTab(1);
	debugPrint("removehandoffsockname: ");
	debugPrint(removehandoffsockname);
	debugPrintEnd();
	#endif

	// attach to the socket and write the process id
	clientsocket	*removehandoffsockun=new clientsocket();
	removehandoffsockun->connectToServer(removehandoffsockname,0,1);
	removehandoffsockun->writeData((unsigned long)getpid());

	// clean up
	delete removehandoffsockun;
	delete[] removehandoffsockname;

	#ifdef SERVER_DEBUG
	debugPrint(0,"done de-registering for handoff");
	#endif
}

int	connection::waitForClient() {

	#ifdef SERVER_DEBUG
	debugPrint(0,"waiting for client...");
	#endif

	// Unless we're in the middle of a suspended session, if we're passing 
	// file descriptors around, wait for one to be passed to us, otherwise,
	// accept on the unix/inet sockets. 
	if (!suspendsession && passdescriptor) {

		// receive the descriptor and use it, if we failed to get the
		// descriptor, delete the socket and return failure
		int	descriptor;
		if (!handoffsockun->receiveFileDescriptor(&descriptor)) {
			delete handoffsockun;
			handoffsockun=NULL;

			#ifdef SERVER_DEBUG
			debugPrint(1,"pass failed");
			debugPrint(0,"done waiting for client");
			#endif

			return -1;
		}
		clientsock=new genericsocket(descriptor);

		#ifdef SERVER_DEBUG
		debugPrint(1,"pass succeeded");
		debugPrint(0,"done waiting for client");
		#endif

		return 1;

	} else {

		serversocket	*whichsock;
		int	retval=waitForClientConnection(accepttimeout,0,
						&whichsock,&clientsock);

		#ifdef SERVER_DEBUG
		if (retval) {
			debugPrint(1,"reconnect succeeded");
		} else {
			debugPrint(1,"reconnect failed");
		}
		debugPrint(0,"done waiting for client");
		#endif

		return retval;
	}
}

void	connection::clientSession() {

	#ifdef SERVER_DEBUG
	debugPrint(0,"client session...");
	#endif

	// a session consists of getting a query and returning a result set
	// over and over
	for (;;) {

		// is this a query, fetch, abort, suspend or resume...
		if (!getCommand()) {

			#ifdef SERVER_DEBUG
			debugPrint(1,"no command");
			#endif

			endSession();
			break;
		}

		// handle some things up front
		if (command==AUTHENTICATE) {
			#ifdef SERVER_DEBUG
			debugPrint(1,"authenticate");
			#endif
			if (!authenticate()) {
				// indicate that an error has occurred
				clientsock->writeData((unsigned short)ERROR);
				endSession();
				break;
			}
			// indicate that no error has occurred
			clientsock->writeData((unsigned short)NO_ERROR);
			continue;

		} else if (command==SUSPEND_SESSION) {
			#ifdef SERVER_DEBUG
			debugPrint(1,"suspend session");
			#endif
			suspendSession();
			break;
		} else if (command==END_SESSION) {
			#ifdef SERVER_DEBUG
			debugPrint(1,"end session");
			#endif
			endSession();
			break;
		} else if (command==PING) {
			#ifdef SERVER_DEBUG
			debugPrint(1,"ping");
			#endif
			clientsock->writeData((unsigned short)ping());
			continue;
		} else if (command==IDENTIFY) {
			#ifdef SERVER_DEBUG
			debugPrint(1,"identify");
			#endif
			char		*ident=identify();
			unsigned short	idlen=(unsigned short)strlen(ident);
			clientsock->writeData(idlen);
			clientsock->writeData(ident,idlen);
			continue;
		} else if (command==AUTOCOMMIT) {
			#ifdef SERVER_DEBUG
			debugPrint(1,"autocommit...");
			#endif
			unsigned short	on;
			clientsock->readData(&on);
			if (on) {
				#ifdef SERVER_DEBUG
				debugPrint(2,"autocommit on");
				#endif
				clientsock->writeData(autoCommitOn());
			} else {
				#ifdef SERVER_DEBUG
				debugPrint(2,"autocommit off");
				#endif
				clientsock->writeData(autoCommitOff());
			}
			continue;
		} else if (command==COMMIT) {
			#ifdef SERVER_DEBUG
			debugPrint(1,"commit");
			#endif
			clientsock->writeData((unsigned short)commit());
			commitorrollback=0;
			continue;
		} else if (command==ROLLBACK) {
			#ifdef SERVER_DEBUG
			debugPrint(1,"rollback");
			#endif
			clientsock->writeData((unsigned short)rollback());
			commitorrollback=0;
			continue;
		}

		// Get the cursor to work with, this may be an existing
		// cursor or a request for a a new one.  Save the
		// result of this, the client may be sending more information
		// and we need to collect it.
		int	cursorstatus;
		if (!(cursorstatus=getCursor())) {
			noAvailableCursors();
			// If no cursor was available, and
			// the client won't be sending any more data,
			// loop back...
			if (command!=SENDING_QUERY) {
				continue;
			}
		}

		// handle a query...
		if (command==SENDING_QUERY) {

			#ifdef SERVER_DEBUG
			debugPrint(1,"sending query");
			#endif

			// handle query will return 1 for success,
			// 0 for network error and -1 for a bad query
			int	querystatus=handleQuery(cursorstatus);
			if (querystatus==1) {

				// reinit lastrow
				lastrow=-1;
				if (!returnResultSetData()) {
					endSession();
					break;
				}

			} else if (querystatus==-1) {
				continue;
			} else {
				endSession();
				break;
			}

		} else if (command==FETCH_RESULT_SET) {
			#ifdef SERVER_DEBUG
			debugPrint(1,"fetch result set");
			#endif
			if (!returnResultSetData()) {
				endSession();
				break;
			}
		} else if (command==ABORT_RESULT_SET) {
			#ifdef SERVER_DEBUG
			debugPrint(1,"abort result set");
			#endif
			cur[currentcur]->abort();
		} else if (command==SUSPEND_RESULT_SET) {
			#ifdef SERVER_DEBUG
			debugPrint(1,"suspend result set");
			#endif
			cur[currentcur]->suspendresultset=1;
		} else if (command==RESUME_RESULT_SET) {
			#ifdef SERVER_DEBUG
			debugPrint(1,"resume result set");
			#endif
			resumeResultSet();
			if (!returnResultSetData()) {
				endSession();
				break;
			}
		} else {
			endSession();
			break;
		}
	}

	// Sometimes the server sends the result set and closes the socket
	// while part of it is buffered but not yet transmitted.  This caused
	// the client to receive a partial result set or error.  Telling the
	// socket to linger doesn't always fix it.  Doing a read here should 
	// guarantee that the client will close it's end of the connection 
	// before the server closes it's end; the server will wait for data 
	// from the client (which it will never receive) and when the client 
	// closes it's end (which it will only do after receiving the entire
	// result set) the read will fall through.  This should guarantee 
	// that the client will get the the entire result set without
	// requiring the client to send data back indicating so.
	#ifdef SERVER_DEBUG
	debugPrint(1,"waiting for client to close the connection...");
	#endif
	unsigned short	dummy;
	clientsock->readData(&dummy);
	clientsock->closeSocket();
	delete clientsock;
	#ifdef SERVER_DEBUG
	debugPrint(1,"done waiting for client to close the connection...");
	#endif

	// If we're no longer in a suspended session and we we're passing 
	// around file descriptors but had to open a set of sockets to handle 
	// a suspended session, close those sockets here.
	if (!suspendsession && passdescriptor) {
		#ifdef SERVER_DEBUG
		debugPrint(1,"closing sockets from a previously suspended session...");
		#endif
		if (serversockun) {
			removeSocket(serversockun);
			delete serversockun;
			serversockun=NULL;
		}
		if (serversockin) {
			removeSocket(serversockin);
			delete serversockin;
			serversockin=NULL;
		}
		#ifdef SERVER_DEBUG
		debugPrint(1,"done closing sockets from a previously suspended session...");
		#endif
	}

	#ifdef SERVER_DEBUG
	debugPrint(0,"done with client session");
	#endif
}

int	connection::authenticate() {

	#ifdef SERVER_DEBUG
	debugPrint(1,"authenticate...");
	#endif

	// get the user/password from the client
	unsigned long size;
	clientsock->readData(&size);
	if (size>USERSIZE || clientsock->readData(userbuffer,size)!=size) {
		#ifdef SERVER_DEBUG
		debugPrint(1,"authentication failed: user size is wrong");
		#endif
		return 0;
	}
	userbuffer[size]=(char)NULL;
	clientsock->readData(&size);
	if (size>USERSIZE || clientsock->readData(passwordbuffer,size)!=size) {
		#ifdef SERVER_DEBUG
		debugPrint(1,"authentication failed: password size is wrong");
		#endif
		return 0;
	}
	passwordbuffer[size]=(char)NULL;

	if (authonconn==1) {

		// handle connection-based authentication
		int	retval=authc->authenticate(userbuffer,passwordbuffer);
		#ifdef SERVER_DEBUG
		if (retval) {
			debugPrint(1,"connection-based authentication succeeded");
		} else {
			debugPrint(1,"connection-based authentication failed: invalid user/password");
		}
		#endif
		return retval;

	} else if (authonconn==2) {

		// handle database-based authentication

		// if the user we want to change to is different from the
		// user that's currently proxied, try to change to that user
		int	authsuccess;
		if ((!lastuserbuffer[0] && !lastpasswordbuffer[0]) || 
			strcmp(lastuserbuffer,userbuffer) ||
			strcmp(lastpasswordbuffer,passwordbuffer)) {

			// change authentication 
			authsuccess=changeUser(userbuffer,passwordbuffer);

			// keep a record of which user we're changing to
			// and whether that user was successful in 
			// authenticating
			strcpy(lastuserbuffer,userbuffer);
			strcpy(lastpasswordbuffer,passwordbuffer);
			lastauthsuccess=authsuccess;
		}

		#ifdef SERVER_DEBUG
		if (lastauthsuccess) {
			debugPrint(1,"database-based authentication succeeded");
		} else {
			debugPrint(1,"database-based authentication failed: invalid user/password");
		}
		#endif
		return lastauthsuccess;
	}

	#ifdef SERVER_DEBUG
	debugPrint(1,"authentication was done on listener");
	#endif
	return 1;
}

int	connection::changeUser(char *newuser, char *newpassword) {
	// this needs to log out, change users and log back in
	// but for now it will just fail by default
	return 0;
}

int	connection::handleQuery(int cursorstatus) {

	#ifdef SERVER_DEBUG
	debugPrint(1,"handling query...");
	#endif

	// get the query, binds, column flag from the client
	if (!(getQuery() && getInputBinds() && 
		getOutputBinds() && getSendColumnInfo())) {
		#ifdef SERVER_DEBUG
		debugPrint(1,"failed to handle query");
		#endif
		return 0;
	}

	// If no cursor was available, loop back...
	if (!cursorstatus) {
		#ifdef SERVER_DEBUG
		debugPrint(1,"failed to handle query: no cursor available");
		#endif
		return 1;
	}

	// loop here to handle down databases
	for (;;) {

		// process the query
		if (processQuery()) {

			// indicate that no error has occurred
			clientsock->writeData(
				(unsigned short)NO_ERROR);

			// send the client the id of the 
			// cursor that it's going to use
			clientsock->writeData(
				(unsigned short)(currentcur));

			// tell the client that this is not a
			// suspended result set
			clientsock->writeData(
				(unsigned short)NO_SUSPENDED_RESULT_SET);

			// if the query processed 
			// ok then return a result set
			// header and loop back to send the
			// result set itself...
			returnResultSetHeader();

			#ifdef SERVER_DEBUG
			debugPrint(1,"handle query succeeded");
			#endif
			return 1;

		} else {

			// If the query didn't process ok,
			// handle the error.
			// If handleError returns a 0 then the error 
			// was a down database that has presumably
			// come back up by now.  Loop back...
			if (handleError()) {

				// client will be sending skip/fetch,
				// better get it even though we're not gonna
				// use it
				unsigned long	skipfetch;
				clientsock->readData(&skipfetch);
				clientsock->readData(&skipfetch);

				cur[currentcur]->abort();
				#ifdef SERVER_DEBUG
				debugPrint(1,"failed to handle query: error");
				#endif
				return -1;
			}
		}
	}

	#ifdef SERVER_DEBUG
	debugPrint(1,"done handling query");
	#endif
	return 1;
}

void	connection::resumeResultSet() {

	#ifdef SERVER_DEBUG
	debugPrint(1,"resume result set...");
	#endif

	if (cur[currentcur]->suspendresultset) {

		#ifdef SERVER_DEBUG
		debugPrint(2,"previous result set was suspended");
		#endif

		// indicate that no error has occurred
		clientsock->writeData((unsigned short)NO_ERROR);

		// send the client the id of the 
		// cursor that it's going to use
		clientsock->writeData((unsigned short)(currentcur));
		clientsock->writeData((unsigned short)SUSPENDED_RESULT_SET);

		// if the requested cursor really had a suspended
		// result set, send the lastrow of it to the client
		// then send the result set header
		clientsock->writeData((unsigned long)lastrow);
		returnResultSetHeader();
	} else {

		#ifdef 	SERVER_DEBUG
		debugPrint(2,"previous result set was not suspended");
		#endif

		// indicate that an error has occurred
		clientsock->writeData((unsigned short)ERROR);

		// send the error itself
		clientsock->writeData((unsigned short)43);
		clientsock->writeData("The requested result set was not suspended.",43);
	}

	#ifdef SERVER_DEBUG
	debugPrint(1,"done resuming result set");
	#endif
}

void	connection::suspendSession() {

	#ifdef SERVER_DEBUG
	debugPrint(1,"suspending session...");
	#endif

	// abort all cursors that aren't already suspended
	#ifdef SERVER_DEBUG
	debugPrint(2,"aborting busy, unsuspended cursors...");
	#endif
	suspendsession=1;
	accepttimeout=sessiontimeout;
	for (int i=0; i<cursorcount; i++) {
		if (!cur[i]->suspendresultset && cur[i]->busy) {
			#ifdef SERVER_DEBUG
			debugPrint(3,(long)i);
			#endif
			cur[i]->abort();
		}
	}

	#ifdef SERVER_DEBUG
	debugPrint(2,"done aborting busy, unsuspended cursors");
	#endif

	// If we're passing file descriptors around, we'll have to listen on a 
	// set of ports like we would if we were not passing descriptors around 
	// so the suspended client has something to resume to.  It's possible 
	// that the current session is just a resumed session though.  In that
	// case, no new sockets will be opened, the old ones will just be 
	// reused.  We'll also have to pass the socket/port to the client here.
	if (passdescriptor) {

		#ifdef SERVER_DEBUG
		debugPrint(2,"opening a socket to resume on...");
		#endif
		openSockets();
		#ifdef SERVER_DEBUG
		debugPrint(2,"done opening a socket to resume on");
		#endif

		#ifdef SERVER_DEBUG
		debugPrint(2,"passing socket info to client...");
		#endif
		if (serversockun) {
			unsigned short	unixsocketsize=strlen(unixsocket);
			clientsock->writeData(unixsocketsize);
			clientsock->writeData(unixsocket,unixsocketsize);
		} else {
			clientsock->writeData((unsigned short)0);
		}
		clientsock->writeData(inetport);
		#ifdef SERVER_DEBUG
		debugPrint(2,"done passing socket info to client...");
		#endif
	}

	#ifdef SERVER_DEBUG
	debugPrint(1,"done suspending session");
	#endif
}

void	connection::endSession() {

	#ifdef SERVER_DEBUG
	debugPrint(1,"ending session...");
	#endif

	// must set suspendsession to 0 here so resumed sessions won't 
	// automatically re-suspend
	suspendsession=0;

	// abort all cursors
	#ifdef SERVER_DEBUG
	debugPrint(2,"aborting all busy cursors...");
	#endif
	for (int i=0; i<cursorcount; i++) {
		if (cur[i]->busy) {
			#ifdef SERVER_DEBUG
			debugPrint(3,(long)i);
			#endif
			cur[i]->abort();
		}
	}
	#ifdef SERVER_DEBUG
	debugPrint(2,"done aborting all busy cursors");
	#endif

	// commit or rollback if necessary
	if (isTransactional() && commitorrollback) {
		if (eoscommit) {
			#ifdef SERVER_DEBUG
			debugPrint(2,"committing...");
			#endif
			commit();
			#ifdef SERVER_DEBUG
			debugPrint(2,"done committing...");
			#endif
		} else {
			#ifdef SERVER_DEBUG
			debugPrint(2,"rolling back...");
			#endif
			rollback();
			#ifdef SERVER_DEBUG
			debugPrint(2,"done rolling back...");
			#endif
		}
	}

	// reset autocommit behavior
	#ifdef SERVER_DEBUG
	debugPrint(2,"resetting autocommit behavior...");
	#endif
	if (autocommit) {
		#ifdef SERVER_DEBUG
		debugPrint(3,"setting autocommit on...");
		#endif
		autoCommitOn();
		#ifdef SERVER_DEBUG
		debugPrint(3,"done setting autocommit on...");
		#endif
	} else {
		#ifdef SERVER_DEBUG
		debugPrint(3,"setting autocommit off...");
		#endif
		autoCommitOff();
		#ifdef SERVER_DEBUG
		debugPrint(3,"done setting autocommit off...");
		#endif
	}
	#ifdef SERVER_DEBUG
	debugPrint(2,"done resetting autocommit behavior...");
	debugPrint(1,"done ending session");
	#endif
}

int	connection::getCursor() {

	#ifdef SERVER_DEBUG
	debugPrint(1,"getting a cursor...");
	#endif

	// which cursor is the client requesting?
	unsigned short	index;
	if (clientsock->readData(&index)!=sizeof(unsigned short)) {
		#ifdef SERVER_DEBUG
		debugPrint(2,"error: client cursor request failed");
		#endif
		return 0;
	}

	// don't allow the client to request a cursor 
	// beyond the end of the array.
	if (index>cursorcount) {
		#ifdef SERVER_DEBUG
		debugPrint(2,"error: client requested an invalid cursor");
		#endif
		return 0;
	}

	// Find a free cursor and send it to the client
	if (cur[index]->busy) {
		#ifdef SERVER_DEBUG
		debugPrint(2,"requested cursor is busy, finding a free cursor...");
		#endif
		for (int i=0; i<cursorcount; i++) {
			if (!cur[i]->busy) {
				currentcur=i;
				cur[currentcur]->busy=1;
				#ifdef SERVER_DEBUG
				debugPrint(3,(long)currentcur);
				debugPrint(2,"found a free cursor...");
				debugPrint(1,"done getting a cursor");
				#endif
				return 1;
			}
		}
		#ifdef SERVER_DEBUG
		debugPrint(1,"get cursor failed: all cursors are busy");
		#endif
		return 0;
	}

	// set the current cursor to the one they requested.
	currentcur=index;
	#ifdef SERVER_DEBUG
	debugPrint(2,"returning requested cursor");
	debugPrint(1,"done getting a cursor");
	#endif
	return 1;
}

void	connection::noAvailableCursors() {

	// indicate that an error has occurred
	clientsock->writeData((unsigned short)ERROR);

	// send the error itself
	clientsock->writeData((unsigned short)62);
	clientsock->writeData("No server-side cursors were available to process the query.",62);
}

int	connection::handleError() {

	#ifdef SERVER_DEBUG
	debugPrint(2,"handling error...");
	#endif

	// return the error unless the error was a dead connection, 
	// in which case, re-establish the connection
	if (!returnError()) {
		#ifdef SERVER_DEBUG
		debugPrint(3,"database is down...");
		#endif
		markDatabaseUnavailable();
		reLogIn();
		markDatabaseAvailable();
		return 0;
	}

	#ifdef SERVER_DEBUG
	debugPrint(2,"done handling error...");
	#endif
	return 1;
}

int	connection::getCommand() {

	#ifdef SERVER_DEBUG
	debugPrint(2,"getting command...");
	#endif

	// get the command
	if (clientsock->readData(&command)!=sizeof(unsigned short)) {
		#ifdef SERVER_DEBUG
		debugPrint(2,"getting command failed: client sent bad command");
		#endif
		return 0;
	}

	#ifdef SERVER_DEBUG
	debugPrint(2,"done getting command");
	#endif
	return 1;
}

int	connection::getQuery() {

	#ifdef SERVER_DEBUG
	debugPrint(2,"getting query...");
	#endif

	// is this a new query or a re-execution
	if (clientsock->readData(&cur[currentcur]->reexecute)!=
					sizeof(unsigned short)) {
		#ifdef SERVER_DEBUG
		debugPrint(2,"getting query failed: client sent bad reexecute flag size");
		#endif
		return 0;
	}
	if (cur[currentcur]->reexecute) {
		#ifdef SERVER_DEBUG
		debugPrint(2,"getting query succeeded, re-executing");
		#endif
		return 1;
	}

	// get the length of the query
	if (clientsock->readData(&cur[currentcur]->querylength)!=
					sizeof(unsigned long)) {
		#ifdef SERVER_DEBUG
		debugPrint(2,"getting query failed: client sent bad query length size");
		#endif
		return 0;
	}

	// bounds checking
	if (cur[currentcur]->querylength>MAXQUERYSIZE) {
		#ifdef SERVER_DEBUG
		debugPrint(2,"getting query failed: client sent bad query size");
		#endif
		return 0;
	}

	// read the query into the buffer
	if (clientsock->readData(cur[currentcur]->querybuffer,
				cur[currentcur]->querylength)!=
					cur[currentcur]->querylength) {
		#ifdef SERVER_DEBUG
		debugPrint(2,"getting query failed: client sent short query");
		#endif
		return 0;
	}
	cur[currentcur]->querybuffer[cur[currentcur]->querylength]=(char)NULL;

	#ifdef SERVER_DEBUG
	debugPrint(3,"query:");
	debugPrint(0,cur[currentcur]->querybuffer);
	debugPrint(2,"getting query succeeded");
	#endif

	return 1;
}


int	connection::getInputBinds() {

	#ifdef SERVER_DEBUG
	debugPrint(2,"getting input binds...");
	#endif

	// get the number of input bind variable/values
	if (clientsock->readData(&cur[currentcur]->inbindcount)!=
					sizeof(unsigned short)) {
		#ifdef SERVER_DEBUG
		debugPrint(2,"getting input binds failed: client sent bad input bind count size");
		#endif
		return 0;
	}

	// bounds checking
	if (cur[currentcur]->inbindcount>MAXVAR) {
		#ifdef SERVER_DEBUG
		debugPrint(2,"getting input binds failed: client tried to send too many input binds");
		#endif
		return 0;
	}
	
	unsigned short	inbindsize;
	unsigned short	type;
	if (cur[currentcur]->inbindcount>0) {

		#ifdef SERVER_DEBUG
		debugPrint(3,"getting input binds...");
		#endif
		
		// fill the buffers
		for (int i=0; 
			i<cur[currentcur]->inbindcount && i<MAXVAR; i++) {

			// get the variable name size
			if (clientsock->readData(&inbindsize)!=
					sizeof(unsigned short)) {
				#ifdef SERVER_DEBUG
				debugPrint(2,"getting input binds failed: bad variable name length size");
				#endif
				return 0;
			}

			// bounds checking
			if (inbindsize>BINDVARLENGTH) {
				#ifdef SERVER_DEBUG
				debugPrint(2,"getting input binds failed: bad variable name length");
				#endif
				return 0;
			}

			// get the variable name
			cur[currentcur]->inbindvars[i].variablesize=
							inbindsize+1;
			cur[currentcur]->inbindvars[i].variable[0]=
							bindVariablePrefix();
			if (clientsock->readData(cur[currentcur]->inbindvars[i].
						variable+1,
						inbindsize)!=inbindsize) {
				#ifdef SERVER_DEBUG
				debugPrint(2,"getting input binds failed: bad variable name");
				#endif
				return 0;
			}
			cur[currentcur]->inbindvars[i].variable[inbindsize+1]=
								(char)NULL;

			#ifdef SERVER_DEBUG
			debugPrint(4,cur[currentcur]->inbindvars[i].variable);
			#endif

			// get the type
			if (clientsock->readData(&type)!=
						sizeof(unsigned short)) {
				#ifdef SERVER_DEBUG
				debugPrint(2,"getting input binds failed: bad type size");
				#endif
				return 0;
			}
			cur[currentcur]->inbindvars[i].type=(bindtype)type;

			// get the value
			if (cur[currentcur]->inbindvars[i].type==NULL_BIND) {

				#ifdef SERVER_DEBUG
				debugPrint(4,"NULL");
				#endif

				cur[currentcur]->inbindvars[i].
							value.stringval[0]=
							(char)NULL;
				cur[currentcur]->inbindvars[i].valuesize=0;
				cur[currentcur]->inbindvars[i].isnull=
							nullBindValue();

			} else if (cur[currentcur]->inbindvars[i].type==
							STRING_BIND) {

				#ifdef SERVER_DEBUG
				debugPrint(4,"STRING");
				#endif

				if (clientsock->readData(&inbindsize)!=
						sizeof(unsigned short)) {
					#ifdef SERVER_DEBUG
					debugPrint(2,"getting input binds failed: bad value length size");
					#endif
					return 0;
				}

				// bounds checking
				if (inbindsize>BINDVALUELENGTH) {
					#ifdef SERVER_DEBUG
					debugPrint(2,"getting input binds failed: bad value length");
					#endif
					return 0;
				}

				if (clientsock->readData(
					cur[currentcur]->inbindvars[i].value.
					stringval,
					inbindsize)!=inbindsize) {
					#ifdef SERVER_DEBUG
					debugPrint(2,"getting input binds failed: bad value");
					#endif
					return 0;
				}
				cur[currentcur]->inbindvars[i].value.
					stringval[inbindsize]=(char)NULL;
				cur[currentcur]->inbindvars[i].
					valuesize=inbindsize;
				cur[currentcur]->inbindvars[i].
					isnull=nonNullBindValue();

				#ifdef SERVER_DEBUG
				debugPrint(4,cur[currentcur]->inbindvars[i].value.stringval);
				#endif

			} else if (cur[currentcur]->inbindvars[i].type==
								LONG_BIND) {

				#ifdef SERVER_DEBUG
				debugPrint(4,"LONG");
				#endif

				if (clientsock->readData(
					&cur[currentcur]->inbindvars[i].value.
					longval)!=sizeof(unsigned long)) {
					#ifdef SERVER_DEBUG
					debugPrint(2,"getting input binds failed: bad value");
					#endif
					return 0;
				}

				#ifdef SERVER_DEBUG
				debugPrint(4,(long)cur[currentcur]->inbindvars[i].value.longval);
				#endif

			} else if (cur[currentcur]->inbindvars[i].type==
								DOUBLE_BIND) {

				#ifdef SERVER_DEBUG
				debugPrint(4,"DOUBLE");
				#endif

				if (clientsock->readData(
					&cur[currentcur]->inbindvars[i].value.
					doubleval.value)!=sizeof(double)) {
					#ifdef SERVER_DEBUG
					debugPrint(2,"getting input binds failed: bad value");
					#endif
					return 0;
				}
				if (clientsock->readData(
					&cur[currentcur]->inbindvars[i].value.
					doubleval.precision)!=
						sizeof(unsigned short)) {
					#ifdef SERVER_DEBUG
					debugPrint(2,"getting input binds failed: bad precision");
					#endif
					return 0;
				}
				if (clientsock->readData(
					&cur[currentcur]->inbindvars[i].value.
					doubleval.scale)!=
						sizeof(unsigned short)) {
					#ifdef SERVER_DEBUG
					debugPrint(2,"getting input binds failed: bad scale");
					#endif
					return 0;
				}

				#ifdef SERVER_DEBUG
				debugPrint(4,cur[currentcur]->inbindvars[i].value.doubleval.value);
				#endif
			}
		}

		#ifdef SERVER_DEBUG
		debugPrint(3,"done getting input binds...");
		#endif
	}

	#ifdef SERVER_DEBUG
	debugPrint(2,"done getting input binds");
	#endif
	return 1;
}


int	connection::getOutputBinds() {

	#ifdef SERVER_DEBUG
	debugPrint(2,"getting output binds...");
	#endif

	// get the number of output bind variable/values
	if (clientsock->readData(&cur[currentcur]->outbindcount)!=
						sizeof(unsigned short)) {
		#ifdef SERVER_DEBUG
		debugPrint(2,"getting output binds failed: client sent bad output bind count size");
		#endif
		return 0;
	}

	// bounds checking
	if (cur[currentcur]->outbindcount>MAXVAR) {
		#ifdef SERVER_DEBUG
		debugPrint(2,"getting output binds failed: client tried to send too many output binds");
		#endif
		return 0;
	}

	unsigned short	outbindsize;
	if (cur[currentcur]->outbindcount>0) {

		#ifdef SERVER_DEBUG
		debugPrint(3,"getting input binds...");
		#endif
		
		// fill the buffers
		for (int i=0; 
			i<cur[currentcur]->outbindcount && i<MAXVAR; i++) {

			// get the variable name size
			if (clientsock->readData(&outbindsize)!=
					sizeof(unsigned short)) {
				#ifdef SERVER_DEBUG
				debugPrint(2,"getting output binds failed: bad variable name length size");
				#endif
				return 0;
			}

			// bounds checking
			if (outbindsize>BINDVARLENGTH) {
				#ifdef SERVER_DEBUG
				debugPrint(2,"getting output binds failed: bad variable name length");
				#endif
				return 0;
			}

			// get the variable name
			cur[currentcur]->outbindvars[i].variablesize=
							outbindsize+1;
			cur[currentcur]->outbindvars[i].variable[0]=
							bindVariablePrefix();
			if (clientsock->readData(
				cur[currentcur]->outbindvars[i].variable+1,
				outbindsize)!=outbindsize) {
				#ifdef SERVER_DEBUG
				debugPrint(2,"getting output binds failed: bad variable name");
				#endif
				return 0;
			}
			cur[currentcur]->outbindvars[i].
					variable[outbindsize+1]=(char)NULL;

			#ifdef SERVER_DEBUG
			debugPrint(4,cur[currentcur]->inbindvars[i].variable);
			#endif

			// get the size that the buffer to store the
			// result needs to be
			if (clientsock->readData(
				&cur[currentcur]->outbindvars[i].valuesize)!=
				sizeof(unsigned short)) {
				#ifdef SERVER_DEBUG
				debugPrint(2,"getting input binds failed: bad value length size");
				#endif
				return 0;
			}

			// size checking
			if (cur[currentcur]->outbindvars[i].valuesize>
							BINDVALUELENGTH) {
				#ifdef SERVER_DEBUG
				debugPrint(2,"getting input binds failed: bad value length");
				#endif
				return 0;
			}
		}

		#ifdef SERVER_DEBUG
		debugPrint(3,"done getting output binds...");
		#endif
	}

	#ifdef SERVER_DEBUG
	debugPrint(2,"done getting output binds");
	#endif
	return 1;
}

int	connection::getSendColumnInfo() {

	#ifdef SERVER_DEBUG
	debugPrint(2,"getting send column info...");
	#endif

	if (clientsock->readData(&sendcolumninfo)!=sizeof(unsigned short)) {
		#ifdef SERVER_DEBUG
		debugPrint(2,"getting send column info failed");
		#endif
		return 0;
	}

	#ifdef SERVER_DEBUG
	if (sendcolumninfo==SEND_COLUMN_INFO) {
		debugPrint(3,"send column info");
	} else {
		debugPrint(3,"don't send column info");
	}
	debugPrint(2,"done getting send column info...");
	#endif

	return 1;
}

int	connection::sendColumnInfo() {
	if (sendcolumninfo==SEND_COLUMN_INFO) {
		return 1;
	}
	return 0;
}

int	connection::processQuery() {

	#ifdef SERVER_DEBUG
	debugPrint(2,"processing query...");
	#endif

	// if the reexecute flag is set, the query doesn't need to be prepared 
	// again.
	int	success=0;
	if (cur[currentcur]->reexecute) {
		#ifdef SERVER_DEBUG
		debugPrint(3,"re-executing...");
		#endif
		success=cur[currentcur]->handleBinds() && 
			cur[currentcur]->executeQuery(
					cur[currentcur]->querybuffer,
					cur[currentcur]->querylength);
	} else {
		#ifdef SERVER_DEBUG
		debugPrint(3,"preparing/executing...");
		#endif
		success=cur[currentcur]->prepareQuery(
					cur[currentcur]->querybuffer,
					cur[currentcur]->querylength) && 
			cur[currentcur]->handleBinds() && 
			cur[currentcur]->executeQuery(
					cur[currentcur]->querybuffer,
					cur[currentcur]->querylength);
	}

	// was the query a commit or rollback?
	commitOrRollback();

	// On success, autocommit if necessary.
	// Connection classes could override autoCommitOn() and autoCommitOff()
	// to do database API-specific things, but will not set 
	// checkautocommit, so this code won't get called at all for those 
	// connections.
	if (success && checkautocommit && isTransactional() && 
			performautocommit && commitorrollback) {
		#ifdef SERVER_DEBUG
		debugPrint(3,"commit necessary...");
		#endif
		success=commit();
		commitorrollback=0;
	}

	#ifdef SERVER_DEBUG
	if (success) {
		debugPrint(2,"processing query succeeded");
	} else {
		debugPrint(2,"processing query failed");
	}
	debugPrint(2,"done processing query");
	#endif

	return success;
}

int	connection::initCursors() {

	#ifdef SERVER_DEBUG
	debugPrint(0,"initializing cursors...");
	#endif

	for (int i=0; i<cursorcount; i++) {

		#ifdef SERVER_DEBUG
		debugPrint(1,(long)i);
		#endif

		cur[i]=initCursor();
		if (!cur[i]->openCursor(i)) {

			#ifdef SERVER_DEBUG
			debugPrint(1,"cursor init failure...");
			#endif

			logOut();
			return 0;
		}
	}

	#ifdef SERVER_DEBUG
	debugPrint(0,"done initializing cursors");
	#endif

	return 1;
}

void	connection::closeCursors() {

	#ifdef SERVER_DEBUG
	debugPrint(0,"closing cursors...");
	#endif

	if (cur) {
		for (int i=0; i<cursorcount; i++) {

			#ifdef SERVER_DEBUG
			debugPrint(1,(long)i);
			#endif

			if (cur[i]) {
				cur[i]->closeCursor();
				deleteCursor(cur[i]);
			}
		}
		delete[] cur;
	}

	#ifdef SERVER_DEBUG
	debugPrint(0,"done closing cursors...");
	#endif
}

short	connection::nonNullBindValue() {
	return 0;
}

short	connection::nullBindValue() {
	return -1;
}

char	connection::bindVariablePrefix() {
	return ':';
}

int	connection::bindValueIsNull(short isnull) {
	return abs(isnull);
}

int	connection::returnError() {

	#ifdef SERVER_DEBUG
	debugPrint(2,"returning error...");
	#endif

	// get the error message from the database
	// return value: 1 if database connection is still alive, 0 if not
	int	liveconnection;
	char	*error=cur[currentcur]->getErrorMessage(&liveconnection);

	// only return an error message if the error wasn't a dead database
	if (liveconnection) {

		// indicate that an error has occurred
		clientsock->writeData((unsigned short)ERROR);

		// send the error itself
		int	errorlen=strlen(error);
		clientsock->writeData((unsigned short)(errorlen+
				strlen(cur[currentcur]->querybuffer)+18));
		clientsock->writeData(error,errorlen);

		// send the attempted query back too
		clientsock->writeData("\nAttempted Query:\n");
		clientsock->writeData(cur[currentcur]->querybuffer);
	}
	
	#ifdef SERVER_DEBUG
	debugPrint(2,"done returning error");
	#endif

	return liveconnection;
}

void	connection::returnResultSetHeader() {

	#ifdef SERVER_DEBUG
	debugPrint(2,"returning result set header...");
	#endif


	// return the row counts
	#ifdef SERVER_DEBUG
	debugPrint(3,"returning row counts...");
	#endif
	cur[currentcur]->returnRowCounts();
	#ifdef SERVER_DEBUG
	debugPrint(3,"done returning row counts");
	#endif


	// write a flag to the client indicating whether 
	// or not the column information will be sent
	clientsock->writeData((unsigned short)sendcolumninfo);

	#ifdef SERVER_DEBUG
	if (sendcolumninfo==SEND_COLUMN_INFO) {
		debugPrint(3,"column info will be sent");
	} else {
		debugPrint(3,"column info will not be sent");
	}
	#endif


	// return the column count
	#ifdef SERVER_DEBUG
	debugPrint(3,"returning column counts...");
	#endif
	cur[currentcur]->returnColumnCount();
	#ifdef SERVER_DEBUG
	debugPrint(3,"done returning column counts");
	#endif


	if (sendcolumninfo==SEND_COLUMN_INFO) {
		// return the column info
		#ifdef SERVER_DEBUG
		debugPrint(3,"returning column info...");
		#endif
		cur[currentcur]->returnColumnInfo();
		#ifdef SERVER_DEBUG
		debugPrint(3,"done returning column info");
		#endif
	}


	// return the output bind vars
	returnOutputBindValues();


	// terminate the bind vars
	clientsock->writeData((unsigned short)END_BIND_VARS);


	#ifdef SERVER_DEBUG
	debugPrint(2,"done returning result set header");
	#endif
}

void	connection::returnOutputBindValues() {

	#ifdef SERVER_DEBUG
	debugPrint(2,"returning output bind values");
	#endif

	// run through the output bind values, sending them back
	for (int i=0; i<cur[currentcur]->outbindcount; i++) {

		#ifdef SERVER_DEBUG
		debugPrintStart();
		debugTab(3);
		debugPrint((long)i);
		#endif

		if (bindValueIsNull(cur[currentcur]->outbindvars[i].isnull)) {

			#ifdef SERVER_DEBUG
			debugPrint("NULL");
			#endif

			clientsock->writeData((unsigned short)NULL_DATA);

		} else {

			#ifdef SERVER_DEBUG
			debugPrint(cur[currentcur]->outbindvars[i].
							value.stringval);
			#endif

			clientsock->writeData((unsigned short)NORMAL_DATA);
			cur[currentcur]->outbindvars[i].valuesize=
				strlen(cur[currentcur]->outbindvars[i].
					value.stringval);
			clientsock->writeData(cur[currentcur]->outbindvars[i].
								valuesize);
			clientsock->writeData(cur[currentcur]->outbindvars[i].
					value.stringval,
					cur[currentcur]->outbindvars[i].
								valuesize);
		}

		#ifdef SERVER_DEBUG
		debugPrintEnd();
		#endif
	}

	#ifdef SERVER_DEBUG
	debugPrint(2,"done returning output bind values");
	#endif
}

int	connection::returnResultSetData() {

	#ifdef SERVER_DEBUG
	debugPrint(2,"returning result set data...");
	#endif

	// see if this result set even has any rows to return
	int	norows=cur[currentcur]->noRowsToReturn();


	// some useful variables
	unsigned long	skip;
	unsigned long	fetch;


	// get the number of rows to skip
	if (clientsock->readData(&skip)!=sizeof(unsigned long)) {
		#ifdef SERVER_DEBUG
		debugPrint(2,"returning result set data failed");
		#endif
		return 0;
	}

	// get the number of rows to fetch
	if (clientsock->readData(&fetch)!=sizeof(unsigned long)) {
		#ifdef SERVER_DEBUG
		debugPrint(2,"returning result set data failed");
		#endif
		return 0;
	}


	// for some queries, there are no rows to return, 
	if (norows) {
		clientsock->writeData((unsigned short)END_RESULT_SET);
		cur[currentcur]->abort();
		#ifdef SERVER_DEBUG
		debugPrint(2,"done returning result set data");
		#endif
		return 1;
	}


	// reinit suspendresultset
	cur[currentcur]->suspendresultset=0;


	// skip the specified number of rows
	if (!skipRows(skip)) {
		clientsock->writeData((unsigned short)END_RESULT_SET);
		cur[currentcur]->abort();
		#ifdef SERVER_DEBUG
		debugPrint(2,"done returning result set data");
		#endif
		return 1;
	}


	// send the specified number of rows back
	for (int i=0; (!fetch || i<fetch); i++) {

		if (!cur[currentcur]->fetchRow()) {
			clientsock->writeData((unsigned short)END_RESULT_SET);
			cur[currentcur]->abort();
			#ifdef SERVER_DEBUG
			debugPrint(2,"done returning result set data");
			#endif
			return 1;
		}

		#ifdef SERVER_DEBUG
		debugPrintStart();
		#endif
		cur[currentcur]->returnRow();
		#ifdef SERVER_DEBUG
		debugPrintEnd();
		#endif

		lastrow++;
	}

	#ifdef SERVER_DEBUG
	debugPrint(2,"done returning result set data");
	#endif
	return 1;
}

int	connection::skipRows(int rows) {

	#ifdef SERVER_DEBUG
	debugPrint(2,"skipping rows...");
	#endif

	for (int i=0; i<rows; i++) {

		#ifdef SERVER_DEBUG
		debugPrint(3,"skip...");
		#endif

		lastrow++;
		if (!cur[currentcur]->skipRow()) {
			#ifdef SERVER_DEBUG
			debugPrint(2,"skipping rows hit the end of the result set");
			#endif
			return 0;
		}
	}

	#ifdef SERVER_DEBUG
	debugPrint(2,"done skipping rows");
	#endif
	return 1;
}

void	connection::sendRowCounts(long actual, long affected) {

	#ifdef SERVER_DEBUG
	debugPrint(2,"sending row counts...");
	#endif


	// send actual rows, if that is known
	if (actual>-1) {

		#ifdef SERVER_DEBUG
		debugPrintStart();
		debugTab(3);
		debugPrint("actual rows: ");
		debugPrint(actual);
		debugPrintEnd();
		#endif

		clientsock->writeData((unsigned short)ACTUAL_ROWS);
		clientsock->writeData((unsigned long)actual);
	} else {

		#ifdef SERVER_DEBUG
		debugPrint(3,"actual rows unknown");
		#endif

		clientsock->writeData((unsigned short)NO_ACTUAL_ROWS);
	}

	
	// send affected rows, if that is known
	if (affected>-1) {

		#ifdef SERVER_DEBUG
		debugPrintStart();
		debugTab(3);
		debugPrint("affected rows: ");
		debugPrint(affected);
		debugPrintEnd();
		#endif

		clientsock->writeData((unsigned short)AFFECTED_ROWS);
		clientsock->writeData((unsigned long)affected);

	} else {

		#ifdef SERVER_DEBUG
		debugPrint(3,"affected rows unknown");
		#endif

		clientsock->writeData((unsigned short)NO_AFFECTED_ROWS);
	}

	#ifdef SERVER_DEBUG
	debugPrint(2,"done sending row counts");
	#endif
}

void	connection::sendColumnCount(unsigned long ncols) {

	#ifdef SERVER_DEBUG
	debugPrint(2,"sending column count...");
	#endif

	#ifdef SERVER_DEBUG
	debugPrint(3,(long)ncols);
	#endif
	clientsock->writeData(ncols);

	#ifdef SERVER_DEBUG
	debugPrint(2,"done sending column count");
	#endif
}

void	connection::sendColumnDefinition(char *name, unsigned short namelen,
					unsigned short type, 
					unsigned long size) {

	#ifdef SERVER_DEBUG
	debugPrintStart();
	debugTab(3);
	debugPrint(name);
	debugPrint(":");
	debugPrint((long)type);
	debugPrint(":");
	debugPrint((long)size);
	debugPrintEnd();
	#endif

	clientsock->writeData(namelen);
	clientsock->writeData(name,namelen);
	clientsock->writeData(type);
	clientsock->writeData(size);
}

void	connection::sendField(char *data, unsigned long size) {

	#ifdef SERVER_DEBUG
	debugPrint("\"");
	debugPrint(data);
	debugPrint("\",");
	#endif

	clientsock->writeData((unsigned short)NORMAL_DATA);
	clientsock->writeData(size);
	clientsock->writeData(data,size);
}

void	connection::sendNullField() {

	#ifdef SERVER_DEBUG
	debugPrint("NULL");
	#endif

	clientsock->writeData((unsigned short)NULL_DATA);
}

void	connection::startSendingLong() {
	clientsock->writeData((unsigned short)START_LONG_DATA);
}

void	connection::sendLongSegment(char *data, unsigned long size) {

	#ifdef SERVER_DEBUG
	debugPrint(data);
	#endif

	clientsock->writeData((unsigned short)NORMAL_DATA);
	clientsock->writeData(size);
	clientsock->writeData(data,size);
}

void	connection::endSendingLong() {

	#ifdef SERVER_DEBUG
	debugPrint(",");
	#endif

	clientsock->writeData((unsigned short)END_LONG_DATA);
}

int	connection::setEnv(char *var, char *string, char *pestr) {

	#if defined(HAVE_SETENV)
		return (setenv(var,string,1)!=-1);
	#elif defined(HAVE_PUTENV)
		return (putenv(pestr)!=-1);
	#else
		#error "Couldn't find a suitable replacement for putenv."
	#endif
}

int	connection::authOnConnection() {
	return authonconn;
}

void	connection::getUnixSocket() {

	#ifdef SERVER_DEBUG
	debugPrint(0,"getting unix socket...");
	#endif


	// open the sequence file and get the current port number
	sockseqname=new char[tmpdirlen+9];
	sprintf(sockseqname,"%s/sockseq",TMP_DIR);

	#ifdef SERVER_DEBUG
	debugPrintStart();
	debugTab(1);
	debugPrint("opening ");
	debugPrint(sockseqname);
	debugPrintEnd();
	#endif

	mode_t	oldumask=umask(011);
	int	sockseq=open(sockseqname,O_RDWR|O_CREAT,
				permissions::everyoneReadWrite());
	umask(oldumask);


	// handle error
	if (sockseq==-1) {
		cerr << "Could not open: " << sockseqname << ENDL;
		cerr << "Make sure that the file and directory are ";
		cerr << "readable and writable." << ENDL << ENDL;
		unixsocketptr[0]=(char)NULL;

		#ifdef SERVER_DEBUG
		debugPrintStart();
		debugTab(1);
		debugPrint("couldn't open ");
		debugPrint(sockseqname);
		debugPrintEnd();
		#endif

		return;
	}


	// lock the file in a platform-independent manner

	#ifdef SERVER_DEBUG
	debugPrintStart();
	debugTab(1);
	debugPrint("locking ");
	debugPrint(sockseqname);
	debugPrintEnd();
	#endif

	struct	flock fl;
	fl.l_type=F_WRLCK;
	fl.l_whence=SEEK_END;
	fl.l_start=0;
	fl.l_len=0;
	fcntl(sockseq,F_SETLKW,&fl);


	// get the sequence number
	long	buffer;
	int	size=read(sockseq,(void *)&buffer,sizeof(long));
	if (size<1) {
		buffer=0;
	}
	sprintf(unixsocketptr,"%d",buffer);

	#ifdef SERVER_DEBUG
	debugPrintStart();
	debugTab(1);
	debugPrint("got sequence number: ");
	debugPrint(unixsocketptr);
	debugPrintEnd();
	#endif


	// increment the sequence number
	if (buffer==pow(2,31)) {
		buffer=0;
	} else {
		buffer=buffer+1;
	}

	#ifdef SERVER_DEBUG
	debugPrintStart();
	debugTab(1);
	debugPrint("writing new sequence number: ");
	debugPrint(buffer);
	debugPrintEnd();
	#endif

	lseek(sockseq,0,SEEK_SET);
	write(sockseq,(void *)&buffer,sizeof(long));


	// unlock and close the file in a platform-independent manner

	#ifdef SERVER_DEBUG
	debugPrintStart();
	debugTab(1);
	debugPrint("unlocking ");
	debugPrint(sockseqname);
	debugPrintEnd();
	#endif

	fl.l_type=F_UNLCK;
	fcntl(sockseq,F_SETLKW,&fl);
	close(sockseq);



	#ifdef SERVER_DEBUG
	debugPrint(0,"done getting unix socket");
	#endif
}

void	connection::markDatabaseUnavailable() {

	#ifdef SERVER_DEBUG
	debugPrintStart();
	debugTab(4);
	debugPrint("unlinking ");
	debugPrint(updown);
	debugPrintEnd();
	#endif

	// the database is down if the file isn't there
	unlink(updown);
}

void	connection::markDatabaseAvailable() {

	#ifdef SERVER_DEBUG
	debugPrintStart();
	debugTab(4);
	debugPrint("creating ");
	debugPrint(updown);
	debugPrintEnd();
	#endif

	// the database is up if the file is there, 
	// opening and closing it will create it
	ofstream	updownfile(updown);
	updownfile.close();
}

void	connection::waitForAvailableDatabase() {

	#ifdef SERVER_DEBUG
	debugPrint(0,"waiting for available database...");
	#endif

	// if the file is there, return, otherwise reconnect to the database
	ifstream	updownfile(updown);
	if (updownfile && updownfile.good()) {

		updownfile.close();

		#ifdef SERVER_DEBUG
		debugPrint(0,"database is available");
		#endif

		return;
	}

	#ifdef SERVER_DEBUG
	debugPrint(0,"database is down");
	#endif

	reLogIn();
	markDatabaseAvailable();

	#ifdef SERVER_DEBUG
	debugPrint(0,"database is available");
	#endif
}

int	connection::isTransactional() {
	return 1;
}

void	connection::initSession() {

	#ifdef SERVER_DEBUG
	debugPrint(0,"initializing session...");
	#endif

	commitorrollback=0;
	suspendsession=0;
	for (int i=0; i<cursorcount; i++) {
		cur[i]->suspendresultset=0;
	}
	accepttimeout=5;

	#ifdef SERVER_DEBUG
	debugPrint(0,"done initializing session...");
	#endif
}

void	connection::commitOrRollback() {

	#ifdef SERVER_DEBUG
	debugPrint(2,"commit or rollback check...");
	#endif

	// if the query was a commit or rollback, set a flag indicating so
	if (isTransactional()) {
		if (cur[currentcur]->queryIsCommitOrRollback()) {
			#ifdef SERVER_DEBUG
			debugPrint(3,"commit or rollback not needed");
			#endif
			commitorrollback=0;
		} else if (cur[currentcur]->queryIsNotSelect()) {
			#ifdef SERVER_DEBUG
			debugPrint(3,"commit or rollback needed");
			#endif
			commitorrollback=1;
		}
	}

	#ifdef SERVER_DEBUG
	debugPrint(2,"done with commit or rollback check");
	#endif
}

void	connection::setAutoCommitBehavior(int ac) {
	autocommit=ac;
}

int	connection::getAutoCommitBehavior() {
	return autocommit;
}

unsigned short	connection::autoCommitOn() {
	checkautocommit=1;
	performautocommit=1;
	return 1;
}

unsigned short	connection::autoCommitOff() {
	checkautocommit=1;
	performautocommit=0;
	return 1;
}

int	connection::commit() {

	#ifdef SERVER_DEBUG
	debugPrint(1,"commit...");
	#endif

	cur[currentcur]->prepareQuery("commit",6);
	int	retval=cur[currentcur]->executeQuery("commit",6);
	cur[currentcur]->abort();

	#ifdef SERVER_DEBUG
	debugPrintStart();
	debugTab(2);
	debugPrint("commit result: ");
	debugPrint((long)retval);
	debugPrintEnd();
	#endif

	return retval;
}

int	connection::rollback() {

	#ifdef SERVER_DEBUG
	debugPrint(1,"rollback...");
	#endif

	cur[currentcur]->prepareQuery("rollback",8);
	int	retval=cur[currentcur]->executeQuery("rollback",8);
	cur[currentcur]->abort();

	#ifdef SERVER_DEBUG
	debugPrintStart();
	debugTab(2);
	debugPrint("rollback result: ");
	debugPrint((long)retval);
	debugPrintEnd();
	#endif

	return retval;
}

#ifdef SERVER_DEBUG
void	connection::openDebugFile() {

	char	*debugfilename=new char[strlen(DEBUG_DIR)+17+20+1];
	sprintf(debugfilename,"%s/sqlr-connection.%d",DEBUG_DIR,getpid());

	mode_t	oldumask=umask(066);
	debugfile=new ofstream(debugfilename);
	umask(oldumask);

	if (debugfile && debugfile->good()) {
		cout << "Debugging to: " << debugfilename << ENDL;
		debug=1;
	} else {
		cerr << "Couldn't open debug file: ";
		cerr << debugfilename << ENDL;
		if (debugfile) {
			debugfile->close();
			delete debugfile;
			debugfile=NULL;
		}
		debug=0;
	}

	delete[] debugfilename;
}

void	connection::closeDebugFile() {
	if (debugfile) {
		debugfile->close();
		delete debugfile;
	}
}

void	connection::debugPrintStart() {
	if (!debug) {
		return;
	}
	*debugfile << "connection " << getpid() << ": ";
}

void	connection::debugTab(int tab) {
	if (!debug) {
		return;
	}
	for (int i=0; i<tab; i++) {
		*debugfile << "	";
	}
}

void	connection::debugPrint(char *string) {
	if (!debug) {
		return;
	}
	*debugfile << string;
}

void	connection::debugPrint(long number) {
	if (!debug) {
		return;
	}
	*debugfile << number;
}

void	connection::debugPrint(double number) {
	if (!debug) {
		return;
	}
	*debugfile << number;
}

void	connection::debugPrint(int tab, char *string) {
	if (!debug) {
		return;
	}
	debugPrintStart();
	debugTab(tab);
	*debugfile << string;
	debugPrintEnd();
}

void	connection::debugPrint(int tab, long number) {
	if (!debug) {
		return;
	}
	debugPrintStart();
	debugTab(tab);
	*debugfile << number;
	debugPrintEnd();
}

void	connection::debugPrint(int tab, double number) {
	if (!debug) {
		return;
	}
	debugPrintStart();
	debugTab(tab);
	*debugfile << number;
	debugPrintEnd();
}

void	connection::debugPrintEnd() {
	if (!debug) {
		return;
	}
	*debugfile << ENDL;
	debugfile->flush();
}
#endif
