// Copyright (c) 1999-2001  David Muse
// See the file COPYING for more information

#ifndef CONNECTION_H
#define CONNECTION_H

#include <config.h>
#include <defaults.h>
#include <rudiments/signalclasses.h>
#include <rudiments/daemonprocess.h>
#include <rudiments/serversocketpool.h>
#include <rudiments/clientsocket.h>
#include <rudiments/semaphoreset.h>
#include <rudiments/sharedmemory.h>
#include <rudiments/commandline.h>
#include <authenticator.h>

#include <cursor.h>

#include <strstream.h>
#include <fstream.h>

#include <defines.h>

class connection : public daemonprocess, public serversocketpool {
	public:
			connection();
		virtual	~connection();

			int	initConnection(int argc, char **argv);
			void	listen();
			void	closeConnection();
			char	*connectStringValue(char *variable);


	protected:
		// interface definition
		virtual	int	getNumberOfConnectStringVars()=0;
		virtual	void	handleConnectString()=0;
		virtual	int	logIn()=0;
		virtual	void	logOut()=0;
		virtual	int	changeUser(char *newuser, char *newpassword);
		virtual unsigned short	autoCommitOn();
		virtual unsigned short	autoCommitOff();
		virtual int	commit();
		virtual int	rollback();
		virtual int	ping()=0;
		virtual char	*identify()=0;
		virtual cursor	*initCursor()=0;
		virtual void	deleteCursor(cursor *curs)=0;
		virtual	short	nonNullBindValue();
		virtual	short	nullBindValue();
		virtual char	bindVariablePrefix();
		virtual int	bindValueIsNull(short isnull);
		virtual	int	skipRows(int rows);
		virtual int	isTransactional();

	public:
			// methods used by derived classes
			void	setAutoCommitBehavior(int ac);
			int	getAutoCommitBehavior();
			int	setEnv(char *var, char *string, char *pestr);
			int	authOnConnection();
			int	sendColumnInfo();
			void	sendRowCounts(long actual, 
						long affected);
			void	sendColumnCount(unsigned long ncols);
			void	sendColumnDefinition(char *name, 
						unsigned short namelen, 
						unsigned short type, 
						unsigned long size);
			void	sendField(char *data, unsigned long size);
			void	sendNullField();
			void	startSendingLong();
			void	sendLongSegment(char *data, unsigned long size);
			void	endSendingLong();

	private:
			// methods used internally
			int	initCursors();
			void	closeCursors();
			void	parseConnectString(char *connectstring);
			void	createSharedMemoryAndSemaphores();
			void	reLogIn();
			void	announceAvailability();
			void	registerForHandoff();
			void	deRegisterForHandoff();
			void	initSession();
			int	waitForClient();
			void	clientSession();
			int	authenticate();
			int	handleQuery(int cursorstatus);
			void	resumeResultSet();
			void	suspendSession();
			void	endSession();
			int	getCommand();
			int	getCursor();
			void	noAvailableCursors();
			int	getQuery();
			int	getInputBinds();
			int	getOutputBinds();
			int	getSendColumnInfo();
			int	processQuery();
			int	handleBinds();
			void	commitOrRollback();
			int	handleError();
			int	returnError();
			void	returnResultSet();
			void	returnOutputBindValues();
			void	returnResultSetHeader();
			int	returnResultSetData();

			long	rowsToFetch();
			long	rowsToSkip();

			void	markDatabaseUnavailable();
			void	markDatabaseAvailable();
			void	waitForAvailableDatabase();

			void	getUnixSocket();

			void	incrementConnectionCount();
			void	decrementConnectionCount();
			void	decrementSessionCount();

			void	blockSignals();
			void	openSockets();

		int		tmpdirlen;

		commandline	*cmdl;

		unsigned short	inetport;
		char		*unixsocket;
		char		*unixsocketptr;
		char		*sockseqname;

		char		*connectstring;
		int		connectstringcount;
		strstream	**connectstringvar;
		strstream	**connectstringval;

		int		ttl;

		unsigned short	sendcolumninfo;

		char		*id;
		char		*idfilename;

		char		*updown;

		semaphoreset	*semset;

		sharedmemory	*idmemory;

		int		init;

		unsigned short	command;

		int		authonconn;

		authenticator	*authc;

		char		userbuffer[USERSIZE+1];
		char		passwordbuffer[USERSIZE+1];

		char		lastuserbuffer[USERSIZE+1];
		char		lastpasswordbuffer[USERSIZE+1];
		int		lastauthsuccess;

		int		commitorrollback;
		int		eoscommit;
		int		autocommit;
		int		checkautocommit;
		int		performautocommit;
		int		sessiontimeout;
		int		accepttimeout;
		int		suspendsession;
		long		lastrow;

		int		dynamicscaling;

		serversocket	*serversockin;
		serversocket	*serversockun;
		genericsocket	*clientsock;
		clientsocket	*handoffsockun;

		int		listenoninet;
		int		listenonunix;

		int		passdescriptor;

	protected:
		int		cursorcount;
		cursor		**cur;
		unsigned short	currentcur;

#ifdef SERVER_DEBUG
	protected:
		int		debug;
		ofstream	*debugfile;

		void		openDebugFile();
		void		debugPrintStart();
		void		debugTab(int tab);
		void		debugPrint(char *string);
		void		debugPrint(long number);
		void		debugPrint(double number);
		void		debugPrint(int tab, char *string);
		void		debugPrint(int tab, long number);
		void		debugPrint(int tab, double number);
		void		debugPrintEnd();
		void		closeDebugFile();
#endif
};

#endif
