// Copyright (c) 1999-2001  David Muse
// See the file COPYING for more information

#include <config.h>

#include <cursor.h>

#include <iomanip.h>
#include <stdlib.h>

#include <math.h>
#include <string.h>
#ifdef HAVE_STRINGS_H
	#include <strings.h>
#endif

cursor::cursor(connection *conn) {
	this->conn=conn;
	inbindcount=0;
	outbindcount=0;
	busy=0;
}

cursor::~cursor() {}

int	cursor::openCursor(int id) {
	// by default do nothing
	return 1;
}

int	cursor::closeCursor() {
	// by default do nothing
	return 1;
}

int	cursor::prepareQuery(char *query, long querylength) {
	// by default, do nothing...
	return 1;
}

int	cursor::handleBinds() {
	
	// iterate through the arrays, binding values to variables
	for (int i=0; i<inbindcount; i++) {

		// bind the value to the variable
		if (inbindvars[i].type==STRING_BIND ||
				inbindvars[i].type==NULL_BIND) {
			if (!inputBindValue(inbindvars[i].variable,
				inbindvars[i].variablesize,
				inbindvars[i].value.stringval,
				inbindvars[i].valuesize,
				&inbindvars[i].isnull)) {
				return 0;
			}
		} else if (inbindvars[i].type==LONG_BIND) {
			if (!inputBindValue(inbindvars[i].variable,
				inbindvars[i].variablesize,
				&inbindvars[i].value.longval)) {
				return 0;
			}
		} else if (inbindvars[i].type==DOUBLE_BIND) {
			if (!inputBindValue(inbindvars[i].variable,
				inbindvars[i].variablesize,
				&inbindvars[i].value.doubleval.value,
				inbindvars[i].value.doubleval.precision,
				inbindvars[i].value.doubleval.scale)) {
				return 0;
			}
		}
	}
	for (int i=0; i<outbindcount; i++) {

		// bind the value to the variable
		if (!outputBindValue(outbindvars[i].variable,
				outbindvars[i].variablesize,
				outbindvars[i].value.stringval,
				outbindvars[i].valuesize,
				&outbindvars[i].isnull)) {
			return 0;
		}
	}
	return 1;
}

int	cursor::inputBindValue(char *variable, unsigned short variablesize,
					char *value, unsigned short valuesize,
					short *isnull) {
	// by default, do nothing...
	return 1;
}

int	cursor::inputBindValue(char *variable, unsigned short variablesize,
					unsigned long *value) {
	// by default, do nothing...
	return 1;
}

int	cursor::inputBindValue(char *variable, unsigned short variablesize,
					double *value, 
					unsigned short precision,
					unsigned short scale) {
	// by default, do nothing...
	return 1;
}

int	cursor::outputBindValue(char *variable, unsigned short variablesize,
				char *value, 
				unsigned short valuesize, 
				short *isnull) {
	// by default, do nothing...
	return 1;
}

strstream	*cursor::fakeInputBinds(char *query) {

	// return null if there aren't any input binds
	if (!inbindcount) {
		return NULL;
	}

	strstream	*outputquery=new strstream();

	// loop through the query, performing substitutions
	char	prefix=inbindvars[0].variable[0];
	char	*ptr=query;
	int	index=1;
	int	inquotes=0;
	while (*ptr) {

		// are we inside of quotes ?
		if (*ptr=='\'') {
			if (inquotes) {
				inquotes==0;
			} else {
				inquotes==1;
			}
		}

		// look for the bind var prefix or ? if not inside of quotes
		if (!inquotes && (*ptr==prefix || *ptr=='?')) {

			// look through the list of vars
			int	match=0;
			for (int i=0; i<inbindcount; i++) {

				// if we find a match, perform the substitution
				// and skip past the variable...
				//
				// for bind-by-name cases, we need to check
				// after the variable for whitespace, a comma
				// or a right parenthesis to make sure that we 
				// don't make the following mistake:
				//
				// select :var1,:var15 from mytable
				//
				// :var1=3
				// :var15=2
				//
				// correct:
				//      select 3,2 from mytable
				//
				// mistake:
				//      select 3,35 from mytable
				if (
					(*ptr=='?' && 
					atoi(inbindvars[i].variable+1)==index) 

					||

					(!strncmp(ptr,inbindvars[i].variable,
						inbindvars[i].variablesize) 
					 		&&
					(*(ptr+inbindvars[i].variablesize)==
					 		' ' ||
					*(ptr+inbindvars[i].variablesize)==
							'	' ||
					*(ptr+inbindvars[i].variablesize)==
							'\n' ||
					*(ptr+inbindvars[i].variablesize)==
							')' ||
					*(ptr+inbindvars[i].variablesize)==
							','))
					) {

					performSubstitution(outputquery,i);
					if (*ptr=='?') {
						ptr++;
					} else {
						ptr=ptr+inbindvars[i].
								variablesize;
					}
					index++;
					break;
				}
			}
		}

		// write the input query to the output query
		if (*ptr) {
			*outputquery << *ptr;
			ptr++;
		}
	}

	*outputquery << ENDS;

	return outputquery;
}

void	cursor::performSubstitution(strstream *buffer, int which) {
	if (inbindvars[which].type==STRING_BIND) {
		*buffer << "'" << inbindvars[which].value.stringval << "'";
	} else if (inbindvars[which].type==LONG_BIND) {
		*buffer << inbindvars[which].value.longval;
	} else if (inbindvars[which].type==DOUBLE_BIND) {
		*buffer << setprecision(inbindvars[which].
					value.doubleval.precision);
		*buffer << floor(inbindvars[which].value.doubleval.value*
			pow(10,inbindvars[which].value.doubleval.scale))/
			pow(10,inbindvars[which].value.doubleval.scale);
	}
}

int	cursor::queryIsNotSelect() {

	// scan the query, bypassing whitespace and comments.
	// look case insensitively for SELECT, return 0 if it's found,
	// 1 if it's not
	char	*ptr=querybuffer;
	while (*ptr && 
		(*ptr==' ' || *ptr=='\n' || *ptr=='	' || *ptr=='-')) {

		// skip any comments
		if (*ptr=='-') {
			while (*ptr && *ptr!='\n') {
				ptr++;
			}
		}
		ptr++;
	}

	// if the query is a select but not a select into then return false,
	// otherwise return true
	if (!strncasecmp(ptr,"select",6) && 
			strncasecmp(ptr,"select into ",12)) {
		return 0;
	}
	return 1;
}

int	cursor::queryIsCommitOrRollback() {

	// scan the query, bypassing whitespace and comments.
	// look case insensitively for SELECT, return 0 if it's found,
	// 1 if it's not
	char	*ptr=querybuffer;
	while (*ptr && 
		(*ptr==' ' || *ptr=='\n' || *ptr=='	' || *ptr=='-')) {

		// skip any comments
		if (*ptr=='-') {
			while (*ptr && *ptr!='\n') {
				ptr++;
			}
		}
		ptr++;
	}

	// if the query is a commit or rollback, return true
	// otherwise return false
	if (!strncasecmp(ptr,"commit",6) || !strncasecmp(ptr,"rollback",8)) {
		return 1;
	}
	return 0;
}

void	cursor::cleanUpData() {
	// by default, do nothing...
	return;
}

void	cursor::abort() {
	cleanUpData();
	suspendresultset=0;
	busy=0;
}
