// Copyright (c) 1999-2001  David Muse
// See the file COPYING for more information

#ifndef CURSOR_H
#define CURSOR_H

#include <defines.h>

#include <strstream.h>

enum bindtype {
	NULL_BIND,
	STRING_BIND,
	LONG_BIND,
	DOUBLE_BIND
};

class bindvar {
	friend class connection;
	friend class cursor;
	private:
		char		variable[BINDVARLENGTH+2];
		short		variablesize;
		union {
			char		stringval[BINDVALUELENGTH+1];
			unsigned long	longval;
			struct	{
				double		value;
				unsigned short	precision;
				unsigned short	scale;
			} doubleval;
		} value;
		unsigned short	valuesize;
		bindtype	type;
		short		isnull;
};

class connection;

class cursor {
	friend class connection;
	public:
			cursor(connection *conn);
		virtual	~cursor();

	protected:
		// interface definition
		virtual	int	openCursor(int id);
		virtual	int	closeCursor();
		virtual	int	prepareQuery(char *query, long length);
		virtual	int	inputBindValue(char *variable, 
						unsigned short variablesize,
						char *value, 
						unsigned short valuesize,
						short *isnull);
		virtual	int	inputBindValue(char *variable, 
						unsigned short variablesize,
						unsigned long *value);
		virtual	int	inputBindValue(char *variable, 
						unsigned short variablesize,
						double *value,
						unsigned short precision,
						unsigned short scale);
		virtual	int	outputBindValue(char *variable, 
						unsigned short variablesize,
						char *value,
						unsigned short valuesize,
						short *isnull);
		virtual	int	executeQuery(char *query, long length)=0;
		virtual	int	queryIsNotSelect();
		virtual	int	queryIsCommitOrRollback();
		virtual	char	*getErrorMessage(int *liveconnection)=0;
		virtual	void	returnRowCounts()=0;
		virtual	void	returnColumnCount()=0;
		virtual	void	returnColumnInfo()=0;
		virtual	int	noRowsToReturn()=0;
		virtual	int	skipRow()=0;
		virtual	int	fetchRow()=0;
		virtual	void	returnRow()=0;
		virtual	void	cleanUpData();

	public:
		// methods/variables used by derived classes
		strstream	*fakeInputBinds(char *query);

		connection	*conn;

	private:
		// methods used internally
		int	handleBinds();
		void	performSubstitution(strstream *buffer, int which);
		void	abort();

		char		querybuffer[MAXQUERYSIZE+1];
		unsigned long	querylength;
		unsigned short	reexecute;

		bindvar		inbindvars[MAXVAR];
		unsigned short	inbindcount;
		bindvar		outbindvars[MAXVAR];
		unsigned short	outbindcount;

		int		suspendresultset;
		int		busy;
};

#endif
