// Copyright (c) 1999-2001  David Muse
// See the file COPYING for more information

#ifndef LISTENER_H
#define LISTENER_H

#include <config.h>
#include <defaults.h>
#include <rudiments/signalclasses.h>
#include <rudiments/daemonprocess.h>
#include <rudiments/serversocketpool.h>
#include <rudiments/semaphoreset.h>
#include <rudiments/sharedmemory.h>
#include <authenticator.h>
#include <rudiments/regularexpression.h>

#include <fstream.h>

#include <defines.h>

class handoffsocketnode {
	friend class listener;
	private:
		unsigned long		pid;
		genericsocket		*sock;
};

class listener : public daemonprocess, public serversocketpool {

	public:
			listener();
			~listener();
		int	initListener(int argc, char **argv);
		void	listen();
		char	*getUnixServerPort();

	private:
		void	cleanUp();

		void	registerHandoff(genericsocket *sock);
		void	deRegisterHandoff(genericsocket *sock);
		int	deniedIp();
		void	disconnectClient();
		int	getAuth();
		void	getAConnection();
		int	passDescriptor();
		handoffsocketnode	*deleteNode(handoffsocketnode *node);
		void	incrementSessionCount();

		void	blockSignals();

		int		tmpdirlen;

		char		*pidfile;

		char		*id;
		char		*config;

		int		authenticate;
		int		passdescriptor;

		int		connections;
		int		maxconnections;
		int		maxqueuelength;
		int		growby;
		int		ttl;
		char		*runasuser;
		char		*runasgroup;
		int		dynamicscaling;

		char		*unixport;
		unsigned short	inetport;
		unsigned long	pid;

		authenticator	*authc;
		char		userbuffer[USERSIZE+1];
		char		passwordbuffer[USERSIZE+1];

		char		unixportstr[MAXPATHLEN+1];
		unsigned short	unixportstrlen;

		semaphoreset	*semset;
		sharedmemory	*idmemory;

		int		initialized;

		serversocket	*clientsockun;
		serversocket	*clientsockin;
		serversocket	*handoffsockun;
		serversocket	*removehandoffsockun;
		genericsocket	*clientsock;

		handoffsocketnode	**handoffsocklist;

		regularexpression	*allowed;
		regularexpression	*denied;

#ifdef SERVER_DEBUG
	protected:
		int		debug;
		ofstream	*debugfile;

		void		openDebugFile();
		void		closeDebugFile();
		void		debugPrintStart();
		void		debugTab(int tab);
		void		debugPrint(char *string);
		void		debugPrint(long number);
		void		debugPrint(double number);
		void		debugPrint(int tab, char *string);
		void		debugPrint(int tab, long number);
		void		debugPrint(int tab, double number);
		void		debugPrintEnd();
#endif
};

#endif
