// Copyright (c) 2001  David Muse
// See the file COPYING for more information.

#include <sqlrelay/sqlrclient.h>
#include <string.h>
#include <stdlib.h>

#include <iostream.h>

sqlrconnection	*con;
sqlrcursor	*cur;
sqlrconnection	*secondcon;
sqlrcursor	*secondcur;

void checkSuccess(char *value, char *success) {

	if (!success) {
		if (!value) {
			cout << "success ";
			return;
		} else {
			cout << "failure ";
			delete cur;
			delete con;
			exit(0);
		}
	}

	if (!strcmp(value,success)) {
		cout << "success ";
	} else {
		cout << "failure ";
		delete cur;
		delete con;
		exit(0);
	}
}

void checkSuccess(int value, int success) {

	if (value==success) {
		cout << "success ";
	} else {
		cout << "failure ";
		delete cur;
		delete con;
		exit(0);
	}
}

int	main(int argc, char **argv) {

	char	*dbtype;
	char	*bindvars[11]={"1","2","3","4","5","6","7","8","9","10",NULL};
	char	*bindvals[10]={"4","4","4","4.4","4.4","4.4",
			"testchar4","testvarchar4","01/01/2004","04:00:00"};
	char	*subvars[4]={"var1","var2","var3",NULL};
	char	*subvalstrings[3]={"hi","hello","bye"};
	long	subvallongs[3]={1,2,3};
	double	subvaldoubles[3]={10.55,10.556,10.5556};
	unsigned short	precs[3]={4,5,6};
	unsigned short	scales[3]={2,3,4};
	char	*numvar;
	char	*stringvar;
	char	*floatvar;
	char	**cols;
	char	**fields;
	int	port;
	char	*socket;
	int	id;
	char	*filename;
	long	*fieldlens;

	// usage...
	if (argc<5) {
		cout << "usage: db2 host port socket user password" << endl;
		exit(0);
	}

	// instantiation
	con=new sqlrconnection(argv[1],atoi(argv[2]), 
					argv[3],argv[4],argv[5],0,1);
	cur=new sqlrcursor(con);

	// get database type
	cout << "IDENTIFY: " << endl;
	checkSuccess(con->identify(),"db2");
	cout << endl;

	// ping
	cout << "PING: " << endl;
	checkSuccess(con->ping(),1);
	cout << endl;

	// drop existing table
	cur->sendQuery("drop table testtable");

	cout << "CREATE TEMPTABLE: " << endl;
	checkSuccess(cur->sendQuery("create table testtable (testsmallint smallint, testint integer, testbigint bigint, testdecimal decimal(10,2), testreal real, testdouble double, testchar char(40), testvarchar varchar(40), testdate date, testtime time, testtimestamp timestamp)"),1);
	cout << endl;

	cout << "INSERT: " << endl;
	checkSuccess(cur->sendQuery("insert into testtable values (1,1,1,1.1,1.1,1.1,'testchar1','testvarchar1','01/01/2001','01:00:00',NULL)"),1);
	cout << endl;

	cout << "BIND BY POSITION: " << endl;
	cur->prepareQuery("insert into testtable values (?,?,?,?,?,?,?,?,?,?,NULL)");
	cur->inputBind("1",2);
	cur->inputBind("2",2);
	cur->inputBind("3",2);
	cur->inputBind("4",2.2,4,2);
	cur->inputBind("5",2.2,4,2);
	cur->inputBind("6",2.2,4,2);
	cur->inputBind("7","testchar2");
	cur->inputBind("8","testvarchar2");
	cur->inputBind("9","01/01/2002");
	cur->inputBind("10","02:00:00");
	checkSuccess(cur->executeQuery(),1);
	cur->clearBinds();
	cur->inputBind("1",3);
	cur->inputBind("2",3);
	cur->inputBind("3",3);
	cur->inputBind("4",3.3,4,2);
	cur->inputBind("5",3.3,4,2);
	cur->inputBind("6",3.3,4,2);
	cur->inputBind("7","testchar3");
	cur->inputBind("8","testvarchar3");
	cur->inputBind("9","01/01/2003");
	cur->inputBind("10","03:00:00");
	checkSuccess(cur->executeQuery(),1);
	cout << endl;

	cout << "ARRAY OF BINDS BY POSITION: " << endl;
	cur->clearBinds();
	cur->inputBinds(bindvars,bindvals);
	checkSuccess(cur->executeQuery(),1);
	cout << endl;

	cout << "INSERT: " << endl;
	checkSuccess(cur->sendQuery("insert into testtable values (5,5,5,5.5,5.5,5.5,'testchar5','testvarchar5','01/01/2005','05:00:00',NULL)"),1);
	checkSuccess(cur->sendQuery("insert into testtable values (6,6,6,6.6,6.6,6.6,'testchar6','testvarchar6','01/01/2006','06:00:00',NULL)"),1);
	checkSuccess(cur->sendQuery("insert into testtable values (7,7,7,7.7,7.7,7.7,'testchar7','testvarchar7','01/01/2007','07:00:00',NULL)"),1);
	checkSuccess(cur->sendQuery("insert into testtable values (8,8,8,8.8,8.8,8.8,'testchar8','testvarchar8','01/01/2008','08:00:00',NULL)"),1);
	cout << endl;

	cout << "AFFECTED ROWS: " << endl;
	checkSuccess(cur->affectedRows(),1);
	cout << endl;

	cout << "STORED PROCEDURE: " << endl;
	checkSuccess(cur->sendQuery("create procedure testproc(in invar int, out outvar int) language sql begin set outvar = invar; end"),1);
	cur->prepareQuery("call testproc(?,?)");
	cur->inputBind("1",5);
	cur->defineOutputBind("2",10);
	checkSuccess(cur->executeQuery(),1);
	checkSuccess(cur->getOutputBind("2"),"5");
	checkSuccess(cur->sendQuery("drop procedure testproc"),1);
	cout << endl;

	cout << "SELECT: " << endl;
	checkSuccess(cur->sendQuery("select * from testtable order by testsmallint"),1);
	cout << endl;

	cout << "COLUMN COUNT: " << endl;
	checkSuccess(cur->colCount(),11);
	cout << endl;

	cout << "COLUMN NAMES: " << endl;
	checkSuccess(cur->getColumnName(0),"TESTSMALLINT");
	checkSuccess(cur->getColumnName(1),"TESTINT");
	checkSuccess(cur->getColumnName(2),"TESTBIGINT");
	checkSuccess(cur->getColumnName(3),"TESTDECIMAL");
	checkSuccess(cur->getColumnName(4),"TESTREAL");
	checkSuccess(cur->getColumnName(5),"TESTDOUBLE");
	checkSuccess(cur->getColumnName(6),"TESTCHAR");
	checkSuccess(cur->getColumnName(7),"TESTVARCHAR");
	checkSuccess(cur->getColumnName(8),"TESTDATE");
	checkSuccess(cur->getColumnName(9),"TESTTIME");
	checkSuccess(cur->getColumnName(10),"TESTTIMESTAMP");
	cols=cur->getColumnNames();
	checkSuccess(cols[0],"TESTSMALLINT");
	checkSuccess(cols[1],"TESTINT");
	checkSuccess(cols[2],"TESTBIGINT");
	checkSuccess(cols[3],"TESTDECIMAL");
	checkSuccess(cols[4],"TESTREAL");
	checkSuccess(cols[5],"TESTDOUBLE");
	checkSuccess(cols[6],"TESTCHAR");
	checkSuccess(cols[7],"TESTVARCHAR");
	checkSuccess(cols[8],"TESTDATE");
	checkSuccess(cols[9],"TESTTIME");
	checkSuccess(cols[10],"TESTTIMESTAMP");
	cout << endl;

	cout << "COLUMN TYPES: " << endl;
	checkSuccess(cur->getColumnType(0),"SMALLINT");
	checkSuccess(cur->getColumnType("TESTSMALLINT"),"SMALLINT");
	checkSuccess(cur->getColumnType(1),"INTEGER");
	checkSuccess(cur->getColumnType("TESTINT"),"INTEGER");
	checkSuccess(cur->getColumnType(2),"BIGINT");
	checkSuccess(cur->getColumnType("TESTBIGINT"),"BIGINT");
	checkSuccess(cur->getColumnType(3),"DECIMAL");
	checkSuccess(cur->getColumnType("TESTDECIMAL"),"DECIMAL");
	checkSuccess(cur->getColumnType(4),"REAL");
	checkSuccess(cur->getColumnType("TESTREAL"),"REAL");
	checkSuccess(cur->getColumnType(5),"DOUBLE");
	checkSuccess(cur->getColumnType("TESTDOUBLE"),"DOUBLE");
	checkSuccess(cur->getColumnType(6),"CHAR");
	checkSuccess(cur->getColumnType("TESTCHAR"),"CHAR");
	checkSuccess(cur->getColumnType(7),"VARCHAR");
	checkSuccess(cur->getColumnType("TESTVARCHAR"),"VARCHAR");
	checkSuccess(cur->getColumnType(8),"DATE");
	checkSuccess(cur->getColumnType("TESTDATE"),"DATE");
	checkSuccess(cur->getColumnType(9),"TIME");
	checkSuccess(cur->getColumnType("TESTTIME"),"TIME");
	checkSuccess(cur->getColumnType(10),"TIMESTAMP");
	checkSuccess(cur->getColumnType("TESTTIMESTAMP"),"TIMESTAMP");
	cout << endl;

	cout << "COLUMN LENGTH: " << endl;
	checkSuccess(cur->getColumnLength(0),2);
	checkSuccess(cur->getColumnLength("TESTSMALLINT"),2);
	checkSuccess(cur->getColumnLength(1),4);
	checkSuccess(cur->getColumnLength("TESTINT"),4);
	checkSuccess(cur->getColumnLength(2),8);
	checkSuccess(cur->getColumnLength("TESTBIGINT"),8);
	checkSuccess(cur->getColumnLength(3),12);
	checkSuccess(cur->getColumnLength("TESTDECIMAL"),12);
	checkSuccess(cur->getColumnLength(4),4);
	checkSuccess(cur->getColumnLength("TESTREAL"),4);
	checkSuccess(cur->getColumnLength(5),8);
	checkSuccess(cur->getColumnLength("TESTDOUBLE"),8);
	checkSuccess(cur->getColumnLength(6),40);
	checkSuccess(cur->getColumnLength("TESTCHAR"),40);
	checkSuccess(cur->getColumnLength(7),40);
	checkSuccess(cur->getColumnLength("TESTVARCHAR"),40);
	checkSuccess(cur->getColumnLength(8),6);
	checkSuccess(cur->getColumnLength("TESTDATE"),6);
	checkSuccess(cur->getColumnLength(9),6);
	checkSuccess(cur->getColumnLength("TESTTIME"),6);
	checkSuccess(cur->getColumnLength(10),16);
	checkSuccess(cur->getColumnLength("TESTTIMESTAMP"),16);
	cout << endl;

	cout << "LONGEST COLUMN: " << endl;
	checkSuccess(cur->getLongest(0),1);
	checkSuccess(cur->getLongest("TESTSMALLINT"),1);
	checkSuccess(cur->getLongest(1),1);
	checkSuccess(cur->getLongest("TESTINT"),1);
	checkSuccess(cur->getLongest(2),1);
	checkSuccess(cur->getLongest("TESTBIGINT"),1);
	checkSuccess(cur->getLongest(3),4);
	checkSuccess(cur->getLongest("TESTDECIMAL"),4);
	//checkSuccess(cur->getLongest(4),3);
	//checkSuccess(cur->getLongest("TESTREAL"),3);
	//checkSuccess(cur->getLongest(5),3);
	//checkSuccess(cur->getLongest("TESTDOUBLE"),3);
	checkSuccess(cur->getLongest(6),40);
	checkSuccess(cur->getLongest("TESTCHAR"),40);
	checkSuccess(cur->getLongest(7),12);
	checkSuccess(cur->getLongest("TESTVARCHAR"),12);
	checkSuccess(cur->getLongest(8),10);
	checkSuccess(cur->getLongest("TESTDATE"),10);
	checkSuccess(cur->getLongest(9),8);
	checkSuccess(cur->getLongest("TESTTIME"),8);
	cout << endl;

	cout << "ROW COUNT: " << endl;
	checkSuccess(cur->rowCount(),8);
	cout << endl;

	cout << "TOTAL ROWS: " << endl;
	checkSuccess(cur->totalRows(),-1);
	cout << endl;

	cout << "FIRST ROW INDEX: " << endl;
	checkSuccess(cur->firstRowIndex(),0);
	cout << endl;

	cout << "END OF RESULT SET: " << endl;
	checkSuccess(cur->endOfResultSet(),1);
	cout << endl;

	cout << "FIELDS BY INDEX: " << endl;
	checkSuccess(cur->getField(0,0),"1");
	checkSuccess(cur->getField(0,1),"1");
	checkSuccess(cur->getField(0,2),"1");
	checkSuccess(cur->getField(0,3),"1.10");
	//checkSuccess(cur->getField(0,4),"1.1");
	//checkSuccess(cur->getField(0,5),"1.1");
	checkSuccess(cur->getField(0,6),"testchar1                               ");
	checkSuccess(cur->getField(0,7),"testvarchar1");
	checkSuccess(cur->getField(0,8),"2001-01-01");
	checkSuccess(cur->getField(0,9),"01:00:00");
	cout << endl;
	checkSuccess(cur->getField(7,0),"8");
	checkSuccess(cur->getField(7,1),"8");
	checkSuccess(cur->getField(7,2),"8");
	checkSuccess(cur->getField(7,3),"8.80");
	//checkSuccess(cur->getField(7,4),"8.8");
	//checkSuccess(cur->getField(7,5),"8.8");
	checkSuccess(cur->getField(7,6),"testchar8                               ");
	checkSuccess(cur->getField(7,7),"testvarchar8");
	checkSuccess(cur->getField(7,8),"2008-01-01");
	checkSuccess(cur->getField(7,9),"08:00:00");
	cout << endl;

	cout << "FIELD LENGTHS BY INDEX: " << endl;
	checkSuccess(cur->getFieldLength(0,0),1);
	checkSuccess(cur->getFieldLength(0,1),1);
	checkSuccess(cur->getFieldLength(0,2),1);
	checkSuccess(cur->getFieldLength(0,3),4);
	//checkSuccess(cur->getFieldLength(0,4),3);
	//checkSuccess(cur->getFieldLength(0,5),3);
	checkSuccess(cur->getFieldLength(0,6),40);
	checkSuccess(cur->getFieldLength(0,7),12);
	checkSuccess(cur->getFieldLength(0,8),10);
	checkSuccess(cur->getFieldLength(0,9),8);
	cout << endl;
	checkSuccess(cur->getFieldLength(7,0),1);
	checkSuccess(cur->getFieldLength(7,1),1);
	checkSuccess(cur->getFieldLength(7,2),1);
	checkSuccess(cur->getFieldLength(7,3),4);
	//checkSuccess(cur->getFieldLength(7,4),3);
	//checkSuccess(cur->getFieldLength(7,5),3);
	checkSuccess(cur->getFieldLength(7,6),40);
	checkSuccess(cur->getFieldLength(7,7),12);
	checkSuccess(cur->getFieldLength(7,8),10);
	checkSuccess(cur->getFieldLength(7,9),8);
	cout << endl;

	cout << "FIELDS BY NAME: " << endl;
	checkSuccess(cur->getField(0,"testsmallint"),"1");
	checkSuccess(cur->getField(0,"testint"),"1");
	checkSuccess(cur->getField(0,"testbigint"),"1");
	checkSuccess(cur->getField(0,"testdecimal"),"1.10");
	//checkSuccess(cur->getField(0,"testreal"),"1.1");
	//checkSuccess(cur->getField(0,"testdouble"),"1.1");
	checkSuccess(cur->getField(0,"testchar"),"testchar1                               ");
	checkSuccess(cur->getField(0,"testvarchar"),"testvarchar1");
	checkSuccess(cur->getField(0,"testdate"),"2001-01-01");
	checkSuccess(cur->getField(0,"testtime"),"01:00:00");
	cout << endl;
	checkSuccess(cur->getField(7,"testsmallint"),"8");
	checkSuccess(cur->getField(7,"testint"),"8");
	checkSuccess(cur->getField(7,"testbigint"),"8");
	checkSuccess(cur->getField(7,"testdecimal"),"8.80");
	//checkSuccess(cur->getField(7,"testreal"),"8.8");
	//checkSuccess(cur->getField(7,"testdouble"),"8.8");
	checkSuccess(cur->getField(7,"testchar"),"testchar8                               ");
	checkSuccess(cur->getField(7,"testvarchar"),"testvarchar8");
	checkSuccess(cur->getField(7,"testdate"),"2008-01-01");
	checkSuccess(cur->getField(7,"testtime"),"08:00:00");
	cout << endl;

	cout << "FIELD LENGTHS BY NAME: " << endl;
	checkSuccess(cur->getFieldLength(0,"testsmallint"),1);
	checkSuccess(cur->getFieldLength(0,"testint"),1);
	checkSuccess(cur->getFieldLength(0,"testbigint"),1);
	checkSuccess(cur->getFieldLength(0,"testdecimal"),4);
	//checkSuccess(cur->getFieldLength(0,"testreal"),3);
	//checkSuccess(cur->getFieldLength(0,"testdouble"),3);
	checkSuccess(cur->getFieldLength(0,"testchar"),40);
	checkSuccess(cur->getFieldLength(0,"testvarchar"),12);
	checkSuccess(cur->getFieldLength(0,"testdate"),10);
	checkSuccess(cur->getFieldLength(0,"testtime"),8);
	cout << endl;
	checkSuccess(cur->getFieldLength(7,"testsmallint"),1);
	checkSuccess(cur->getFieldLength(7,"testint"),1);
	checkSuccess(cur->getFieldLength(7,"testbigint"),1);
	checkSuccess(cur->getFieldLength(7,"testdecimal"),4);
	//checkSuccess(cur->getFieldLength(7,"testreal"),3);
	//checkSuccess(cur->getFieldLength(7,"testdouble"),3);
	checkSuccess(cur->getFieldLength(7,"testchar"),40);
	checkSuccess(cur->getFieldLength(7,"testvarchar"),12);
	checkSuccess(cur->getFieldLength(7,"testdate"),10);
	checkSuccess(cur->getFieldLength(7,"testtime"),8);
	cout << endl;

	cout << "FIELDS BY ARRAY: " << endl;
	fields=cur->getRow(0);
	checkSuccess(fields[0],"1");
	checkSuccess(fields[1],"1");
	checkSuccess(fields[2],"1");
	checkSuccess(fields[3],"1.10");
	//checkSuccess(fields[4],"1.1");
	//checkSuccess(fields[5],"1.1");
	checkSuccess(fields[6],"testchar1                               ");
	checkSuccess(fields[7],"testvarchar1");
	checkSuccess(fields[8],"2001-01-01");
	checkSuccess(fields[9],"01:00:00");
	cout << endl;

	cout << "FIELD LENGTHS BY ARRAY: " << endl;
	fieldlens=cur->getRowLengths(0);
	checkSuccess(fieldlens[0],1);
	checkSuccess(fieldlens[1],1);
	checkSuccess(fieldlens[2],1);
	checkSuccess(fieldlens[3],4);
	//checkSuccess(fieldlens[4],3);
	//checkSuccess(fieldlens[5],3);
	checkSuccess(fieldlens[6],40);
	checkSuccess(fieldlens[7],12);
	checkSuccess(fieldlens[8],10);
	checkSuccess(fieldlens[9],8);
	cout << endl;

	cout << "INDIVIDUAL SUBSTITUTIONS: " << endl;
	cur->prepareQuery("values ($(var1),'$(var2)','$(var3)')");
	cur->substitution("var1",1);
	cur->substitution("var2","hello");
	cur->substitution("var3",10.5556,6,4);
	checkSuccess(cur->executeQuery(),1);
	cout << endl;

	cout << "FIELDS: " << endl;
	checkSuccess(cur->getField(0,0),"1");
	checkSuccess(cur->getField(0,1),"hello");
	checkSuccess(cur->getField(0,2),"10.5556");
	cout << endl;

	cout << "ARRAY SUBSTITUTIONS: " << endl;
	cur->prepareQuery("values ('$(var1)','$(var2)','$(var3)')");
	cur->substitutions(subvars,subvalstrings);
	checkSuccess(cur->executeQuery(),1);
	cout << endl;

	cout << "FIELDS: " << endl;
	checkSuccess(cur->getField(0,0),"hi");
	checkSuccess(cur->getField(0,1),"hello");
	checkSuccess(cur->getField(0,2),"bye");
	cout << endl;

	cout << "ARRAY SUBSTITUTIONS: " << endl;
	cur->prepareQuery("values ($(var1),$(var2),$(var3))");
	cur->substitutions(subvars,subvallongs);
	checkSuccess(cur->executeQuery(),1);
	cout << endl;

	cout << "FIELDS: " << endl;
	checkSuccess(cur->getField(0,0),"1");
	checkSuccess(cur->getField(0,1),"2");
	checkSuccess(cur->getField(0,2),"3");
	cout << endl;

	cout << "ARRAY SUBSTITUTIONS: " << endl;
	cur->prepareQuery("values ($(var1),$(var2),$(var3))");
	cur->substitutions(subvars,subvaldoubles,precs,scales);
	checkSuccess(cur->executeQuery(),1);
	cout << endl;

	cout << "FIELDS: " << endl;
	checkSuccess(cur->getField(0,0),"10.55");
	checkSuccess(cur->getField(0,1),"10.556");
	checkSuccess(cur->getField(0,2),"10.5556");
	cout << endl;

	cout << "NULLS as Nulls: " << endl;
	cur->sendQuery("drop table testtable1");
	cur->sendQuery("create table testtable1 (col1 char(1), col2 char(1), col3 char(1))");
	cur->getNullsAsNulls();
	checkSuccess(cur->sendQuery("insert into testtable1 values ('1',NULL,NULL)"),1);
	checkSuccess(cur->sendQuery("select * from testtable1"),1);
	checkSuccess(cur->getField(0,0),"1");
	checkSuccess(cur->getField(0,1),NULL);
	checkSuccess(cur->getField(0,2),NULL);
	cur->getNullsAsEmptyStrings();
	checkSuccess(cur->sendQuery("select * from testtable1"),1);
	checkSuccess(cur->getField(0,0),"1");
	checkSuccess(cur->getField(0,1),"");
	checkSuccess(cur->getField(0,2),"");
	checkSuccess(cur->sendQuery("drop table testtable1"),1);
	cout << endl;
	
	cout << "RESULT SET BUFFER SIZE: " << endl;
	checkSuccess(cur->getResultSetBufferSize(),0);
	cur->setResultSetBufferSize(2);
	checkSuccess(cur->sendQuery("select * from testtable order by testsmallint"),1);
	checkSuccess(cur->getResultSetBufferSize(),2);
	cout << endl;
	checkSuccess(cur->firstRowIndex(),0);
	checkSuccess(cur->endOfResultSet(),0);
	checkSuccess(cur->rowCount(),2);
	checkSuccess(cur->getField(0,0),"1");
	checkSuccess(cur->getField(1,0),"2");
	checkSuccess(cur->getField(2,0),"3");
	cout << endl;
	checkSuccess(cur->firstRowIndex(),2);
	checkSuccess(cur->endOfResultSet(),0);
	checkSuccess(cur->rowCount(),4);
	checkSuccess(cur->getField(6,0),"7");
	checkSuccess(cur->getField(7,0),"8");
	cout << endl;
	checkSuccess(cur->firstRowIndex(),6);
	checkSuccess(cur->endOfResultSet(),0);
	checkSuccess(cur->rowCount(),8);
	checkSuccess(cur->getField(8,0),NULL);
	cout << endl;
	checkSuccess(cur->firstRowIndex(),8);
	checkSuccess(cur->endOfResultSet(),1);
	checkSuccess(cur->rowCount(),8);
	cout << endl;

	cout << "DONT GET COLUMN INFO: " << endl;
	cur->dontGetColumnInfo();
	checkSuccess(cur->sendQuery("select * from testtable order by testsmallint"),1);
	checkSuccess(cur->getColumnName(0),NULL);
	checkSuccess(cur->getColumnLength(0),0);
	checkSuccess(cur->getColumnType(0),NULL);
	cur->getColumnInfo();
	checkSuccess(cur->sendQuery("select * from testtable order by testsmallint"),1);
	checkSuccess(cur->getColumnName(0),"TESTSMALLINT");
	checkSuccess(cur->getColumnLength(0),2);
	checkSuccess(cur->getColumnType(0),"SMALLINT");
	cout << endl;

	cout << "SUSPENDED SESSION: " << endl;
	checkSuccess(cur->sendQuery("select * from testtable order by testsmallint"),1);
	checkSuccess(con->suspendSession(),1);
	port=con->getConnectionPort();
	socket=con->getConnectionSocket();
	checkSuccess(con->resumeSession(port,socket),1);
	cout << endl;
	checkSuccess(cur->getField(0,0),"1");
	checkSuccess(cur->getField(1,0),"2");
	checkSuccess(cur->getField(2,0),"3");
	checkSuccess(cur->getField(3,0),"4");
	checkSuccess(cur->getField(4,0),"5");
	checkSuccess(cur->getField(5,0),"6");
	checkSuccess(cur->getField(6,0),"7");
	checkSuccess(cur->getField(7,0),"8");
	cout << endl;
	checkSuccess(cur->sendQuery("select * from testtable order by testsmallint"),1);
	checkSuccess(con->suspendSession(),1);
	port=con->getConnectionPort();
	socket=con->getConnectionSocket();
	checkSuccess(con->resumeSession(port,socket),1);
	cout << endl;
	checkSuccess(cur->getField(0,0),"1");
	checkSuccess(cur->getField(1,0),"2");
	checkSuccess(cur->getField(2,0),"3");
	checkSuccess(cur->getField(3,0),"4");
	checkSuccess(cur->getField(4,0),"5");
	checkSuccess(cur->getField(5,0),"6");
	checkSuccess(cur->getField(6,0),"7");
	checkSuccess(cur->getField(7,0),"8");
	cout << endl;
	checkSuccess(cur->sendQuery("select * from testtable order by testsmallint"),1);
	checkSuccess(con->suspendSession(),1);
	port=con->getConnectionPort();
	socket=con->getConnectionSocket();
	checkSuccess(con->resumeSession(port,socket),1);
	cout << endl;
	checkSuccess(cur->getField(0,0),"1");
	checkSuccess(cur->getField(1,0),"2");
	checkSuccess(cur->getField(2,0),"3");
	checkSuccess(cur->getField(3,0),"4");
	checkSuccess(cur->getField(4,0),"5");
	checkSuccess(cur->getField(5,0),"6");
	checkSuccess(cur->getField(6,0),"7");
	checkSuccess(cur->getField(7,0),"8");
	cout << endl;

	cout << "SUSPENDED RESULT SET: " << endl;
	cur->setResultSetBufferSize(2);
	checkSuccess(cur->sendQuery("select * from testtable order by testsmallint"),1);
	checkSuccess(cur->getField(2,0),"3");
	id=cur->getResultSetId();
	cur->suspendResultSet();
	checkSuccess(con->suspendSession(),1);
	port=con->getConnectionPort();
	socket=con->getConnectionSocket();
	checkSuccess(con->resumeSession(port,socket),1);
	checkSuccess(cur->resumeResultSet(id),1);
	cout << endl;
	checkSuccess(cur->firstRowIndex(),4);
	checkSuccess(cur->endOfResultSet(),0);
	checkSuccess(cur->rowCount(),6);
	checkSuccess(cur->getField(7,0),"8");
	cout << endl;
	checkSuccess(cur->firstRowIndex(),6);
	checkSuccess(cur->endOfResultSet(),0);
	checkSuccess(cur->rowCount(),8);
	checkSuccess(cur->getField(8,0),NULL);
	cout << endl;
	checkSuccess(cur->firstRowIndex(),8);
	checkSuccess(cur->endOfResultSet(),1);
	checkSuccess(cur->rowCount(),8);
	cur->setResultSetBufferSize(0);
	cout << endl;

	cout << "CACHED RESULT SET: " << endl;
	cur->cacheToFile("cachefile1");
	cur->setCacheTtl(200);
	checkSuccess(cur->sendQuery("select * from testtable order by testsmallint"),1);
	filename=cur->getCacheFileName();
	checkSuccess(filename,"cachefile1");
	cur->cacheOff();
	checkSuccess(cur->openCachedResultSet(filename),1);
	checkSuccess(cur->getField(7,0),"8");
	cout << endl;

	cout << "COLUMN COUNT FOR CACHED RESULT SET: " << endl;
	checkSuccess(cur->colCount(),11);
	cout << endl;

	cout << "COLUMN NAMES FOR CACHED RESULT SET: " << endl;
	checkSuccess(cur->getColumnName(0),"TESTSMALLINT");
	checkSuccess(cur->getColumnName(1),"TESTINT");
	checkSuccess(cur->getColumnName(2),"TESTBIGINT");
	checkSuccess(cur->getColumnName(3),"TESTDECIMAL");
	checkSuccess(cur->getColumnName(4),"TESTREAL");
	checkSuccess(cur->getColumnName(5),"TESTDOUBLE");
	checkSuccess(cur->getColumnName(6),"TESTCHAR");
	checkSuccess(cur->getColumnName(7),"TESTVARCHAR");
	checkSuccess(cur->getColumnName(8),"TESTDATE");
	checkSuccess(cur->getColumnName(9),"TESTTIME");
	checkSuccess(cur->getColumnName(10),"TESTTIMESTAMP");
	cols=cur->getColumnNames();
	checkSuccess(cols[0],"TESTSMALLINT");
	checkSuccess(cols[1],"TESTINT");
	checkSuccess(cols[2],"TESTBIGINT");
	checkSuccess(cols[3],"TESTDECIMAL");
	checkSuccess(cols[4],"TESTREAL");
	checkSuccess(cols[5],"TESTDOUBLE");
	checkSuccess(cols[6],"TESTCHAR");
	checkSuccess(cols[7],"TESTVARCHAR");
	checkSuccess(cols[8],"TESTDATE");
	checkSuccess(cols[9],"TESTTIME");
	checkSuccess(cols[10],"TESTTIMESTAMP");
	cout << endl;

	cout << "CACHED RESULT SET WITH RESULT SET BUFFER SIZE: " << endl;
	cur->setResultSetBufferSize(2);
	cur->cacheToFile("cachefile1");
	cur->setCacheTtl(200);
	checkSuccess(cur->sendQuery("select * from testtable order by testsmallint"),1);
	filename=cur->getCacheFileName();
	checkSuccess(filename,"cachefile1");
	cur->cacheOff();
	checkSuccess(cur->openCachedResultSet(filename),1);
	checkSuccess(cur->getField(7,0),"8");
	checkSuccess(cur->getField(8,0),NULL);
	cur->setResultSetBufferSize(0);
	cout << endl;

	cout << "FROM ONE CACHE FILE TO ANOTHER: " << endl;
	cur->cacheToFile("cachefile2");
	checkSuccess(cur->openCachedResultSet("cachefile1"),1);
	cur->cacheOff();
	checkSuccess(cur->openCachedResultSet("cachefile2"),1);
	checkSuccess(cur->getField(7,0),"8");
	checkSuccess(cur->getField(8,0),NULL);
	cout << endl;

	cout << "FROM ONE CACHE FILE TO ANOTHER WITH RESULT SET BUFFER SIZE: " << endl;
	cur->setResultSetBufferSize(2);
	cur->cacheToFile("cachefile2");
	checkSuccess(cur->openCachedResultSet("cachefile1"),1);
	cur->cacheOff();
	checkSuccess(cur->openCachedResultSet("cachefile2"),1);
	checkSuccess(cur->getField(7,0),"8");
	checkSuccess(cur->getField(8,0),NULL);
	cur->setResultSetBufferSize(0);
	cout << endl;

	cout << "CACHED RESULT SET WITH SUSPEND AND RESULT SET BUFFER SIZE: " << endl;
	cur->setResultSetBufferSize(2);
	cur->cacheToFile("cachefile1");
	cur->setCacheTtl(200);
	checkSuccess(cur->sendQuery("select * from testtable order by testsmallint"),1);
	checkSuccess(cur->getField(2,0),"3");
	filename=cur->getCacheFileName();
	checkSuccess(filename,"cachefile1");
	id=cur->getResultSetId();
	cur->suspendResultSet();
	checkSuccess(con->suspendSession(),1);
	port=con->getConnectionPort();
	socket=con->getConnectionSocket();
	cout << endl;
	checkSuccess(con->resumeSession(port,socket),1);
	checkSuccess(cur->resumeCachedResultSet(id,filename),1);
	cout << endl;
	checkSuccess(cur->firstRowIndex(),4);
	checkSuccess(cur->endOfResultSet(),0);
	checkSuccess(cur->rowCount(),6);
	checkSuccess(cur->getField(7,0),"8");
	cout << endl;
	checkSuccess(cur->firstRowIndex(),6);
	checkSuccess(cur->endOfResultSet(),0);
	checkSuccess(cur->rowCount(),8);
	checkSuccess(cur->getField(8,0),NULL);
	cout << endl;
	checkSuccess(cur->firstRowIndex(),8);
	checkSuccess(cur->endOfResultSet(),1);
	checkSuccess(cur->rowCount(),8);
	cur->cacheOff();
	cout << endl;
	checkSuccess(cur->openCachedResultSet(filename),1);
	checkSuccess(cur->getField(7,0),"8");
	checkSuccess(cur->getField(8,0),NULL);
	cur->setResultSetBufferSize(0);
	cout << endl;

	// drop existing table
	cur->sendQuery("drop table testtable");
	cout << endl;

	// invalid queries...
	cout << "INVALID QUERIES: " << endl;
	checkSuccess(cur->sendQuery("select * from testtable order by testsmallint"),0);
	checkSuccess(cur->sendQuery("select * from testtable order by testsmallint"),0);
	checkSuccess(cur->sendQuery("select * from testtable order by testsmallint"),0);
	checkSuccess(cur->sendQuery("select * from testtable order by testsmallint"),0);
	cout << endl;
	checkSuccess(cur->sendQuery("insert into testtable values (1,2,3,4)"),0);
	checkSuccess(cur->sendQuery("insert into testtable values (1,2,3,4)"),0);
	checkSuccess(cur->sendQuery("insert into testtable values (1,2,3,4)"),0);
	checkSuccess(cur->sendQuery("insert into testtable values (1,2,3,4)"),0);
	cout << endl;
	checkSuccess(cur->sendQuery("create table testtable"),0);
	checkSuccess(cur->sendQuery("create table testtable"),0);
	checkSuccess(cur->sendQuery("create table testtable"),0);
	checkSuccess(cur->sendQuery("create table testtable"),0);
	cout << endl;
}
