// Copyright (c) 2001  David Muse
// See the file COPYING for more information.

#include <sqlrelay/sqlrclient.h>
#include <string.h>
#include <stdlib.h>

#include <iostream.h>

sqlrconnection	*con;
sqlrcursor	*cur;
sqlrconnection	*secondcon;
sqlrcursor	*secondcur;

void checkSuccess(char *value, char *success) {

	if (!success) {
		if (!value) {
			cout << "success ";
			return;
		} else {
			cout << "failure ";
			delete cur;
			delete con;
			exit(0);
		}
	}

	if (!strcmp(value,success)) {
		cout << "success ";
	} else {
		cout << "failure ";
		delete cur;
		delete con;
		exit(0);
	}
}

void checkSuccess(int value, int success) {

	if (value==success) {
		cout << "success ";
	} else {
		cout << "failure ";
		delete cur;
		delete con;
		exit(0);
	}
}

int	main(int argc, char **argv) {

	char	*dbtype;
	char	*subvars[4]={"var1","var2","var3",NULL};
	char	*subvalstrings[3]={"hi","hello","bye"};
	long	subvallongs[3]={1,2,3};
	double	subvaldoubles[3]={10.55,10.556,10.5556};
	unsigned short	precs[3]={4,5,6};
	unsigned short	scales[3]={2,3,4};
	char	*numvar;
	char	*stringvar;
	char	*floatvar;
	char	**cols;
	char	**fields;
	int	port;
	char	*socket;
	int	id;
	char	*filename;
	long	*fieldlens;

	// usage...
	if (argc<5) {
		cout << "usage: sqlite host port socket user password" << endl;
		exit(0);
	}

	// instantiation
	con=new sqlrconnection(argv[1],atoi(argv[2]), 
					argv[3],argv[4],argv[5],0,1);
	cur=new sqlrcursor(con);

	// get database type
	cout << "IDENTIFY: " << endl;
	checkSuccess(con->identify(),"sqlite");
	cout << endl;

	// ping
	cout << "PING: " << endl;
	checkSuccess(con->ping(),1);
	cout << endl;

	// drop existing table
	cur->sendQuery("begin transaction");
	cur->sendQuery("drop table testtable");
	con->commit();

	// create a new table
	cout << "CREATE TEMPTABLE: " << endl;
	cur->sendQuery("begin transaction");
	checkSuccess(cur->sendQuery("create table testtable (testint int, testfloat float, testchar char(40), testvarchar varchar(40))"),1);
	con->commit();
	cout << endl;

	cout << "INSERT: " << endl;
	cur->sendQuery("begin transaction");
	checkSuccess(cur->sendQuery("insert into testtable values (1,1.1,'testchar1','testvarchar1')"),1);
	checkSuccess(cur->sendQuery("insert into testtable values (2,2.2,'testchar2','testvarchar2')"),1);
	checkSuccess(cur->sendQuery("insert into testtable values (3,3.3,'testchar3','testvarchar3')"),1);
	checkSuccess(cur->sendQuery("insert into testtable values (4,4.4,'testchar4','testvarchar4')"),1);
	cout << endl;

	cout << "AFFECTED ROWS: " << endl;
	checkSuccess(cur->affectedRows(),-1);
	cout << endl;

	cout << "BIND BY NAME: " << endl;
	cur->prepareQuery("insert into testtable values (:var1,:var2,:var3,:var4)");
	cur->inputBind("var1",5);
	cur->inputBind("var2",5.5,4,2);
	cur->inputBind("var3","testchar5");
	cur->inputBind("var4","testvarchar5");
	checkSuccess(cur->executeQuery(),1);
	cur->clearBinds();
	cur->inputBind("var1",6);
	cur->inputBind("var2",6.6,4,2);
	cur->inputBind("var3","testchar6");
	cur->inputBind("var4","testvarchar6");
	checkSuccess(cur->executeQuery(),1);
	cur->clearBinds();
	cur->inputBind("var1",7);
	cur->inputBind("var2",7.7,4,2);
	cur->inputBind("var3","testchar7");
	cur->inputBind("var4","testvarchar7");
	checkSuccess(cur->executeQuery(),1);
	cout << endl;

	cout << "BIND BY NAME WITH VALIDATION: " << endl;
	cur->clearBinds();
	cur->inputBind("var1",8);
	cur->inputBind("var2",8.8,4,2);
	cur->inputBind("var3","testchar8");
	cur->inputBind("var4","testvarchar8");
	cur->validateBinds();
	checkSuccess(cur->executeQuery(),1);
	cout << endl;

	cout << "SELECT: " << endl;
	checkSuccess(cur->sendQuery("select * from testtable order by testint"),1);
	cout << endl;

	cout << "COLUMN COUNT: " << endl;
	checkSuccess(cur->colCount(),4);
	cout << endl;

	cout << "COLUMN NAMES: " << endl;
	checkSuccess(cur->getColumnName(0),"testint");
	checkSuccess(cur->getColumnName(1),"testfloat");
	checkSuccess(cur->getColumnName(2),"testchar");
	checkSuccess(cur->getColumnName(3),"testvarchar");
	cols=cur->getColumnNames();
	checkSuccess(cols[0],"testint");
	checkSuccess(cols[1],"testfloat");
	checkSuccess(cols[2],"testchar");
	checkSuccess(cols[3],"testvarchar");
	cout << endl;

	cout << "COLUMN TYPES: " << endl;
	checkSuccess(cur->getColumnType(0),"UNKNOWN");
	checkSuccess(cur->getColumnType("testint"),"UNKNOWN");
	checkSuccess(cur->getColumnType(1),"UNKNOWN");
	checkSuccess(cur->getColumnType("testfloat"),"UNKNOWN");
	checkSuccess(cur->getColumnType(2),"UNKNOWN");
	checkSuccess(cur->getColumnType("testchar"),"UNKNOWN");
	checkSuccess(cur->getColumnType(3),"UNKNOWN");
	checkSuccess(cur->getColumnType("testvarchar"),"UNKNOWN");
	cout << endl;

	cout << "COLUMN LENGTH: " << endl;
	checkSuccess(cur->getColumnLength(0),0);
	checkSuccess(cur->getColumnLength("testint"),0);
	checkSuccess(cur->getColumnLength(1),0);
	checkSuccess(cur->getColumnLength("testfloat"),0);
	checkSuccess(cur->getColumnLength(2),0);
	checkSuccess(cur->getColumnLength("testchar"),0);
	checkSuccess(cur->getColumnLength(3),0);
	checkSuccess(cur->getColumnLength("testvarchar"),0);
	cout << endl;

	cout << "LONGEST COLUMN: " << endl;
	checkSuccess(cur->getLongest(0),1);
	checkSuccess(cur->getLongest("testint"),1);
	checkSuccess(cur->getLongest(1),3);
	checkSuccess(cur->getLongest("testfloat"),3);
	checkSuccess(cur->getLongest(2),9);
	checkSuccess(cur->getLongest("testchar"),9);
	checkSuccess(cur->getLongest(3),12);
	checkSuccess(cur->getLongest("testvarchar"),12);
	cout << endl;

	cout << "ROW COUNT: " << endl;
	checkSuccess(cur->rowCount(),8);
	cout << endl;

	cout << "TOTAL ROWS: " << endl;
	checkSuccess(cur->totalRows(),8);
	cout << endl;

	cout << "FIRST ROW INDEX: " << endl;
	checkSuccess(cur->firstRowIndex(),0);
	cout << endl;

	cout << "END OF RESULT SET: " << endl;
	checkSuccess(cur->endOfResultSet(),1);
	cout << endl;

	cout << "FIELDS BY INDEX: " << endl;
	checkSuccess(cur->getField(0,0),"1");
	checkSuccess(cur->getField(0,1),"1.1");
	checkSuccess(cur->getField(0,2),"testchar1");
	checkSuccess(cur->getField(0,3),"testvarchar1");
	cout << endl;
	checkSuccess(cur->getField(7,0),"8");
	checkSuccess(cur->getField(7,1),"8.8");
	checkSuccess(cur->getField(7,2),"testchar8");
	checkSuccess(cur->getField(7,3),"testvarchar8");
	cout << endl;

	cout << "FIELD LENGTHS BY INDEX: " << endl;
	checkSuccess(cur->getFieldLength(0,0),1);
	checkSuccess(cur->getFieldLength(0,1),3);
	checkSuccess(cur->getFieldLength(0,2),9);
	checkSuccess(cur->getFieldLength(0,3),12);
	cout << endl;
	checkSuccess(cur->getFieldLength(7,0),1);
	checkSuccess(cur->getFieldLength(7,1),3);
	checkSuccess(cur->getFieldLength(7,2),9);
	checkSuccess(cur->getFieldLength(7,3),12);
	cout << endl;

	cout << "FIELDS BY NAME: " << endl;
	checkSuccess(cur->getField(0,"testint"),"1");
	checkSuccess(cur->getField(0,"testfloat"),"1.1");
	checkSuccess(cur->getField(0,"testchar"),"testchar1");
	checkSuccess(cur->getField(0,"testvarchar"),"testvarchar1");
	cout << endl;
	checkSuccess(cur->getField(7,"testint"),"8");
	checkSuccess(cur->getField(7,"testfloat"),"8.8");
	checkSuccess(cur->getField(7,"testchar"),"testchar8");
	checkSuccess(cur->getField(7,"testvarchar"),"testvarchar8");
	cout << endl;

	cout << "FIELD LENGTHS BY NAME: " << endl;
	checkSuccess(cur->getFieldLength(0,"testint"),1);
	checkSuccess(cur->getFieldLength(0,"testfloat"),3);
	checkSuccess(cur->getFieldLength(0,"testchar"),9);
	checkSuccess(cur->getFieldLength(0,"testvarchar"),12);
	cout << endl;
	checkSuccess(cur->getFieldLength(7,"testint"),1);
	checkSuccess(cur->getFieldLength(7,"testfloat"),3);
	checkSuccess(cur->getFieldLength(7,"testchar"),9);
	checkSuccess(cur->getFieldLength(7,"testvarchar"),12);
	cout << endl;

	cout << "FIELDS BY ARRAY: " << endl;
	fields=cur->getRow(0);
	checkSuccess(fields[0],"1");
	checkSuccess(fields[1],"1.1");
	checkSuccess(fields[2],"testchar1");
	checkSuccess(fields[3],"testvarchar1");
	cout << endl;

	cout << "FIELD LENGTHS BY ARRAY: " << endl;
	fieldlens=cur->getRowLengths(0);
	checkSuccess(fieldlens[0],1);
	checkSuccess(fieldlens[1],3);
	checkSuccess(fieldlens[2],9);
	checkSuccess(fieldlens[3],12);
	cout << endl;

	cout << "INDIVIDUAL SUBSTITUTIONS: " << endl;
	cur->sendQuery("drop table testtable1");
	checkSuccess(cur->sendQuery("create table testtable1 (col1 int, col2 char, col3 float)"),1);
	cur->prepareQuery("insert into testtable1 values ($(var1),'$(var2)',$(var3))");
	cur->substitution("var1",1);
	cur->substitution("var2","hello");
	cur->substitution("var3",10.5556,6,4);
	checkSuccess(cur->executeQuery(),1);
	cout << endl;

	cout << "FIELDS: " << endl;
	checkSuccess(cur->sendQuery("select * from testtable1"),1);
	checkSuccess(cur->getField(0,0),"1");
	checkSuccess(cur->getField(0,1),"hello");
	checkSuccess(cur->getField(0,2),"10.5556");
	checkSuccess(cur->sendQuery("delete from testtable1"),1);
	cout << endl;

	cout << "ARRAY SUBSTITUTIONS: " << endl;
	cur->prepareQuery("insert into testtable1 values ('$(var1)','$(var2)','$(var3)')");
	cur->substitutions(subvars,subvalstrings);
	checkSuccess(cur->executeQuery(),1);
	cout << endl;

	cout << "FIELDS: " << endl;
	checkSuccess(cur->sendQuery("select * from testtable1"),1);
	checkSuccess(cur->getField(0,0),"hi");
	checkSuccess(cur->getField(0,1),"hello");
	checkSuccess(cur->getField(0,2),"bye");
	checkSuccess(cur->sendQuery("delete from testtable1"),1);
	cout << endl;


	cout << "ARRAY SUBSTITUTIONS: " << endl;
	cur->prepareQuery("insert into testtable1 values ($(var1),'$(var2)',$(var3))");
	cur->substitutions(subvars,subvallongs);
	checkSuccess(cur->executeQuery(),1);
	cout << endl;

	cout << "FIELDS: " << endl;
	checkSuccess(cur->sendQuery("select * from testtable1"),1);
	checkSuccess(cur->getField(0,0),"1");
	checkSuccess(cur->getField(0,1),"2");
	checkSuccess(cur->getField(0,2),"3");
	checkSuccess(cur->sendQuery("delete from testtable1"),1);
	cout << endl;


	cout << "ARRAY SUBSTITUTIONS: " << endl;
	cur->prepareQuery("insert into testtable1 values ($(var1),'$(var2)',$(var3))");
	cur->substitutions(subvars,subvaldoubles,precs,scales);
	checkSuccess(cur->executeQuery(),1);
	cout << endl;

	cout << "FIELDS: " << endl;
	checkSuccess(cur->sendQuery("select * from testtable1"),1);
	checkSuccess(cur->getField(0,0),"10.55");
	checkSuccess(cur->getField(0,1),"10.556");
	checkSuccess(cur->getField(0,2),"10.5556");
	checkSuccess(cur->sendQuery("delete from testtable1"),1);
	cout << endl;


	cout << "NULLS as Nulls: " << endl;
	cur->getNullsAsNulls();
	checkSuccess(cur->sendQuery("insert into testtable1 values (1,NULL,NULL)"),1);
	checkSuccess(cur->sendQuery("select * from testtable1"),1);
	checkSuccess(cur->getField(0,0),"1");
	checkSuccess(cur->getField(0,1),NULL);
	checkSuccess(cur->getField(0,2),NULL);
	cur->getNullsAsEmptyStrings();
	checkSuccess(cur->sendQuery("select * from testtable1"),1);
	checkSuccess(cur->getField(0,0),"1");
	checkSuccess(cur->getField(0,1),"");
	checkSuccess(cur->getField(0,2),"");
	cout << endl;

	cout << "RESULT SET BUFFER SIZE: " << endl;
	checkSuccess(cur->getResultSetBufferSize(),0);
	cur->setResultSetBufferSize(2);
	checkSuccess(cur->sendQuery("select * from testtable order by testint"),1);
	checkSuccess(cur->getResultSetBufferSize(),2);
	cout << endl;
	checkSuccess(cur->firstRowIndex(),0);
	checkSuccess(cur->endOfResultSet(),0);
	checkSuccess(cur->rowCount(),2);
	checkSuccess(cur->getField(0,0),"1");
	checkSuccess(cur->getField(1,0),"2");
	checkSuccess(cur->getField(2,0),"3");
	cout << endl;
	checkSuccess(cur->firstRowIndex(),2);
	checkSuccess(cur->endOfResultSet(),0);
	checkSuccess(cur->rowCount(),4);
	checkSuccess(cur->getField(6,0),"7");
	checkSuccess(cur->getField(7,0),"8");
	cout << endl;
	checkSuccess(cur->firstRowIndex(),6);
	checkSuccess(cur->endOfResultSet(),0);
	checkSuccess(cur->rowCount(),8);
	checkSuccess(cur->getField(8,0),NULL);
	cout << endl;
	checkSuccess(cur->firstRowIndex(),8);
	checkSuccess(cur->endOfResultSet(),1);
	checkSuccess(cur->rowCount(),8);
	cout << endl;

	cout << "DONT GET COLUMN INFO: " << endl;
	cur->dontGetColumnInfo();
	checkSuccess(cur->sendQuery("select * from testtable order by testint"),1);
	checkSuccess(cur->getColumnName(0),NULL);
	checkSuccess(cur->getColumnLength(0),0);
	checkSuccess(cur->getColumnType(0),NULL);
	cur->getColumnInfo();
	checkSuccess(cur->sendQuery("select * from testtable order by testint"),1);
	checkSuccess(cur->getColumnName(0),"testint");
	checkSuccess(cur->getColumnLength(0),0);
	checkSuccess(cur->getColumnType(0),"UNKNOWN");
	cout << endl;

	cout << "SUSPENDED SESSION: " << endl;
	checkSuccess(cur->sendQuery("select * from testtable order by testint"),1);
	checkSuccess(con->suspendSession(),1);
	port=con->getConnectionPort();
	socket=con->getConnectionSocket();
	checkSuccess(con->resumeSession(port,socket),1);
	cout << endl;
	checkSuccess(cur->getField(0,0),"1");
	checkSuccess(cur->getField(1,0),"2");
	checkSuccess(cur->getField(2,0),"3");
	checkSuccess(cur->getField(3,0),"4");
	checkSuccess(cur->getField(4,0),"5");
	checkSuccess(cur->getField(5,0),"6");
	checkSuccess(cur->getField(6,0),"7");
	checkSuccess(cur->getField(7,0),"8");
	cout << endl;
	checkSuccess(cur->sendQuery("select * from testtable order by testint"),1);
	checkSuccess(con->suspendSession(),1);
	port=con->getConnectionPort();
	socket=con->getConnectionSocket();
	checkSuccess(con->resumeSession(port,socket),1);
	cout << endl;
	checkSuccess(cur->getField(0,0),"1");
	checkSuccess(cur->getField(1,0),"2");
	checkSuccess(cur->getField(2,0),"3");
	checkSuccess(cur->getField(3,0),"4");
	checkSuccess(cur->getField(4,0),"5");
	checkSuccess(cur->getField(5,0),"6");
	checkSuccess(cur->getField(6,0),"7");
	checkSuccess(cur->getField(7,0),"8");
	cout << endl;
	checkSuccess(cur->sendQuery("select * from testtable order by testint"),1);
	checkSuccess(con->suspendSession(),1);
	port=con->getConnectionPort();
	socket=con->getConnectionSocket();
	checkSuccess(con->resumeSession(port,socket),1);
	cout << endl;
	checkSuccess(cur->getField(0,0),"1");
	checkSuccess(cur->getField(1,0),"2");
	checkSuccess(cur->getField(2,0),"3");
	checkSuccess(cur->getField(3,0),"4");
	checkSuccess(cur->getField(4,0),"5");
	checkSuccess(cur->getField(5,0),"6");
	checkSuccess(cur->getField(6,0),"7");
	checkSuccess(cur->getField(7,0),"8");
	cout << endl;

	cout << "SUSPENDED RESULT SET: " << endl;
	cur->setResultSetBufferSize(2);
	checkSuccess(cur->sendQuery("select * from testtable order by testint"),1);
	checkSuccess(cur->getField(2,0),"3");
	id=cur->getResultSetId();
	cur->suspendResultSet();
	checkSuccess(con->suspendSession(),1);
	port=con->getConnectionPort();
	socket=con->getConnectionSocket();
	checkSuccess(con->resumeSession(port,socket),1);
	checkSuccess(cur->resumeResultSet(id),1);
	cout << endl;
	checkSuccess(cur->firstRowIndex(),4);
	checkSuccess(cur->endOfResultSet(),0);
	checkSuccess(cur->rowCount(),6);
	checkSuccess(cur->getField(7,0),"8");
	cout << endl;
	checkSuccess(cur->firstRowIndex(),6);
	checkSuccess(cur->endOfResultSet(),0);
	checkSuccess(cur->rowCount(),8);
	checkSuccess(cur->getField(8,0),NULL);
	cout << endl;
	checkSuccess(cur->firstRowIndex(),8);
	checkSuccess(cur->endOfResultSet(),1);
	checkSuccess(cur->rowCount(),8);
	cur->setResultSetBufferSize(0);
	cout << endl;

	cout << "CACHED RESULT SET: " << endl;
	cur->cacheToFile("cachefile1");
	cur->setCacheTtl(200);
	checkSuccess(cur->sendQuery("select * from testtable order by testint"),1);
	filename=cur->getCacheFileName();
	checkSuccess(filename,"cachefile1");
	cur->cacheOff();
	checkSuccess(cur->openCachedResultSet(filename),1);
	checkSuccess(cur->getField(7,0),"8");
	cout << endl;

	cout << "COLUMN COUNT FOR CACHED RESULT SET: " << endl;
	checkSuccess(cur->colCount(),4);
	cout << endl;

	cout << "COLUMN NAMES FOR CACHED RESULT SET: " << endl;
	checkSuccess(cur->getColumnName(0),"testint");
	checkSuccess(cur->getColumnName(1),"testfloat");
	checkSuccess(cur->getColumnName(2),"testchar");
	checkSuccess(cur->getColumnName(3),"testvarchar");
	cols=cur->getColumnNames();
	checkSuccess(cols[0],"testint");
	checkSuccess(cols[1],"testfloat");
	checkSuccess(cols[2],"testchar");
	checkSuccess(cols[3],"testvarchar");
	cout << endl;

	cout << "CACHED RESULT SET WITH RESULT SET BUFFER SIZE: " << endl;
	cur->setResultSetBufferSize(2);
	cur->cacheToFile("cachefile1");
	cur->setCacheTtl(200);
	checkSuccess(cur->sendQuery("select * from testtable order by testint"),1);
	filename=cur->getCacheFileName();
	checkSuccess(filename,"cachefile1");
	cur->cacheOff();
	checkSuccess(cur->openCachedResultSet(filename),1);
	checkSuccess(cur->getField(7,0),"8");
	checkSuccess(cur->getField(8,0),NULL);
	cur->setResultSetBufferSize(0);
	cout << endl;

	cout << "FROM ONE CACHE FILE TO ANOTHER: " << endl;
	cur->cacheToFile("cachefile2");
	checkSuccess(cur->openCachedResultSet("cachefile1"),1);
	cur->cacheOff();
	checkSuccess(cur->openCachedResultSet("cachefile2"),1);
	checkSuccess(cur->getField(7,0),"8");
	checkSuccess(cur->getField(8,0),NULL);
	cout << endl;

	cout << "FROM ONE CACHE FILE TO ANOTHER WITH RESULT SET BUFFER SIZE: " << endl;
	cur->setResultSetBufferSize(2);
	cur->cacheToFile("cachefile2");
	checkSuccess(cur->openCachedResultSet("cachefile1"),1);
	cur->cacheOff();
	checkSuccess(cur->openCachedResultSet("cachefile2"),1);
	checkSuccess(cur->getField(7,0),"8");
	checkSuccess(cur->getField(8,0),NULL);
	cur->setResultSetBufferSize(0);
	cout << endl;

	cout << "CACHED RESULT SET WITH SUSPEND AND RESULT SET BUFFER SIZE: " << endl;
	cur->setResultSetBufferSize(2);
	cur->cacheToFile("cachefile1");
	cur->setCacheTtl(200);
	checkSuccess(cur->sendQuery("select * from testtable order by testint"),1);
	checkSuccess(cur->getField(2,0),"3");
	filename=cur->getCacheFileName();
	checkSuccess(filename,"cachefile1");
	id=cur->getResultSetId();
	cur->suspendResultSet();
	checkSuccess(con->suspendSession(),1);
	port=con->getConnectionPort();
	socket=con->getConnectionSocket();
	cout << endl;
	checkSuccess(con->resumeSession(port,socket),1);
	checkSuccess(cur->resumeCachedResultSet(id,filename),1);
	cout << endl;
	checkSuccess(cur->firstRowIndex(),4);
	checkSuccess(cur->endOfResultSet(),0);
	checkSuccess(cur->rowCount(),6);
	checkSuccess(cur->getField(7,0),"8");
	cout << endl;
	checkSuccess(cur->firstRowIndex(),6);
	checkSuccess(cur->endOfResultSet(),0);
	checkSuccess(cur->rowCount(),8);
	checkSuccess(cur->getField(8,0),NULL);
	cout << endl;
	checkSuccess(cur->firstRowIndex(),8);
	checkSuccess(cur->endOfResultSet(),1);
	checkSuccess(cur->rowCount(),8);
	cur->cacheOff();
	cout << endl;
	checkSuccess(cur->openCachedResultSet(filename),1);
	checkSuccess(cur->getField(7,0),"8");
	checkSuccess(cur->getField(8,0),NULL);
	cur->setResultSetBufferSize(0);
	cout << endl;

	cout << "COMMIT AND ROLLBACK: " << endl;
	secondcon=new sqlrconnection(argv[1],
				atoi(argv[2]), 
				argv[3],argv[4],argv[5],0,1);
	secondcur=new sqlrcursor(secondcon);
	checkSuccess(secondcur->sendQuery("select count(*) from testtable"),0);
	checkSuccess(con->commit(),1);
	checkSuccess(secondcur->sendQuery("select count(*) from testtable"),1);
	checkSuccess(secondcur->getField(0,0),"8");
	checkSuccess(cur->sendQuery("insert into testtable values (10,10.1,'testchar10','testvarchar10')"),1);
	checkSuccess(secondcur->sendQuery("select count(*) from testtable"),1);
	checkSuccess(secondcur->getField(0,0),"9");
	cout << endl;

	// drop existing table
	cur->sendQuery("drop table testtable");

	// invalid queries...
	cout << "INVALID QUERIES: " << endl;
	checkSuccess(cur->sendQuery("select * from testtable"),0);
	checkSuccess(cur->sendQuery("select * from testtable"),0);
	checkSuccess(cur->sendQuery("select * from testtable"),0);
	checkSuccess(cur->sendQuery("select * from testtable"),0);
	cout << endl;
	checkSuccess(cur->sendQuery("insert into testtable values (1,2,3,4)"),0);
	checkSuccess(cur->sendQuery("insert into testtable values (1,2,3,4)"),0);
	checkSuccess(cur->sendQuery("insert into testtable values (1,2,3,4)"),0);
	checkSuccess(cur->sendQuery("insert into testtable values (1,2,3,4)"),0);
	cout << endl;
	checkSuccess(cur->sendQuery("create table testtable"),0);
	checkSuccess(cur->sendQuery("create table testtable"),0);
	checkSuccess(cur->sendQuery("create table testtable"),0);
	checkSuccess(cur->sendQuery("create table testtable"),0);
	cout << endl;

}
